
#include "xb_menu.h"
#include "v_inp.h"
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <curses.h>

static int menu_initialized = FALSE;
static int which_interface = INTERF_NORMAL;

void set_interface(interface_type which){
  which_interface = which;
}

interface_type get_interface(){
  return which_interface;
}

#define ANZ_TITLES       9
#define ANZ_ITEMS MENU_ITEM_OUT_OF_RANGE

#define CR              13
#define CtrlW           23

typedef char itemname[40];
typedef char kuerzeltyp[3];

typedef enum {
  no, up, down, left, right, return_, refresh_, char_read
} com_type;   /* Cursor movements and Return */

typedef struct itemtype {
  /* One Item */
  itemname name;   /* Name to be displayed  */
  kuerzeltyp kuerzel;   /* Command for Batchmode */
  long zeile, spalte;   /* Position in Menu      */
} itemtype;

typedef struct titletype {
  /* One title */
  itemname title;
  long start, anzahl, maxlen;
} titletype;


static itemtype menu[ANZ_ITEMS];
static titletype titles[ANZ_TITLES];
static long akt_x, akt_y, istart, max_zeilen;
com_type comm;

#define TITLE_Help      0
#define TITLE_Read      1
#define TITLE_Allplo    2
#define TITLE_Plot      3
#define TITLE_Format    4
#define TITLE_Write     5
#define TITLE_Special   6
#define TITLE_Buffer    7
#define TITLE_Quit      8

void menu_set_up()
{
  /* Create the Menu */
  long i, len;
  itemtype *WITH;
  titletype *WITH1;
  char STR1[256];
  char STR2[62];
  char STR4[256];
  init_v_inp();
  menu_initialized = TRUE;
  akt_x = 1;
  akt_y = 1;
  /* Set names of titles */
  strcpy(titles[TITLE_Help].title,     " Help  ");
  strcpy(titles[TITLE_Read].title,     " Read ");
  strcpy(titles[TITLE_Allplo].title,   " Allplo ");
  strcpy(titles[TITLE_Plot].title,     " Plot   ");
  strcpy(titles[TITLE_Format].title,   " Format ");
  strcpy(titles[TITLE_Write].title,    " Write ");
  strcpy(titles[TITLE_Special].title,  " Special ");
  strcpy(titles[TITLE_Buffer].title,   " Buffer ");
  strcpy(titles[TITLE_Quit].title,     " Quit ");
  /* Set names, lines, and columns(=titles) for items */
  /*--- Help ---*/
  WITH = &menu[MENU_HELP_GENERAL];
  strcpy(WITH->name, " General ");
  strcpy(WITH->kuerzel, "HG");
  WITH->spalte = TITLE_Help;
  WITH->zeile = 3;
  WITH = &menu[MENU_HELP_ITEM];
  strcpy(WITH->name, " on Item ");
  strcpy(WITH->kuerzel, "HI");
  WITH->spalte = TITLE_Help;
  WITH->zeile = 2;
  WITH = &menu[MENU_HELP_SHOW];
  strcpy(WITH->name, " Show Info ");
  strcpy(WITH->kuerzel, "HS");
  WITH->spalte = TITLE_Help;
  WITH->zeile = 1;
  /*--- Input ---*/
  WITH = &menu[MENU_READ_ALLPLO];
  strcpy(WITH->name, " Benson-File (Allplo) ");
  strcpy(WITH->kuerzel, "RB");
  WITH->spalte = TITLE_Read;
  WITH->zeile = 1;
  WITH = &menu[MENU_READ_XPL];
  strcpy(WITH->name, " Benson-File (XPL/YPL) ");
  strcpy(WITH->kuerzel, "RX");
  WITH->spalte = TITLE_Read;
  WITH->zeile = 2;
  WITH = &menu[MENU_READ_HPGL];
  strcpy(WITH->name, " HPGL-File ");
  strcpy(WITH->kuerzel, "RH");
  WITH->spalte = TITLE_Read;
  WITH->zeile = 3;
  WITH = &menu[MENU_QUIT_QUIT];
  strcpy(WITH->name, " Quit Program ");
  strcpy(WITH->kuerzel, "QQ");
  WITH->spalte = TITLE_Quit;
  WITH->zeile = 1;
  WITH = &menu[MENU_QUIT_SPAWN];
  strcpy(WITH->name, " Spawn shell ");
  strcpy(WITH->kuerzel, "QS");
  WITH->spalte = TITLE_Quit;
  WITH->zeile = 2;
  WITH = &menu[MENU_QUIT_ATTACH];
  strcpy(WITH->name, " Attach ");
  strcpy(WITH->kuerzel, "QA");
  WITH->spalte = TITLE_Quit;
  WITH->zeile = 3;
  /*--- List ---*/
  WITH = &menu[MENU_ALLPLO_EDIT];
  strcpy(WITH->name, " Edit ALLPLO.BAT ");
  strcpy(WITH->kuerzel, "AE");
  WITH->spalte = TITLE_Allplo;
  WITH->zeile = 1;
  WITH = &menu[MENU_ALLPLO_RUN];
  strcpy(WITH->name, " Run ALLPLO ");
  strcpy(WITH->kuerzel, "AR");
  WITH->spalte = TITLE_Allplo;
  WITH->zeile = 2;
  WITH = &menu[MENU_ALLPLO_XPL];
  strcpy(WITH->name, " Run XPL ");
  strcpy(WITH->kuerzel, "AX");
  WITH->spalte = TITLE_Allplo;
  WITH->zeile = 3;
  WITH = &menu[MENU_ALLPLO_YPL];
  strcpy(WITH->name, " Run YPL ");
  strcpy(WITH->kuerzel, "AY");
  WITH->spalte = TITLE_Allplo;
  WITH->zeile = 4;
  WITH = &menu[MENU_ALLPLO_TAKE];
  strcpy(WITH->name, " Take existing BAT-File ");
  strcpy(WITH->kuerzel, "AT");
  WITH->spalte = TITLE_Allplo;
  WITH->zeile = 5;
  WITH = &menu[MENU_PLOT_PLOT];
  strcpy(WITH->name, " Plot to Screen ");
  strcpy(WITH->kuerzel, "PP");
  WITH->spalte = TITLE_Plot;
  WITH->zeile = 1;
  WITH = &menu[MENU_FORMAT_PAPER];
  strcpy(WITH->name, " Select Device & Paper ");
  strcpy(WITH->kuerzel, "FP");
  WITH->spalte = TITLE_Format;
  WITH->zeile = 1;
  WITH = &menu[MENU_FORMAT_SCALE];
  strcpy(WITH->name, " Scale/Shift ");
  strcpy(WITH->kuerzel, "FS");
  WITH->spalte = TITLE_Format;
  WITH->zeile = 2;
  WITH = &menu[MENU_FORMAT_SAVE];
  strcpy(WITH->name, " Save Format ");
  strcpy(WITH->kuerzel, "FA");
  WITH->spalte = TITLE_Format;
  WITH->zeile = 3;
  WITH = &menu[MENU_FORMAT_LOAD];
  strcpy(WITH->name, " Load Format ");
  strcpy(WITH->kuerzel, "FL");
  WITH->spalte = TITLE_Format;
  WITH->zeile = 4;
  /*--- Output ---*/
  WITH = &menu[MENU_WRITE_HPGL];
  strcpy(WITH->name, " HPGL-File ");
  strcpy(WITH->kuerzel, "WH");
  WITH->spalte = TITLE_Write;
  WITH->zeile = 1;
  WITH = &menu[MENU_WRITE_PS];
  strcpy(WITH->name, " Postscript-File ");
  strcpy(WITH->kuerzel, "WP");
  WITH->spalte = TITLE_Write;
  WITH->zeile = 2;
  WITH = &menu[MENU_SPECIAL_RENAME];
  strcpy(WITH->name, " Rename allplo.* ");
  strcpy(WITH->kuerzel, "SR");
  WITH->spalte = TITLE_Special;
  WITH->zeile = 1;
  WITH = &menu[MENU_SPECIAL_PURGE];
  strcpy(WITH->name, " -------------- ");
  strcpy(WITH->kuerzel, "SP");
  WITH->spalte = TITLE_Special;
  WITH->zeile = 2;
  WITH = &menu[MENU_SPECIAL_DELETE];
  strcpy(WITH->name, " Delete Allplo.b*;* ");
  /*  strcpy(WITH->name, " Delete allplo.ben/bat"); */
  strcpy(WITH->name, " Delete allplo.b*   ");
  strcpy(WITH->kuerzel, "SD");
  WITH->spalte = TITLE_Special;
  WITH->zeile = 3;
  WITH = &menu[MENU_SPECIAL_CHDIR];
  strcpy(WITH->name, " Change Directory ");
  strcpy(WITH->kuerzel, "SC");
  WITH->spalte = TITLE_Special;
  WITH->zeile = 4;
  WITH = &menu[MENU_BUFFER_ACTUAL];
  strcpy(WITH->name, " Actual Buffer ");
  strcpy(WITH->kuerzel, "BA");
  WITH->spalte = TITLE_Buffer;
  WITH->zeile = 1;
  WITH = &menu[MENU_BUFFER_VISIBLE];
  strcpy(WITH->name, " Visibility ");
  strcpy(WITH->kuerzel, "BV");
  WITH->spalte = TITLE_Buffer;
  WITH->zeile = 2;
  WITH = &menu[MENU_BUFFER_COMBINE];
  strcpy(WITH->name, " Combine ");
  strcpy(WITH->kuerzel, "BC");
  WITH->spalte = TITLE_Buffer;
  WITH->zeile = 3;
  WITH = &menu[MENU_BUFFER_TRANSFORM];
  strcpy(WITH->name, " Transform ");
  strcpy(WITH->kuerzel, "BT");
  WITH->spalte = TITLE_Buffer;
  WITH->zeile = 4;
  /* Calculate starting positions of titles */
  for (i = 0; i < ANZ_TITLES; i++) {
    WITH1 = &titles[i];
    WITH1->anzahl = 0;
    WITH1->maxlen = strlen(WITH1->title);
  }
  titles[0].start = 1;
  for (i = 1; i < ANZ_TITLES; i++)
    titles[i].start = titles[i - 1].maxlen + titles[i - 1].start;
  /* Find numbers of items for each title, the maximal number, */
  /* and the length of the longest item name in each column    */
  max_zeilen = 0;
  for (i = 0; i < ANZ_ITEMS; i++) {
    WITH = &menu[i];
    WITH1 = &titles[WITH->spalte];
    if (strlen(WITH->name) > WITH1->maxlen) WITH1->maxlen = strlen(WITH->name);
    if (WITH->zeile > WITH1->anzahl) WITH1->anzahl = WITH->zeile;
    if (WITH->zeile > max_zeilen) max_zeilen = WITH1->anzahl;
  }
  /* Make all item names in a column equal length */
  for (i = 0; i < ANZ_ITEMS; i++) {
    WITH = &menu[i];
    len = titles[WITH->spalte].maxlen;
    sprintf(STR2, " %%-%1lds  (%%2s) ",len);
    sprintf(STR1,STR2,WITH->name,WITH->kuerzel);
    strcpy(WITH->name, STR1);
  }
  for (i = 1; i < ANZ_TITLES; i++) titles[i].maxlen += 7;
}

com_type menu_command(unsigned char *inchar){
  /* get key, only if it is cursor key or return or Ctrl-W */
  com_type result;
  unsigned char code;

  result = no;
  do {
    code = read_key();
    switch (code) {

    case CR:
      result = return_;
      break;

    case CtrlW:
      result = refresh_;
      break;

    case SMG_K_TRM_UP:
      result = up;
      break;

    case SMG_K_TRM_DOWN:
      result = down;
      break;

    case SMG_K_TRM_RIGHT:
      result = right;
      break;

    case SMG_K_TRM_LEFT:
      result = left;
      break;

    default:
      if (code >= 32 && code <= 126) {
	result = char_read;
	*inchar = toupper(code);
      } else
	result = no;
      break;
    }
  } while (result == no);
  return result;
}

int main_menu()
{
  long akt,i,sel;
  char datum[11], zeit[11];
  unsigned char inchar;
  int titel_char;
  char outstr[81];
  long FORLIM;
  titletype *WITH;
  itemtype *WITH1;
  if (! menu_initialized) menu_set_up();
  titel_char = TRUE;
  if (which_interface==INTERF_COMMAND){
    do{
      vaxtime(zeit); zeit[8] = '\0';
      newline();
      printf("**** Command **** %8s ****",zeit);
      newline();
      getstr(outstr); upstring(outstr); outstr[2] = '\0';
      for (i = 0; i < ANZ_ITEMS; i++)
        if (strcmp(outstr,menu[i].kuerzel)==0){ return i; }
      newline();  
      printf("Error: Unrecognized command: %s",outstr);
      newline();
      newline(); 
    } while (TRUE); 
  }
  do {
    lib_wait(0.2);
    clrscr();
    gotoxy(1, 1);
    vaxdate(datum);
    printf("       X B E N - M A I N - M E N U");
    printf("     %-11s    %-10s",datum,proc_username());newline();
    memset(outstr,'=',79); outstr[79]=0; printf("%s",outstr);newline();newline();
    memset(outstr,'-',79); outstr[79]=0; printf("%s",outstr);newline();
    gotoxy(78, 1);
    printf("??");newline();
    do {
      /* Write the menu */
      vaxtime(zeit);
      gotoxy(70, 1);
      sprintf(outstr, "%.11s", zeit);
      printf("%.5s", outstr);newline();
      for (i = 0; i < ANZ_TITLES; i++) {
	if (! smg_key_pressed()) {
	  WITH = &titles[i];
	  if (i == akt_x)
	    printf("\033[7m");
	  gotoxy((int)WITH->start, 3);
	  puts(WITH->title);
	  printf("\033[0m");newline();
	}
      }
      gotoxy(1, 5);
      if (! smg_key_pressed()) {
	FORLIM = max_zeilen;
	for (i = 1; i <= FORLIM; i++)
	  printf("\033[0K\n");
      }
      for (i = 0; i < ANZ_ITEMS; i++) {
	if (! smg_key_pressed()) {
	  WITH1 = &menu[i];
	  WITH = &titles[WITH1->spalte];
	  if (WITH1->spalte == akt_x) {
	    if (WITH->start + WITH->maxlen > 79)
	      istart = 80 - WITH->maxlen;
	    else
	      istart = WITH->start;
	    if (WITH1->zeile == akt_y)
	      printf("\033[7m");
	    gotoxy((int)istart, (int)(WITH1->zeile + 4));
	    puts(WITH1->name);
	    printf("\033[0m\n");
	  }
	}
      }
      /* Read and interpret the key pressed */
      comm = menu_command(&inchar);
      if (comm != char_read)
	titel_char = TRUE;
      switch (comm) {

      case up:
	if (akt_y > 1)
	  akt_y--;
	else
	  akt_y = titles[akt_x].anzahl;
	break;

      case down:
	if (akt_y < titles[akt_x].anzahl)
	  akt_y++;
	else
	  akt_y = 1;
	break;

      case left:
	if (akt_x > 0)
	  akt_x--;
	else
	  akt_x = ANZ_TITLES-1;
	akt_y = 1;
	break;

      case right:
	if (akt_x < ANZ_TITLES-1)
	  akt_x++;
	else
	  akt_x = 0;
	akt_y = 1;
	break;

      case char_read:
	if (titel_char) {
	  for (i = 0; i < ANZ_ITEMS; i++) {
	    WITH1 = &menu[i];
	    if (inchar == WITH1->kuerzel[0]) {
	      akt_x = WITH1->spalte;
	      akt_y = 1;
	      titel_char = FALSE;
	      gotoxy(78, 1);
	      printf("%c", inchar);newline();
	    }
	  }
	} else {
	  for (i = 0; i < ANZ_ITEMS; i++) {
	    WITH1 = &menu[i];
	    if (inchar == WITH1->kuerzel[1] && akt_x == WITH1->spalte)
	      akt_y = WITH1->zeile;
	  }
	  titel_char = TRUE;
	  gotoxy(78, 1);
	  printf("?");newline();
	}
	break;
      }
    } while ((comm != return_) && (comm != refresh_));
  } while (comm != return_);
  /* Get selected item number */
  for (i = 0; i < ANZ_ITEMS; i++) {
    WITH1 = &menu[i];
    if (WITH1->zeile == akt_y && WITH1->spalte == akt_x) sel = i;
  }
  clrscr();
  lib_wait(0.2);
  return sel;
}

void XBEN_taste_warten(FILE *volterm,int wait){
  if (! wait) return;
  switch (which_interface){
    case INTERF_NORMAL : taste_warten(volterm,TRUE); return;
    default : return;
  }
}

void XBEN_message(FILE *volterm,char mess[]){
  switch (which_interface){
    case INTERF_NORMAL : message(volterm,mess); return;
    default : {fprintf(volterm,"%s\n",mess);}; return;
  }
}
