#include "u_pl_def.h"

/*=========== Init/Exit plot package =================*/

/* initialize, load symbols. Returns FALSE if symbols */
/* were not found, TRUE otherwise                     */
int plot_init(void);

void plot_exita(void);
void plot_exit(int i);

/* Open a plot handle, i.e. Window, Plotterfile,... */
/* Returns 0 on error, otherwise handle             */
/* Name is Windowtitle or filename for Plotter      */
int plot_open(int handle,int devicetyp, int sizex, int sizey, int posx, int posy,
   	      char name[], char *backgroundcolor);

/* return to textmode:                         */
/*  - stdin/stdout available for text output   */
/*  - graphic output prohibited                */
void plot_textmode(void);

/* switch to graphic mode                                  */
/*  - inquire actual size for each open handle (XWindow !) */
/*  - allow graphic output                                 */
void plot_graphmode(void);

/* return last error message */
char *plot_message(void);

/*===================== Handle =====================*/
/* Modify existing handle, returns status */
int plot_modify(int handle, int sizex, int sizey, int posx, int posy,
   	        char name[]);

/* close a handle */
void plot_close(int handle);

/* Clear a handle: Clear Screen or Window, FormFeed on Plotter */
void plot_clear(int handle);

/*===================== Inquire Status ========================*/
/* Status of handle, or general status if handle==0 */
int plot_status(int handle);

/* return devicetype and dimensions */
int plot_getdev(int handle, int *winposx, int *winposy,int *sizex, int *sizey);

/* return capabilities of the device as a longword bitmap.  */
/* To inquire a special capability, pass its value, in this */
/* case, TRUE is return if the capability is present        */
unsigned long plot_capabilities(int handle, unsigned long capability);

/*====================== Modify Parameters ====================*/
/* All routines return the value actually used, which may be   */
/* different from the value desired                            */

/* Set foreground colour byname */
int plot_setfgbyname(int handle, char *foregroundcolor);

/* Set background colour byname */
int plot_setbgbyname(int handle, char *backgroundcolor);

/* Set writing mode (replace/xor/or/erase/...) */
int plot_setmode(int handle, int mode);

/* Set data window */
void plot_setwin(int handle, float xmin, float ymin,
		             float xmax, float ymax);

/* Set data viewport */
void plot_setview(int handle, int xmin, int ymin, int xmax, int ymax);

/* set clipping */
void plot_setclip(int handle, int clip);

/* set symbol size */
float plot_setsymsize(int handle, float size);

/*====================== inquire Parameters ====================*/
/* get foreground colour index */
int plot_getfg(int handle);

/* get background colour index */
int plot_getbg(int handle);

/* get writing mode (replace/xor/or/erase/...) */
int plot_getmode(int handle);

/* get data window */
void plot_getwin(int handle, float *xmin, float *xmax,
		             float *ymin, float *ymax);

/* get data viewport */
void plot_getview(int handle, int *xmin, int *xmax, int *ymin, int *ymax);

/* get clipping */
int plot_getclip(int handle);

/* get symbol size */
float plot_getsymsize(int handle);

/*=========== Output routines, world coordinates only ============*/

void plot_point(int handle, float X, float Y);

void plot_symbol(int handle, int sym, float X, float Y);

void plot_polysymbol(int handle, int pts, int sym, float X[], float Y[]);

void plot_line(int handle, float X0, float Y0, float X1, float Y1);

void plot_polyline(int handle, int pts, float X[], float Y[]);

/*=================== Text output ================*/
/* Set text alignment */
void plot_settextalign(int handle, int xalign, int yalign);

/* write txt at position (X,Y) */
void plot_writetext(int handle, int worldcoor, float X, float Y, char txt[]);

/* erase txt at position (X,Y). Usually done by plot_setmode(ERASE) and */
/* plot_writetext, but in some cases it has to be done in another way...*/
void plot_erasetext(int handle, int worldcoor, float X, float Y, char txt[]);

/*================= Pointer position ==============*/

/* Get pointer position and return terminator. Returns 0 if handle if */
/* output only (Plotter,...), character if a key was pressed, or one  */
/* of the codes defined as PLOT_TRM_... in "u_pl_def.h"               */
int plot_getpointer(int ignore,float *X, float *Y);

/* Check for event, if event present, get pointer position and return */
/* terminator. Returns 0 if handle if output only (Plotter,...) or    */
/* no event, character if a key was pressedm, or one of the codes     */
/* defined as PLOT_TRM_... in "u_pl_def.h"                            */
int plot_checkevent(int handle, float *X, float *Y);

/*======== Higher level routines: relative lines etc. =======*/
void plot_lineabs(int handle, float x1, float y1, float x2, float y2);
void plot_lineto(int handle, float x, float y);
void plot_linerel(int handle, float x, float y);
void plot_moverel(int handle, float x, float y);
void plot_moveto(int handle, float x, float y);
void plot_rectangle(int handle, float x1, float y1, float x2, float y2,
                    int filled);

/*==================== Future work ================*/
/* Set font for text in graphic mode */
/* Define color                      */



