      subroutine cLORGAU(gamnat,e_xsi,srw,w_Lorentz,w_Gauss,nLG,cLG)
      implicit none 
!     version August 2013
!     calculation of the Lorentz-Gauss functions                        
      real,    intent(in) :: gamnat,e_xsi,srw,w_Lorentz,w_Gauss 
      integer, intent(in) :: nLG
      complex, intent(out), dimension(0:nLG):: cLG        
 
      integer  n
      real     w_aLorentz,w_aGauss,wpi,dxc,xc,yc,vxy,dvxy,              &
               halpda1,halpda2,hbeta1,hbeta2,beta1,beta2,beta3,beta4,   &
               alpha1,alpha2,alpha3,alpha4,widthl2,      &
               disp,hp                                           
      real, parameter:: a1=-1.2150,a2=-1.3509,b1= 1.2359,b2= 0.3786,    &
                        c1=-0.3085,c2= 0.5906,d1= 0.0210,d2=-1.1858,    &
                        d12=0.0105,d22=-0.5928,wln2=0.832554611
      complex  hxsi 

      w_aLorentz=abs(w_Lorentz)+1. 
      w_aGauss=abs(w_Gauss) 
!      write(6,'(a,/,7x,5f10.2,i6)')'cLORGAU gamnat,e_xsi,srw,w_Lorentz,w_Gauss,nLG=',&
!                               gamnat,e_xsi,srw,w_Lorentz,w_Gauss,nLG

       if(w_aGauss.gt.0.01)then 
!       Voigt profile according to A.B. McLean, C.E.J. Mitchell and    
!       D.M. Swantston, J.Electron Spectroscopy 69(1994)125-132        
        wpi=1.772453851/w_aLorentz 
        hxsi=cmplx(1.,2*e_xsi)*wpi 
        dxc=2.0*wln2*srw/gamnat/w_aGauss 
        yc=wln2*w_aLorentz/w_aGauss 
        halpda1=c1*(yc-a1)-d1*b1 
        halpda2=c2*(yc-a2)-d2*b2 
        hbeta1=(yc-a1)**2 
        hbeta2=(yc-a2)**2 
        xc=-dxc 
        do  n=0,nLG 
         xc=xc+dxc 
         beta1=hbeta1+(xc-b1)**2 
         beta3=hbeta1+(xc+b1)**2 
         beta2=hbeta2+(xc-b2)**2 
         beta4=hbeta2+(xc+b2)**2 
         alpha1=(halpda1+d1*xc)/beta1 
         alpha3=(halpda1-d1*xc)/beta3 
         alpha2=(halpda2+d2*xc)/beta2 
         alpha4=(halpda2-d2*xc)/beta4 
         vxy=alpha1+alpha2+alpha3+alpha4 
         dvxy=( d12-alpha1*(xc-b1))/beta1+                             &
              (-d12-alpha3*(xc+b1))/beta3+                             &
              ( d22-alpha2*(xc-b2))/beta2+                             &
              (-d22-alpha4*(xc+b2))/beta4                              
          cLG(n)=cmplx(xc*vxy+dvxy,-yc*vxy)*hxsi 
        end do 
       else 
!       Lorentz                                                         
        widthL2=w_aLorentz*gamnat/2. 
        disp=srw/widthL2 
        hp=-disp 
        hxsi=cmplx(1.,2*e_xsi)/w_aLorentz 
        do  n=0,nLG 
         hp=hp+disp 
         cLG(n)=cmplx(hp,-1.)/(1.0+hp*hp)*hxsi 
        end do 
       end if 
                                                                        
!     If a Gauss broadening is used (simulating                         
!     a hyperfine field distribution), the sum of the Gauss function    
!     values (weighting factors) is 1.                         
! Norm of the Lorentz-curve: Int -oo to +oo is gamma*pi/2      
!        hc=(0.,0.) 
!         do n=-nLG+1,nLG-1,2 
!          hc=hc+ cLG(n)*4 
!         end do 
!         do n=-nLG+2,nLG-2,2 
!          hc=hc+ cLG(n)*2 
!         end do 
!         hc=hc+ cLG(nLG)+cLG(-nLG); hc=hc/3. 
!         Print*,'cLG lorentz sum=',2*hc*srw/gamnat/3.14159,nLG
!   lim nLG->00 of the imaginary part  2*hc*srw/gamnat/3.14159 = -1.
!                  the real part is 0
!   nLG=1024 gives -0.9923;   nLG=1536 gives -0.9948         
                                                                        
       return
     END subroutine cLORGAU
                                                                        
                                                                        
     subroutine aLORGAU(Eagd,gamnat,e_xsi,srw,w_Lorentz,w_Gauss,iflag_Voigt,&
                         nLG,cLG)
     implicit none 
!    version August 2013
!    the position Eagd typically differs from cal to call, so that are no
!    parameter stored on array_fr
!    the result is written to cLG(-mLG:mLG)
!    LOR and Gauss arrays are cache memory

      real,    intent(in):: Eagd,gamnat,e_xsi,srw,w_Lorentz,w_Gauss 
      integer, intent(in):: nLG,iflag_Voigt
      complex, intent(out), dimension(-nLG:nLG):: cLG        
                                                                         
      real    w_aLorentz,w_aGauss,wpi,dxc,xc,yc,vxy,dvxy,              &
              halpda1,halpda2,hbeta1,hbeta2,beta1,beta2,beta3,beta4,   &
              alpha1,alpha2,alpha3,alpha4,widthg,widthl2,disp,hp,h0,h1  
      complex hc,hxsi
      integer   i,j,ibg,n,nLor 
      real, parameter:: a1=-1.2150,a2=-1.3509,b1= 1.2359,b2= 0.3786,    &
                        c1=-0.3085,c2= 0.5906,d1= 0.0210,d2=-1.1858,    &
                        d12=0.0105,d22=-0.5928,wln2=0.832554611

      complex, dimension(:), allocatable:: cLOR
      real,    dimension(:), allocatable:: Gauss
   
!      write(6,'(2a)')' aLORGAU:',&
!                     'Eagd,gamnat,srw,w_Lorentz,w_Gauss,iflag_Voigt,nLG'
!      write(6,'(f12.5,f8.4,f10.5,2f8.3,2x,i3,i6)') &                    
!                      Eagd,gamnat,srw,w_Lorentz,w_Gauss,iflag_Voigt,nLG 
                                                                        
!      1/(j*srw-Eagd)+iG/2) =(j*srw-Eagd)/(j*srw-Eagd)**2+G**2/4)    
!                            -iG/2/(j*srw-Eagd)**2+G**2/4)             
      w_aGauss=abs(w_Gauss) 
      w_aLorentz=w_Lorentz+1. 
      if(iflag_Voigt.eq.0) then 
                                                                        
       if(w_aGauss.lt.0.01) then   ! Lorentz 
        widthL2=w_aLorentz*gamnat/2. 
        disp=srw/widthL2 
        hp=-Eagd/widthL2-disp*float(nLG+1) 
        h1=-Eagd/widthL2
        do  n=0,nLG 
         hp=h1+disp*float(n) 
         cLG(n)=cmplx(hp,-1.)/(1.0+hp*hp)/w_aLorentz 
         hp=h1-disp*float(n)
         cLG(-n)=cmplx(hp,-1.)/(1.0+hp*hp)/w_aLorentz
        end do 
!       write(6,'(a)') 'cLOR(n)='                                        
!       write(6,'(i4,2f12.3)') (n,cLG(n)*100.,n=-5,5)                   
       else 
!       Voigt profile according to A.B. McLean, C.E.J. Mitchell and     
!       D.M. Swantston, Queens's University, Kingston, Ontario, Canada  
                                                                        
        wpi=1.772453851/w_aLorentz 
        yc=wln2/w_aGauss*w_aLorentz 
        halpda1=c1*(yc-a1)-d1*b1 
        halpda2=c2*(yc-a2)-d2*b2 
        hbeta1=(yc-a1)**2 
        hbeta2=(yc-a2)**2 
                                                                        
        dxc=2.0*wln2*srw/gamnat/w_aGauss 
        xc=-2.0*wln2*Eagd/gamnat/w_aGauss-dxc*float(nLG+1) 
        do  n=-nLG,nLG 
         xc=xc+dxc 
         beta1=hbeta1+(xc-b1)**2 
         beta3=hbeta1+(xc+b1)**2 
         beta2=hbeta2+(xc-b2)**2 
         beta4=hbeta2+(xc+b2)**2 
         alpha1=(halpda1+d1*xc)/beta1 
         alpha3=(halpda1-d1*xc)/beta3 
         alpha2=(halpda2+d2*xc)/beta2 
         alpha4=(halpda2-d2*xc)/beta4 
         vxy=alpha1+alpha2+alpha3+alpha4 
         dvxy=( d12-alpha1*(xc-b1))/beta1+                              &
              (-d12-alpha3*(xc+b1))/beta3+                              &
              ( d22-alpha2*(xc-b2))/beta2+                              &
              (-d22-alpha4*(xc+b2))/beta4                               
          cLG(n)=cmplx(xc*vxy+dvxy,-yc*vxy)*wpi 
        end do 
       end if 
       if(abs(e_xsi).gt.gamnat/100.) then 
        hxsi=cmplx(1.,2.*e_xsi) 
        do  n=-nLG,nLG 
         cLG(n)=cLG(n)*hxsi 
        end do 
       end if 
      else if(iflag_Voigt.eq.1) then 
                                                                        
       widthG=w_aGauss*gamnat 
       ibg=(6.*widthG/srw+1.)/2.  ! 0.0005 precission   
       ibg=2*ibg 
!      Gauss is always calculated for aLORGAU                           
       if(ibg.gt.nLG/3) ibg=nLG/3
                                                                        
       nLor=nLG+ibg 
       widthL2=w_aLorentz*gamnat/2. 
       disp=srw/widthL2 
       hp=-Eagd/widthL2-disp*float(nLor+1) 
                                                                        
!      convolution numerically                                          
!      Lorentz curve  with stepwidth (srw)                              
!                                                                       
!      -nLor-------------EAGD----------------nLor                     

       allocate(cLOR(-nLor:nLor),Gauss(0:ibg+1))
                                                                        
       do  n=-nLor,nLor 
        hp=hp+disp 
        cLOR(n)=cmplx(hp,-1.)/(1.0+hp*hp)/w_aLorentz 
       end do 
                                                                        
                                                                         
       if(ibg.gt.0) then 
!        -ibg--------0--------ibg           Gauss                       
                                                                        
         disp=2.0*wln2*srw/widthG 
         h0=-disp 
         do i=0,ibg+1 
          h0=h0+disp
          hp=h0**2
          if(hp.gt.36)then
           Gauss(i)=0.0
          else   
           Gauss(i)=exp(-hp)
          end if
         end do 
          h1=0. 
         do j=1,ibg-1,2 
          h1=h1+ Gauss(j)*2. 
         end do 
         do j=2,ibg-2,2 
          h1=h1+ Gauss(j) 
         end do 
         h1=(h1*2.+Gauss(0)+Gauss(ibg)) 
         h1=2.*h1/3. 
         do i=0,ibg+1 
          Gauss(i)=Gauss(i)/h1 
         end do 
                                                                        
                                                                        
!       convolution with exp(-x**2/(sGgL*gamnat)**2)                    
!                                                                       
!       0------------------------------------nLG---nLor                 
!                                     ibg----0----ibg                   
!       0------------------------------------nLG                        
!       this convolution is calculated with stepwidth srw              
!       write(6,'(a,i4)')  ' ####aLORGAU igb=',ibg                     
         do i=-nLG,nLG 
          hc=(0.,0.) 
          do j=-ibg+1,-1,2 
            hc=hc+Gauss(-j)*cLOR(i+j)*2. 
          end do 
          do j=1,ibg-1,2 
           hc=hc+Gauss(j)*cLOR(i+j)*2. 
          end do 
          do j=-ibg+2,-2,2 
           hc=hc+Gauss(-j)*cLOR(i+j) 
          end do 
          do j=0,ibg-2,2 
           hc=hc+Gauss(j)*cLOR(i+j) 
          end do 
          hc=hc*2.+Gauss(ibg)*(cLOR(i-ibg)+cLOR(i+ibg)) 
          cLG(i)=hc/3. 
         end do 
       else
        do i=-nLG,nLG 
         cLG(i)=cLOR(i)
        end do 
       end if           ! if ibg >0  
       if(abs(e_xsi).gt.gamnat/100.) then 
        hxsi=cmplx(1.,2.*e_xsi) 
        do  n=-nLG,nLG 
         cLG(n)=cLG(n)*hxsi 
        end do 
       end if
       deallocate(cLOR,Gauss) 
      end if  ! convolution numerically    

! Norm of the Lorentz-curve: Int -oo to +ooof the imaginary part is gamma*pi/2
      
!        hc=(0.,0.) 
!         do n=-nLG+1,nLG-1,2 
!          hc=hc+ cLG(n)*4 
!         end do 
!         do n=-nLG+2,nLG-2,2 
!          hc=hc+ cLG(n)*2 
!         end do 
!         hc=hc+ cLG(nLG)+cLG(-nLG); hc=hc/3. 
!         Print*,'aLG lorentz sum=',2*hc*srw/gamnat/3.14159,nLG
!   lim nLG->00 of the imaginary part  2*hc*srw/gamnat/3.14159 = -1.
!                  the real part Int dE (E-Ev)*((E-Ev)**2+gamnat**2/4)
!                  depends on E for finite limits of Int
         
!   nLG=1024 gives -0.9923;   nLG=1536 gives -0.9948         
    return 
    END subroutine aLORGAU

      
   subroutine tensor(maxp,mp,p,nwr,nrd,ncl,info)   ! interactive
   implicit none
   integer, intent(in):: maxp,mp,nwr,nrd,ncl
   real, dimension(0:maxp), intent(inout):: p
   integer, intent(out):: info
   
   character(len=132):: helpentry,ch132,cp132,querytype
   character(len=80) :: topic
   character(len=7)  :: form
   character(len=2),dimension(3):: ebi
   character(len=1)  :: ch1
   character(len=80),dimension(2)::inquiries,help_entries
   real,dimension(32) :: out
   real,dimension(3,3):: uveca,uvecb,uvecn,uveco
   real,dimension(3)  :: uvec,uvecv,phipo,thepo,uvec1,uvec2,uvec3
   complex,dimension(-1:1,-1:1):: d21

   integer   k,j,i,i3,ia,ibc,ibx,iby,ibz,iefg,lp,lq,n,nn   
    real     alpha,beta,c2,cm,cp,efgo,eta,gamma,h,etan,      &
             phiII,pi,rad,s2,sm,st,value,sp,arcos
   logical   b_term,b_efg,b_msd 


      b_efg=.false. 
      b_msd=.false. 
      ia=0 
      b_term=.false. 
      do k=1,3 
       do j=1,3 
        uvecb(k,j)=0.0 
       end do 
      end do 
      ebi(1)='ex' 
      ebi(2)='ey' 
      ebi(3)='ez' 
                                                                        
      pi=4.0*atan(1.0) 
      rad=pi/180.0 

      n=2
      help_entries(1)='<tensor>/EFG' 
      help_entries(2)='<tensor>/MSD'
      inquiries(1)='electric field gradient tensor EFG' 
      inquiries(2)='traceless part of mean square displacement tensor MSD' 
      topic='put eta to limits 0 < eta <1'                                                                   
      call decide(topic,n,inquiries,help_entries,lq)
      if(lq.eq.0) return 
      if(lq.eq.1)then 
        b_efg=.true. 
        topic='EFG,eta,phi,theta,psi from parameters (pi,...,pi+4)' 
      end if 
      if(lq.eq.2)then 
       b_msd=.true. 
        topic='x**2,eta,phi,theta,psi from parameters (pi,...,pi+4)' 
      end if 
                                                                        
      n=2 
      help_entries(1)='<tensor>/euler/parameter' 
      help_entries(2)='<tensor>/euler/terminal' 
      inquiries(1)=topic 
      inquiries(2)='eta,phi,theta,psi from terminal' 
      topic='input' 
      call decide(topic,n,inquiries,help_entries,lq)
      if(lq.eq.0) return 
      if(lq.eq.1) goto 10 
                                                                        
      b_term=.true. 
      querytype=' asymmetry parameter eta=' 
      form='f9.4' 
      helpentry='<tensor>/etasign' 
      value=0.0 
      call askvalue(querytype,form,value,helpentry,ncl,nrd,nwr) 
      eta=value 
                                                                        
    5  n=2 
       help_entries(1)='<tensor>/angles' 
       help_entries(2)='<tensor>/unitvectors' 
       if(b_efg)then 
        inquiries(1)='Euler angles of the EFG-tensor' 
        inquiries(2)='unit vectors of the PAS of the EFG' 
       else 
        inquiries(1)='Euler angles of the MSD-tensor' 
        inquiries(2)='unit vectors of the PAS of the MSD' 
       end if 
       topic='Euler angles/unitvectors' 
       call decide(topic,n,inquiries,help_entries,lq)
       if(lq.eq.0) return 
       if(lq.eq.2) goto 20 
                                                                        
!      Euler angles                                                     
       querytype=' Three Euler angles (alpha,beta,gamma):' 
       helpentry='<tensor>/euler_angles' 
       ch132='0,0,0' 
       call askstring(querytype,ch132,helpentry,ncl,nwr) 
       ch1=ch132(1:1) 
       if(ch1.eq.'*'.or.ch1.eq.'q') return 
       call trlch(80,ch132,nn,out) 
       if(nn.ne.3) goto 5 
       goto 15 
                                                                        
   10  continue 
       if(b_efg)querytype=' parameter number of EFG pi' 
       if(b_msd)querytype=' parameter number of x**2 pi' 
       form='i3' 
       helpentry='<tensor>/eta/parnumber' 
       value=0.0 
       call askvalue(querytype,form,value,helpentry,ncl,nrd,nwr) 
       ia=value 
       if(ia.eq.0.or.ia.gt.mp-3) then 
        write(nwr,'(a)')'parameter number of eta out of range' 
        goto 5 
       end if 
       efgo=p(ia) 
       eta=p(ia+1) 
       out(1)=p(ia+2) 
       out(2)=p(ia+3) 
       out(3)=p(ia+4) 
       phiII=2.0*efgo/sqrt(1+eta**2/3) 
       if(b_efg)write(nwr,'(2(a,f9.4),a,3(f8.2,a1),/)')                 &
     &     ' EFG=',efgo,' eta=',eta,                                    &
     &     '     Euler angles =(',out(1),',',out(2),',',out(3),')'      
       if(b_msd)write(nwr,'(2(a,f9.4),a,3(f8.2,a1),/)')                 &
     &     ' x**2=',efgo,' eta=',eta,                                   &
     &     '     Euler angles =(',out(1),',',out(2),',',out(3),')'      
       querytype='parameter are correct ' 
       ch1='y' 
       helpentry='<tensor>/vector/transfer' 
       call yesno(querytype,ch1,helpentry,ncl,nrd,nwr) 
      if(ch1.eq.'*'.or.ch1.eq.'q') return 
      if(ch1.eq.'n') goto 10 
                                                                        
                                                                        
   15  continue 
       alpha=out(1)*rad 
       beta=out(2)*rad 
       gamma=out(3)*rad 
       c2=cos(beta/2)**2 
       s2=1.0-c2 
       cp=cos(alpha+gamma) 
       sp=sin(alpha+gamma) 
       cm=cos(alpha-gamma) 
       sm=sin(alpha-gamma) 
                                                                        
       uvecb(1,1)= c2*cp-s2*cm 
       uvecb(1,2)= c2*sp-s2*sm 
       uvecb(1,3)=-sin(beta)*cos(gamma) 
                                                                        
       uvecb(2,1)=-c2*sp-s2*sm 
       uvecb(2,2)= c2*cp+s2*cm 
       uvecb(2,3)= sin(beta)*sin(gamma) 
                                                                        
       uvecb(3,1)= sin(beta)*cos(alpha) 
       uvecb(3,2)= sin(beta)*sin(alpha) 
       uvecb(3,3)= cos(beta) 
!----------------------------------------------------------------       
       call dimj(alpha,beta,gamma,1,d21)

       uvecb(1,1)=0.5*real(d21(1,1)-d21(-1,1)-d21(1,-1)+d21(-1,-1)) 
       uvecb(1,2)=-0.5*aimag(d21(1,1)+d21(-1,1)-d21(1,-1)-d21(-1,-1)) 
       uvecb(1,3)=-1./sqrt(2.)*real(d21(0,1)-d21(0,-1)) 
                                                                        
       uvecb(2,1)=0.5*aimag(d21(1,1)-d21(-1,1)+d21(1,-1)-d21(-1,-1)) 
       uvecb(2,2)=0.5*real(d21(1,1)+d21(-1,1)+d21(1,-1)+d21(-1,-1)) 
       uvecb(2,3)=-1./sqrt(2.)*aimag(d21(0,1)+d21(0,-1)) 
                                                                        
       uvecb(3,1)=-1./sqrt(2.)*real(d21(1,0)-d21(-1,0)) 
       uvecb(3,2)=1./sqrt(2.)*aimag(d21(1,0)+d21(-1,0)) 
       uvecb(3,3)=real(d21(0,0)) 
                                                                        
!----------------------------                                           
       do k=1,3 
        thepo(k)=arcos(uvecb(k,3))/rad 
        h=1.0-uvecb(k,3)**2 
        if(h.le.0.0) then 
         phipo(k)=0.0 
        else 
         h=sqrt(h) 
         phipo(k)=arcos(uvecb(k,1)/h) 
         if(uvecb(k,2).lt.0) phipo(k)=2.0*pi-phipo(k) 
        end if 
        phipo(k)=phipo(k)/rad 
       end do 
                                                                        
       if(b_efg)write(nwr,'(/,a)')                                      &
     &    '     unit vectors  and polar angles of system SE in S'       
       if(b_msd)write(nwr,'(/,a)')                                      &
     &    '     unit vectors  and polar angles of system S_MSD in S'    
      do k=1,3 
       write(nwr,'(6x,a2,a,3(f8.3,a),2(f8.2,a1))')ebi(k),'=(',          &
     & uvecb(k,1),',',uvecb(k,2),',',uvecb(k,3),')    ('                &
     &                                    ,thepo(k),',',phipo(k),')'    
      end do 
                                                                        
                                                                        
       goto 100 
                                                                        
!      unit vectors                                                     
   20  n=2 
       help_entries(1)='<tensor>/unitvector/input' 
       help_entries(2)='<tensor>/unitvector/input' 
       inquiries(1)='unit vectors:(*,y,z),(x,*,z) or (x,y,*)' 
       inquiries(2)='unit vectors: beta,phi' 
       topic='unit vectors representation' 
      call decide(topic,n,inquiries,help_entries,lp)
       if(lp.eq.0) return 
                                                                        
       if(lp.eq.1) then 
         querytype='one pair of unit vectors:' 
         helpentry='<tensor>/unitvectors/format13' 
         ch132='ex=(1,0,*) ez=(*,0,1)' 
         call askstring(querytype,ch132,helpentry,ncl,nwr) 
         ch1=ch132(1:1) 
         if(ch1.eq.'*'.or.ch1.eq.'q') return 
       else if(lp.eq.2) then 
         querytype='one pair of unit vectors:' 
         helpentry='<tensor>/unitvectors/format23' 
         ch132='ex=(90,0) ez=(0,0)' 
         call askstring(querytype,ch132,helpentry,ncl,nwr) 
         ch1=ch132(1:1) 
         if(ch1.eq.'*'.or.ch1.eq.'q') return 
       end if 
                                                                        
!      evaluation of ch132                                              
       if(lp.eq.1) then 
!       ch132='eax=(1,0,*) eaz=(*,0,1)'                                 
        ibx=index(ch132,'ex') 
        iby=index(ch132,'ey') 
        ibz=index(ch132,'ez') 
                                                                        
        i3=0 
        if(ibx.gt.0) then 
         ibc=index(ch132(ibx:),')') 
         cp132=ch132(ibx+4:ibx+ibc-2) 
!         write(6,'(a,a)')'vector ex=',cp132(:60)                       
         call uvector(cp132,uvec,info) 
         do k=1,3 
          uvecb(1,k)=uvec(k) 
         end do 
         i3=i3+1 
               ! ibx                                                    
        end if 
                                                                        
        if(iby.gt.0) then 
         ibc=index(ch132(iby:),')') 
         cp132=ch132(iby+4:iby+ibc-2) 
!         write(6,'(a,a)')'vector ey=',cp132(:60)                       
         call uvector(cp132,uvec,info) 
         do k=1,3 
          uvecb(2,k)=uvec(k) 
         end do 
         i3=i3+1 
               ! iby                                                    
        end if 
                                                                        
        if(ibz.gt.0) then 
         ibc=index(ch132(ibz:),')') 
         cp132=ch132(ibz+4:ibz+ibc-2) 
!         write(6,'(a,a)')'vector ez=',cp132(:60)                       
         call uvector(cp132,uvec,info) 
         do k=1,3 
          uvecb(3,k)=uvec(k) 
         end do 
         i3=i3+1 
               ! ibz                                                    
        end if 
                                                                        
        if(i3.lt.3.and.i3.gt.0) then 
         if(i3.eq.1) then 
          write(nwr,'(a)') 'at least two unit vectors eb neeeded' 
          goto 20 
         end if 
         if(ibx.gt.0.and.iby.gt.0) then 
          do k=1,3 
           uvec1(k)=uvecb(1,k) 
           uvec2(k)=uvecb(2,k) 
          end do 
!         write(6,'(a,3f15.2)')'uvec1=',(uvec1(j),j=1,3)                
!         write(6,'(a,3f15.2)')'uvec2=',(uvec2(j),j=1,3)                
          call crospr(uvec1,uvec2,uvec3) 
!         write(6,'(a,3f15.2)')'uvec3=',(uvec3(j),j=1,3)                
          call crospr(uvec3,uvec1,uvec2) 
                                                                        
          do k=1,3 
           uvecb(3,k)=uvec3(k) 
           uvecb(2,k)=uvec2(k) 
          end do 
         end if 
         if(ibx.gt.0.and.ibz.gt.0) then 
          do k=1,3 
           uvec1(k)=uvecb(3,k) 
           uvec2(k)=uvecb(1,k) 
          end do 
          call crospr(uvec1,uvec2,uvec3) 
          call crospr(uvec3,uvec1,uvec2) 
          do k=1,3 
           uvecb(2,k)=uvec3(k) 
           uvecb(1,k)=uvec2(k) 
          end do 
         end if 
                                                                        
         if(iby.gt.0.and.ibz.gt.0) then 
          do k=1,3 
           uvec1(k)=uvecb(2,k) 
           uvec2(k)=uvecb(3,k) 
          end do 
          call crospr(uvec1,uvec2,uvec3) 
          call crospr(uvec3,uvec1,uvec2) 
          do k=1,3 
           uvecb(1,k)=uvec3(k) 
           uvecb(3,k)=uvec2(k) 
          end do 
         end if 
        end if 
                                                                        
!        do k=1,3                                                       
!         write(6,'(3f15.2)')(uvecb(k,j),j=1,3)                         
!        end do                                                         
                                                                        
       else if(lp.eq.2) then 
!        ch132='ex=(90,0) ez=(0,0)'                                     
!        write(6,'(a,a)')'angle ',ch132(:60)                            
         ibx=index(ch132,'ex') 
         iby=index(ch132,'ey') 
         ibz=index(ch132,'ez') 
                                                                        
        i3=0 
        if(ibx.gt.0) then 
         ibc=index(ch132(ibx:),')') 
         cp132=ch132(ibx+4:ibx+ibc-2) 
!         write(6,'(a,a)')'angle ex=',cp132(:60)                        
         call trlch(ibc,cp132,nn,out) 
         st=sin(out(1)*rad) 
         uvecb(1,1)=st*cos(out(2)*rad) 
         uvecb(1,2)=st*sin(out(2)*rad) 
         uvecb(1,3)=cos(out(1)*rad) 
         i3=i3+1 
               ! ibx                                                    
        end if 
                                                                        
        if(iby.gt.0) then 
         ibc=index(ch132(iby:),')') 
         cp132=ch132(iby+4:iby+ibc-2) 
!         write(6,'(a,a)')'angle ey=',cp132(:60)                        
         call trlch(ibc,cp132,nn,out) 
         st=sin(out(1)*rad) 
         uvecb(2,1)=st*cos(out(2)*rad) 
         uvecb(2,2)=st*sin(out(2)*rad) 
         uvecb(2,3)=cos(out(1)*rad) 
         i3=i3+1 
               ! iby                                                    
        end if 
                                                                        
        if(ibz.gt.0) then 
         ibc=index(ch132(ibz:),')') 
         cp132=ch132(ibz+4:ibz+ibc-2) 
!         write(6,'(a,a)')'angle ez=',cp132(:60)                        
         call trlch(ibc,cp132,nn,out) 
         st=sin(out(1)*rad) 
         uvecb(3,1)=st*cos(out(2)*rad) 
         uvecb(3,2)=st*sin(out(2)*rad) 
         uvecb(3,3)=cos(out(1)*rad) 
         i3=i3+1 
               ! ibz                                                    
        end if 
                                                                        
!        write(6,'(a)')'uvecb(k,j),j=1,3)'                              
!        do k=1,3                                                       
!         write(6,'(3f15.2)')(uvecb(k,j),j=1,3)                         
!        end do                                                         
                                                                        
        if(i3.lt.3.and.i3.gt.0) then 
         if(i3.eq.1) then 
          write(nwr,'(a)') 'at least two unit vectors eb neeeded' 
          goto 20 
         end if 
         if(ibx.gt.0.and.iby.gt.0) then 
                                                                        
          do k=1,3 
           uvec1(k)=uvecb(1,k) 
           uvec2(k)=uvecb(2,k) 
          end do 
                                                                        
          call crospr(uvec1,uvec2,uvec3) 
          call crospr(uvec3,uvec1,uvec2) 
                                                                        
          do k=1,3 
           uvecb(3,k)=uvec3(k) 
           uvecb(2,k)=uvec2(k) 
          end do 
         end if 
         if(ibx.gt.0.and.ibz.gt.0) then 
          do k=1,3 
           uvec1(k)=uvecb(3,k) 
           uvec2(k)=uvecb(1,k) 
          end do 
          call crospr(uvec1,uvec2,uvec3) 
          call crospr(uvec3,uvec1,uvec2) 
          do k=1,3 
           uvecb(2,k)=uvec3(k) 
           uvecb(1,k)=uvec2(k) 
          end do 
         end if 
         if(iby.gt.0.and.ibz.gt.0) then 
          do k=1,3 
           uvec1(k)=uvecb(2,k) 
           uvec2(k)=uvecb(3,k) 
          end do 
          call crospr(uvec1,uvec2,uvec3) 
          call crospr(uvec3,uvec1,uvec2) 
          do k=1,3 
           uvecb(1,k)=uvec3(k) 
           uvecb(3,k)=uvec2(k) 
          end do 
         end if 
        end if 
                                                                        
                  ! lp                                                  
       end if 
!      write(6,'(a)')'uvecb(k,j),j=1,3)'                                
!      do k=1,3                                                         
!       write(6,'(3f15.2)')(uvecb(k,j),j=1,3)                           
!      end do                                                           
                                                                        
  100  continue 
       do k=1,3 
        do i=1,3 
         uveco(i,k)=uvecb(i,k) 
        end do 
       end do 
                                                                        
!      cases of eta of the EFG                                          
       iefg=1 
       if(eta.lt.0.0) then 
                                                                        
        if(eta.ge.-1) then 
         do k=1,3 
          uvecb(1,k)=uveco(2,k) 
         end do 
         do k=1,3 
          uvecb(2,k)=uveco(1,k) 
         end do 
         do k=1,3 
          uvecb(3,k)=-uveco(3,k) 
         end do 
         etan=-eta 
        else if(eta.lt.-1.0.and.eta.gt.-3.) then 
         do k=1,3 
          uvecb(1,k)=uveco(2,k) 
         end do 
         do k=1,3 
          uvecb(2,k)=uveco(3,k) 
         end do 
         do k=1,3 
          uvecb(3,k)=uveco(1,k) 
         end do 
         etan=(3+eta)/(1-eta) 
         iefg=-1 
        else if(eta.lt.-3.) then 
         do k=1,3 
          uvecb(1,k)=uveco(3,k) 
         end do 
         do k=1,3 
          uvecb(2,k)=uveco(2,k) 
         end do 
         do k=1,3 
          uvecb(3,k)=-uveco(1,k) 
         end do 
         etan=-(3+eta)/(1-eta) 
         iefg=-1 
        end if 
       else if(eta.gt.1) then 
        if(eta.gt.3.0)then 
         do k=1,3 
          uvecb(1,k)=uveco(3,k) 
         end do 
         do k=1,3 
          uvecb(3,k)=uveco(2,k) 
         end do 
         do k=1,3 
          uvecb(2,k)=uveco(1,k) 
         end do 
         etan=(-3+eta)/(1+eta) 
         iefg=-1 
                                                                        
        else if(eta.lt.3.0) then 
         do k=1,3 
          uvecb(1,k)=uveco(1,k) 
         end do 
         do k=1,3 
          uvecb(2,k)=-uveco(3,k) 
         end do 
         do k=1,3 
          uvecb(3,k)=uveco(2,k) 
         end do 
         etan=(3-eta)/(1+eta) 
         iefg=-1 
                                                                        
        end if 
       else 
        etan=eta 
       end if 
                                                                        
      do k=1,3 
       do j=1,3 
        uveca(k,j)=0.0 
       end do 
      end do 
      uveca(1,1)=1.0 
      uveca(2,2)=1.0 
      uveca(3,3)=1.0 
                                                                        
!      vectors uveca and uvecb are known                                
!      calculation of Euler angle alpha,beta,gamma                      
       h=0.0 
       do k=1,3 
        h=h+uveca(3,k)*uvecb(3,k) 
       end do 
       beta=arcos(h) 
       st=sin(beta) 
       if(beta.lt.0.001) then 
        beta=0.0 
        h=0.0 
        do k=1,3 
         h=h+uveca(1,k)*uvecb(1,k) 
        end do 
        alpha=arcos(h) 
        h=0.0 
        do k=1,3 
         h=h+uveca(1,k)*uvecb(2,k) 
        end do 
        if(h.gt.0.0) alpha=2.0*pi-alpha 
        gamma=0.0 
             ! beta=0                                                   
       else 
                                   ! sin(beta)=0                        
        if(abs(st).lt.0.001) then 
         h=0.0 
         do k=1,3 
          h=h+uveca(1,k)*uvecb(1,k) 
         end do 
         gamma=0.0 
         alpha=arcos(-h) 
         h=0.0 
         do k=1,3 
          h=h+uveca(1,k)*uvecb(2,k) 
         end do 
         if(h.gt.0.0) alpha=2.0*pi-alpha 
        else 
         h=0.0 
         do k=1,3 
          h=h+uveca(1,k)*uvecb(3,k) 
         end do 
         alpha=arcos(h/st) 
         h=0.0 
         do k=1,3 
          h=h+uveca(2,k)*uvecb(3,k) 
         end do 
         if(h.lt.0.0) alpha=2.0*pi-alpha 
                                                                        
         h=0.0 
         do k=1,3 
          h=h+uveca(3,k)*uvecb(1,k) 
         end do 
         gamma=arcos(-h/st) 
         h=0.0 
         do k=1,3 
          h=h+uveca(3,k)*uvecb(2,k) 
         end do 
        if(h.lt.0.0) gamma=2.0*pi-gamma 
               ! sin(beta)#0                                            
       end if 
               ! beta=0                                                 
       end if 
                                                                        
       c2=cos(beta/2)**2 
       s2=1.0-c2 
       cp=cos(alpha+gamma) 
       sp=sin(alpha+gamma) 
       cm=cos(alpha-gamma) 
       sm=sin(alpha-gamma) 
                                                                        
       uvecn(1,1)= c2*cp-s2*cm 
       uvecn(1,2)= c2*sp-s2*sm 
       uvecn(1,3)=-sin(beta)*cos(gamma) 
                                                                        
       uvecn(2,1)=-c2*sp-s2*sm 
       uvecn(2,2)= c2*cp+s2*cm 
       uvecn(2,3)= sin(beta)*sin(gamma) 
                                                                        
       uvecn(3,1)= sin(beta)*cos(alpha) 
       uvecn(3,2)= sin(beta)*sin(alpha) 
       uvecn(3,3)= cos(beta) 
                                                                        
       beta=beta/rad 
       alpha=alpha/rad 
       gamma=gamma/rad 
       if(b_term) then 
        if(b_efg)then 
         if(iefg.eq.-1) then 
          write(nwr,'(//,a,f9.4,a,3(f8.2,a1))')'EFG -> -EFG  eta=',     &
     &        etan,'     Euler angles =(',alpha,',',beta,',',gamma,')'  
         else 
          write(nwr,'(//,a,f9.4,a,3(f8.2,a1))')'             eta=',     &
     &        etan,'     Euler angles =(',alpha,',',beta,',',gamma,')'  
         end if 
        end if 
       end if 
       if(b_msd)write(nwr,'(//,2(a,f9.4),a,3(f8.2,a1))')' x**2=',       &
     &   iefg*efgo/sqrt(1.+etan**2/3.)*sqrt(1.+eta**2/3.),              &
     & ' eta=',etan,'     Euler angles =(',alpha,',',beta,',',gamma,')' 
                                                                        
       do k=1,3 
        thepo(k)=arcos(uvecn(k,3))/rad 
        h=1.0-uvecn(k,3)**2 
        if(h.le.0.0) then 
         phipo(k)=0.0 
        else 
         h=sqrt(h) 
         phipo(k)=arcos(uvecn(k,1)/h) 
         if(uvecn(k,2).lt.0) phipo(k)=2.0*pi-phipo(k) 
        end if 
        phipo(k)=phipo(k)/rad 
       end do 
       if(b_efg) write(nwr,'(/,a)')                                     &
     &    ' new unit vectors  and polar angles of system SE in S'       
       if(b_msd) write(nwr,'(/,a)')                                     &
     &    ' new unit vectors  and polar angles of system S_MSD in S'    
      do k=1,3 
       write(nwr,'(6x,a2,a,3(f8.3,a),2(f8.2,a1))')ebi(k),'=(',          &
     & uvecn(k,1),',',uvecn(k,2),',',uvecn(k,3),')    ('                &
     &                                    ,thepo(k),',',phipo(k),')'    
      end do 
                                                                        
      if(ia.gt.0) then 
       p(ia)=float(iefg)*efgo 
       if(b_msd)p(ia)=p(ia)/sqrt(1.+etan**2/3.)*sqrt(1.+eta**2/3.) 
       p(ia+1)=etan 
       p(ia+2)=alpha 
       p(ia+3)=beta 
       p(ia+4)=gamma 
      end if 
                                                                        
      write(nwr,'(/)') 
      querytype='transformation of a vector to the new system ' 
      ch1='n' 
      helpentry='<tensor>/vector/transform' 
      call yesno(querytype,ch1,helpentry,ncl,nrd,nwr) 
      if(ch1.eq.'*'.or.ch1.eq.'q') return 
      if(ch1.eq.'n') goto 200 
                                                                        
      n=2 
      help_entries(1)='<tensor>/polar/parameter' 
      help_entries(2)='<tensor>/polar/terminal' 
      inquiries(1)=                                                       &
      'polar angle theta,phi of a vector H from parameters (pi,pi+1)'  
      inquiries(2)='polar angle theta,phi from terminal' 
      topic='input' 
      call decide(topic,n,inquiries,help_entries,lq)
      if(lq.eq.0) return 
      if(lq.eq.2) goto 110 
                                                                        
  105 querytype='parameter number of polarangle theta pi=' 
      form='i3' 
      helpentry='<tensor>/theta/parnumber' 
      value=0.0 
      call askvalue(querytype,form,value,helpentry,ncl,nrd,nwr) 
      ia=value 
      if(ia.eq.0.or.ia.gt.mp-1) then 
       write(nwr,'(a)')'parameter number of theta out of range' 
       goto 105 
      end if 
                                                                        
      out(1)=p(ia) 
      out(2)=p(ia+1) 
                                                                        
       write(nwr,'(/,a)')                                               &
     &    '     vector H, polar angles in  system SE'                   
       write(nwr,'(6x,a,2(f8.2,a1),/)') 'e_H=(',                        &
     &                                    out(1),',',out(2),')'         
                                                                        
      querytype='theta,phi of vector H are correct ' 
      ch1='y' 
      helpentry='<tensor>/vector/transfer' 
      call yesno(querytype,ch1,helpentry,ncl,nrd,nwr) 
      if(ch1.eq.'*'.or.ch1.eq.'q') return 
      if(ch1.eq.'n') goto 105 
                                                                        
      goto 120 
                                                                        
  110 continue 
                                                                        
      querytype='polar angle theta,phi of a vector H:' 
      helpentry='<tensor>/vector/polar' 
      ch132='0,0' 
      call askstring(querytype,ch132,helpentry,ncl,nwr) 
      ch1=ch132(1:1) 
      if(ch1.eq.'*'.or.ch1.eq.'q') return 
                                                                        
      call trlch(30,ch132,nn,out) 
                                                                        
  120 continue 
                                                                        
      st=sin(out(1)*rad) 
      uvec(1)=st*cos(out(2)*rad) 
      uvec(2)=st*sin(out(2)*rad) 
      uvec(3)=cos(out(1)*rad) 
                                                                        
!      do k=1,3                                                         
!       h=0.0                                                           
!       do j=1,3                                                        
!        h=h+uvec(j)*uveco(j,k)                                         
!       end do                                                          
!       uvec1(k)=h                                                      
!      end do                                                           
!      write(6,'(a,3f8.2,a1)')'vector in Sc: (',uvec1,')'               
                                                                        
!      do k=1,3                                                         
!       h=0.0                                                           
!       do j=1,3                                                        
!        h=h+uvec1(j)*uvecn(k,j)                                        
!       end do                                                          
!       uvecv(k)=h                                                      
!      end do                                                           
!      write(6,'(a,3f8.2,a1)')'vector in SE: (',uvecv,')'               
                                                                        
      do i=1,3 
       h=0.0 
       do j=1,3 
        do k=1,3 
         h=h+uvec(j)*uveco(j,k)*uvecn(i,k) 
        end do 
       end do 
       uvecv(i)=h 
      end do 
                                                                        
!     polar angle from coordinates                                      
      thepo(1)=arcos(uvecv(3))/rad 
      h=1.0-uvecv(3)**2 
      if(h.le.0.0) then 
       phipo(1)=0.0 
      else 
       h=sqrt(h) 
       phipo(1)=arcos(uvecv(1)/h) 
       if(uvecv(2).lt.0.0) phipo(1)=2.0*pi-phipo(1) 
      end if 
      phipo(1)=phipo(1)/rad 
                                                                        
       write(nwr,'(/,a)')                                               &
     &    ' new vector H, polar angles in  system SE'                   
       write(nwr,'(6x,a,2(f8.2,a1))') 'e_H=(',                          &
     &                                    thepo(1),',',phipo(1),')'     
                                                                        
      if(ia.gt.0)then 
       p(ia)=thepo(1) 
       p(ia+1)=phipo(1) 
      end if 
  200 continue 
      if(ia.gt.0) call calpl('test') 
      return 
      END                                           


      real function arcos(h) 
      implicit none
      real, parameter:: pi2=2.0*atan(1.0) 
      real  h,hh,s
                                                                  
      hh=h*h 
      if(hh.ge.1) then 
       arcos=0.0 
       if(h.lt.0.0) arcos=2.0*pi2 
      else 
       s=sqrt(1.0-hh) 
       if(abs(h).lt.0.0001) then 
        arcos=pi2 
       else 
        arcos=atan(s/h) 
        if(h.lt.0) arcos=2.0*pi2+arcos 
       end if 
      end if 
      return 
      END                                           
                                                                        
      subroutine uvector(cp132,uvec,info)
      implicit none                                                                  
      character(len=132):: cp132 
      real      h,uvec(3),out(32) 
      integer   info,i,il,ist,k,nn
                                                                 
       il=LEN_TRIM(cp132) 
       do k=2,il-1 
        if(cp132(k:k).eq.' ') then 
         do i=k,il-1 
          cp132(i:i)=cp132(i+1:i+1) 
         end do 
        end if 
       end do 
                                                                        
       ist=index(cp132,',*,') 
       if(ist.gt.0) then 
        cp132(ist:ist+2)='   ' 
        call trlch(il,cp132,nn,out) 
        if(nn.lt.2) then 
         print'(a)', 'less than 2 components found for eax' 
         info=1
         return 
        end if 
        h=out(1)**2+out(2)**2 
        if(h.gt.1) then 
         h=sqrt(h) 
         out(1)=out(1)/h 
         out(2)=out(2)/h 
         out(3)=0.0 
        else 
         out(3)=sqrt(1.0-h) 
        end if 
        uvec(1)=out(1) 
        uvec(2)=out(3) 
        uvec(3)=out(2) 
        goto 100 
       end if 
                                                                        
       ist=index(cp132,'*,') 
       if(ist.gt.0) then 
        cp132(ist:ist+1)='   ' 
        call trlch(il,cp132,nn,out) 
        if(nn.lt.2) then 
         print'(a)', 'less than 2 components found for eax' 
         info=1
         return 
        end if 
        h=out(1)**2+out(2)**2 
        if(h.gt.1) then 
         h=sqrt(h) 
         out(1)=out(1)/h 
         out(2)=out(2)/h 
         out(3)=0.0 
        else 
         out(3)=sqrt(1.0-h) 
        end if 
        uvec(1)=out(3) 
        uvec(2)=out(1) 
        uvec(3)=out(2) 
        goto 100 
       end if 
                                                                        
       ist=index(cp132,',*') 
       if(ist.gt.0) then 
        cp132(ist:ist+1)='   ' 
        call trlch(il,cp132,nn,out) 
        if(nn.lt.2) then 
         print'(a)', 'less than 2 components found for eax' 
         info=1
         return 
        end if 
        h=out(1)**2+out(2)**2 
        if(h.gt.1) then 
         h=sqrt(h) 
         out(1)=out(1)/h 
         out(2)=out(2)/h 
         out(3)=0.0 
        else 
         out(3)=sqrt(1.0-h) 
        end if 
        uvec(1)=out(1) 
        uvec(2)=out(2) 
        uvec(3)=out(3) 
        goto 100 
       end if 
                                                                        
       call trlch(il,cp132,nn,out) 
       uvec(1)=out(1) 
       uvec(2)=out(2) 
       uvec(3)=out(3) 
                                                                        
  100 continue 
      return 
      END                                           
                                                                        
      subroutine crospr(uvec1,uvec2,uvec3) 
      implicit none
      real, dimension(3):: uvec1,uvec2,uvec3
      real     h 
      integer  k  
                                                                
      uvec3(1)= uvec1(2)*uvec2(3)-uvec1(3)*uvec2(2) 
      uvec3(2)=-uvec1(1)*uvec2(3)+uvec1(3)*uvec2(1) 
      uvec3(3)= uvec1(1)*uvec2(2)-uvec1(2)*uvec2(1) 
      h=0.0 
      do k=1,3 
       h=h+ uvec3(k)**2 
      end do 
      h=sqrt(h) 
      do k=1,3 
       uvec3(k)=uvec3(k)/h 
      end do 
!      write(6,'(3f12.3)')(uvec1(k),k=1,3)                              
!      write(6,'(3f12.3)')(uvec2(k),k=1,3)                              
!      write(6,'(3f12.3)')(uvec3(k),k=1,3)                              
      return 
      END                                           
                                                                        
     subroutine euler_omega1(ax_theta,ax_phi,omega,oalpha,obeta,ogamma,&
                      alpha,beta,gamma,thickness,case,enwr,infotext,info)     
      implicit none
     integer, intent(in) :: case 
     real,    intent(in) :: ax_theta,ax_phi,omega,oalpha,obeta,ogamma  
     real,    intent(out):: alpha,beta,gamma,thickness
     character(len=4), dimension(16), intent(in):: enwr
     character(len=132), dimension(8), intent(out):: infotext
     integer, intent(out):: info 

     character(len=6),dimension(3),parameter:: eai=(/'      x','e_C = y','      z'/) 
     character(len=6),dimension(3),parameter:: ebi=(/'      x','r_ax= y','      z'/)
     character(len=8),dimension(3),parameter::text=(/' in the ',' Lab.   ',' system '/) 
     real, dimension(3,3):: uveca,uvecb,uvec 
     real, parameter:: pi=3.141592654,rad=atan(1.)/45., eps=0.001
     logical   b_error 
     integer   l,k 
     real      cosb,cosg,cosa,omcosg,sina,sinb,sing,xalpha,xbeta,xgamma, & 
               cos2b,sin2b,coshps,coshms,sinhps,sinhms,phi,dx,dy,dz,h,st,arcos  

!  write(6,'(5f12.4)')ax_theta,ax_phi,omega,oalpha,obeta,ogamma,&
!                            alpha,beta,gamma,thickness
!    write(6,'(a,3f10.1)')' alpha,beta,gamma=',oalpha,obeta,ogamma   
!    oalpha,obeta,ogamma is the transformation of the crystal system t
!    absorber system.                                                 
!    omega rotation around the axis ax_theta,ax_phi in the absorbersys
                                                                      
!    inverse matrix uveca  
     cos2b=cos(obeta*rad/2.)**2; sin2b=sin(obeta*rad/2.)**2
     coshps=cos((oalpha+ogamma)*rad); sinhps=sin((oalpha+ogamma)*rad)
     coshms=cos((oalpha-ogamma)*rad); sinhms=sin((oalpha-ogamma)*rad)
 
     uveca(1,1)=cos2b*coshps-sin2b*coshms
     uveca(1,2)=cos2b*sinhps-sin2b*sinhms
     uveca(1,3)=-sin(obeta*rad)*cos(ogamma*rad)
                 
     uveca(2,1)=-(cos2b*sinhps+sin2b*sinhms)
     uveca(2,2)= (cos2b*coshps+sin2b*coshms)
     uveca(2,3)=sin(obeta*rad)*sin(ogamma*rad)
                 
     uveca(3,1)=sin(obeta*rad)*cos(oalpha*rad)
     uveca(3,2)=sin(obeta*rad)*sin(oalpha*rad)
     uveca(3,3)=cos(obeta*rad)  

!     do k=1,3 
!      print'(a12,x,2(a6,a3,3(f7.3,a1),3x))', text(k),               &
!      eai(k),' ( ',uveca(k,1),',',uveca(k,2),',',uveca(k,3),')'      
!     end do 

     xalpha=ax_phi*rad 
     xbeta=ax_theta*rad 
     xgamma=omega*rad 
                                                                      
     cosa=cos(xalpha) 
     sina=sin(xalpha) 
     cosb=cos(xbeta) 
     sinb=sin(xbeta) 
                                                                      
     cosg=cos(xgamma) 
     sing=sin(xgamma) 
     omcosg=1.0-cosg 
                                                                      
     dx=sinb*cosa 
     dy=sinb*sina 
     dz=cosb 
                                                                      
                                                                      
     uvecb(1,1)=dx**2*omcosg +cosg 
     uvecb(1,2)=dx*dy*omcosg+dz*sing 
     uvecb(1,3)=dx*dz*omcosg-dy*sing 
                                                                      
     uvecb(2,1)=dx*dy*omcosg-dz*sing 
     uvecb(2,2)=dy**2*omcosg+cosg 
     uvecb(2,3)=dz*dy*omcosg+dx*sing 
                                                                      
     uvecb(3,1)=dx*dz*omcosg+dy*sing 
     uvecb(3,2)=dy*dz*omcosg-dx*sing 
     uvecb(3,3)=dz**2*omcosg+cosg  
                                                                      
     uvec=matmul(uveca,uvecb)
     uvecb=uvec        

     uveca=0.
     do l=1,3 
      uveca(l,l)=1. 
     end do 
                                                                      
     b_error=.false. 
     if(case.eq.3)then 
!     1. case: Rotation axis (defined with respect to the lab. system 
!             is the g-direction) is normal to the surface of the samp
!             The thickness of the sample increases by the factor 1/co
!     2. case: rotation axis lies in the sample plane (ax_theta=90°) 
!             The sample is rotated out of the position orthonormal to
!             g-direction by the rotation angle omega. The thickness o
!             sample increases by the factor 1/cos(omega)             
!     Which case is decided by theta<90°.                            
      b_error=.true. 
      if(abs(ax_theta-90).lt.0.01) then 
       if(cosg.gt.0.05)then 
        thickness=1./cosg 
        b_error=.false. 
       end if 
      else 
       if(cosb.gt.0.05)then 
        thickness=1./cosb 
        b_error=.false. 
       end if 
      end if 
      if(b_error)then 
       info=1
       infotext(1)='<euler>/sample/rotation/axis' 
       infotext(2)(1:17)=' Rotation axis is ill defined' 
       infotext(3)=' ' 
       goto 10 
      end if 
     end if 
                                                                      
!    vectors uveca and uvecb are known                                
!    calculation of Euler angle alpha,beta,gamma                      
                                                                      
     h=0.0 
     do k=1,3 
      h=h+uveca(3,k)*uvecb(3,k) 
     end do 
     beta=arcos(h) 
     st=sin(beta)*(1.+eps/100.) 
     if(beta.lt.eps) then 
      beta=0.0 
      h=0.0 
      do k=1,3 
       h=h+uveca(1,k)*uvecb(1,k) 
      end do 
      alpha=arcos(h) 
      h=0.0 
      do k=1,3 
       h=h+uveca(1,k)*uvecb(2,k) 
      end do 
      if(h.gt.0.0) alpha=2.0*pi-alpha 
      gamma=0.0 
      ! beta=0                                                   
     else    ! sin(beta)=0  
      if(abs(st).lt.eps) then 
       h=0.0 
       do k=1,3 
        h=h+uveca(1,k)*uvecb(1,k) 
       end do 
       gamma=0.0 
       alpha=arcos(-h) 
       h=0.0 
       do k=1,3 
        h=h+uveca(1,k)*uvecb(2,k) 
       end do 
       if(h.gt.0.0) alpha=2.0*pi-alpha 
      else 
       h=0.0 
       do k=1,3 
        h=h+uveca(1,k)*uvecb(3,k) 
       end do 
       alpha=arcos(h/st) 
       h=0.0 
       do k=1,3 
        h=h+uveca(2,k)*uvecb(3,k) 
       end do 
       if(h.lt.0.0) alpha=2.0*pi-alpha 
       h=0.0 
       do k=1,3 
        h=h+uveca(3,k)*uvecb(1,k) 
       end do 
       gamma=arcos(-h/st) 
       h=0.0 
       do k=1,3 
        h=h+uveca(3,k)*uvecb(2,k) 
       end do 
      if(h.lt.0.0) gamma=2.0*pi-gamma
      end if  ! sin(beta)#0   
     end if   ! beta=0  


     if(enwr(7).eq.'thph')then
      phi=atan(-sin(gamma)/cos(gamma))                                      
      if(-sin(beta)*cos(gamma).lt.0.) phi=phi+pi      
      print'(a,2f12.2)','theta,phi=',beta/rad,phi/rad
     end if

     beta=beta/rad 
     alpha=alpha/rad 
     gamma=gamma/rad 
                                                                      
!    write(nwr,'(/,a)') 'unit vectors of system Sb in system S'      
!    caution! vector a is the transpose matrix in order to print the  
!    unit vectors of the rotatet system in the original system.       

 10 if(enwr(7).eq.'eulr'.or.b_error) then 
     print'(/,a,/,3(a,f10.2),/)',' unit vectors of system Sa and system Sb',&
                    ' ax_theta=',ax_theta,' ax_phi=',ax_phi,' omega=',omega
     do k=1,3 
      print'(a12,x,2(a6,a3,3(f7.3,a1),3x))', text(k),               &
      eai(k),' ( ',uvecb(k,1),',',uvecb(k,2),',',uvecb(k,3),')'      
     end do 
     print'(/,20x,a,/,a,3(f6.1,a),/)','Euler angles alpha,beta,gamma',&
                                    ' eb(n)=Sum;m ea(m) D1,mn(alpha=',&
                              alpha,',beta=',beta,',gamma=',gamma,')'
     if(b_error) return 
    end if 

    return 

  END subroutine euler_omega1
                                                                        
                                                                        
                                                                        
    subroutine euler_omega2(ax_theta,ax_phi,omega,ax_theta2,ax_phi2,omega2,&
                              alpha,beta,gamma )                        
     implicit none
     real, intent(in) :: ax_theta,ax_phi,omega,ax_theta2,ax_phi2,omega2       
     real, intent(out):: alpha,beta,gamma

     real, dimension(3,3):: uveca,uvecb,uvec 
     character(len=6),dimension(3),parameter:: eai=(/'      x','e1  = y','      z'/) 
     character(len=6),dimension(3),parameter:: ebi=(/'      x','e21 = y','      z'/)
     integer  k,l
     real     cosa,cosb,cosg,omcosg,sina,sinb,sing,xalpha,xbeta,xgamma,&
              arcos,dx,dy,dz,h,st
     real, parameter:: pi=3.141592654,rad=0.017453293, eps=0.0001
                                                              
     xalpha=ax_phi2*rad 
     xbeta=ax_theta2*rad 
     xgamma=omega2*rad 
                                                                      
     cosa=cos(xalpha) 
     sina=sin(xalpha) 
     cosb=cos(xbeta) 
     sinb=sin(xbeta) 
                                                                      
     cosg=cos(xgamma) 
     sing=sin(xgamma) 
     omcosg=1.0-cosg 
                                                                      
     dx=sinb*cosa 
     dy=sinb*sina 
     dz=cosb 
                                                                      
     uveca(1,1)=dx**2*omcosg +cosg 
     uveca(1,2)=dx*dy*omcosg+dz*sing 
     uveca(1,3)=dx*dz*omcosg-dy*sing 
                                                                      
     uveca(2,1)=dx*dy*omcosg-dz*sing 
     uveca(2,2)=dy**2*omcosg+cosg 
     uveca(2,3)=dz*dy*omcosg+dx*sing 
                                                                      
     uveca(3,1)=dx*dz*omcosg+dy*sing 
     uveca(3,2)=dy*dz*omcosg-dx*sing 
     uveca(3,3)=dz**2*omcosg+cosg 
                                                                      
                                                                      
                                                                      
     xalpha=ax_phi*rad 
     xbeta=ax_theta*rad 
     xgamma=omega*rad 
                                                                      
     cosa=cos(xalpha) 
     sina=sin(xalpha) 
     cosb=cos(xbeta) 
     sinb=sin(xbeta) 
                                                                      
     cosg=cos(xgamma) 
     sing=sin(xgamma) 
     omcosg=1.0-cosg 
                                                                      
     dx=sinb*cosa 
     dy=sinb*sina 
     dz=cosb 
                                                                      
     uvecb(1,1)=dx**2*omcosg +cosg 
     uvecb(1,2)=dx*dy*omcosg+dz*sing 
     uvecb(1,3)=dx*dz*omcosg-dy*sing 
                                                                      
     uvecb(2,1)=dx*dy*omcosg-dz*sing 
     uvecb(2,2)=dy**2*omcosg+cosg 
     uvecb(2,3)=dz*dy*omcosg+dx*sing 
                                                                      
     uvecb(3,1)=dx*dz*omcosg+dy*sing 
     uvecb(3,2)=dy*dz*omcosg-dx*sing 
     uvecb(3,3)=dz**2*omcosg+cosg 
                                                                      
     uvec=matmul(uvecb,uveca)
     uvecb=uvec
     uvec=uveca

     uveca=0.
     do l=1,3 
      uveca(l,l)=1. 
     end do 
                                                                      
!    vectors uveca and uvecb are known                                
!    calculation of Euler angle alpha,beta,gamma                      
                                                                      
     h=0.0 
     do k=1,3 
      h=h+uveca(3,k)*uvecb(3,k) 
     end do 
     beta=arcos(h) 
     st=sin(beta) 
     if(beta.lt.eps) then 
      beta=0.0 
      beta=beta/rad 
      h=0.0 
      do k=1,3 
       h=h+uveca(1,k)*uvecb(1,k) 
      end do 
      alpha=arcos(h) 
      h=0.0 
      do k=1,3 
       h=h+uveca(1,k)*uvecb(2,k) 
      end do 
      if(h.gt.0.0) alpha=2.0*pi-alpha 
      gamma=0.0 
     else                     ! beta=0 
      if(abs(st).lt.eps) then ! sin(beta)=0   
       h=0.0 
       do k=1,3 
        h=h+uveca(1,k)*uvecb(1,k) 
       end do 
       gamma=0.0 
       alpha=arcos(-h) 
       h=0.0 
       do k=1,3 
        h=h+uveca(1,k)*uvecb(2,k) 
       end do 
       if(h.gt.0.0) alpha=2.0*pi-alpha 
      else 
       h=0.0 
       do k=1,3 
        h=h+uveca(1,k)*uvecb(3,k) 
       end do 
       alpha=arcos(h/st) 
       h=0.0 
       do k=1,3 
        h=h+uveca(2,k)*uvecb(3,k) 
       end do 
       if(h.lt.0.0) alpha=2.0*pi-alpha 
                                                                      
       h=0.0 
       do k=1,3 
        h=h+uveca(3,k)*uvecb(1,k) 
       end do 
       gamma=arcos(-h/st) 
       h=0.0 
       do k=1,3 
        h=h+uveca(3,k)*uvecb(2,k) 
       end do 
      if(h.lt.0.0) gamma=2.0*pi-gamma 
      end if ! sin(beta)#0 
     end if ! beta=0 
     beta=beta/rad 
     alpha=alpha/rad 
     gamma=gamma/rad 
                                                                      
!     write(nwr,'(/,a)') 'unit vectors of system Sb in system S'      
!    caution! vector a is the transpose matrix in order to print the  
!    unit vectors of the rotatet system in the original system.       
                                                                      
     do k=1,3 
      print'(a8,x,2(a6,a3,3(f7.3,a1),3x))','        ',                &
      eai(k),' ( ',uvec(k,1),',',uvec(k,2),',',uvec(k,3),')',         &
      ebi(k),' ( ',uvecb(k,1),',',uvecb(k,2),',',uvecb(k,3),')'       
     end do 
     print'(/,20x,a,/,a,3(f6.1,a),/)','Euler angles alpha,beta,gamma',&
                                     'eb(n)=Sum;m ea(m) D1,mn(alpha=',&
                              alpha,',beta=',beta,',gamma=',gamma,')'                       
    return 
   END                                           
                                                                        

      subroutine exyz(phi,theta,psi,invers) 
      implicit none 
      integer invers,m,n
      real    phi,theta,psi,hx,hy,hz
      complex djv(-1:1,-1:1),ev(-1:1),evt(-1:1),hc1 
                                                                        
       if(invers.eq.-1) then 
        call dimj(-psi,-theta,-phi,1,djv) 
       else 
        call dimj(phi,theta,psi,1,djv)
       end if 
                                                                        
       ev(1)=-cmplx(1,0)/sqrt(2.) 
       ev(0)=(0.,0.) 
       ev(-1)=-conjg(ev(1)) 
       do n=-1,1 
        hc1=(0.,0.) 
        do m=-1,1 
         hc1=hc1+ev(m)*djv(m,n) 
        end do 
        evt(n)=hc1 
       end do 
       hx=-real(evt(1)-evt(-1))/sqrt(2.0) 
       hy= real((evt(1)+evt(-1))*cmplx(0.,1.))/sqrt(2.0) 
       hz= real(evt(0)) 
       print'(a,3f8.3,a1)', ' ex= (',hx,hy,hz,')' 
                                                                        
       ev(1)=-cmplx(0,1)/sqrt(2.) 
       ev(0)=(0.,0.) 
       ev(-1)=-conjg(ev(1)) 
       do n=-1,1 
        hc1=(0.,0.) 
        do m=-1,1 
         hc1=hc1+ev(m)*djv(m,n) 
        end do 
        evt(n)=hc1 
       end do 
       hx=-real(evt(1)-evt(-1))/sqrt(2.0) 
       hy= real((evt(1)+evt(-1))*cmplx(0.,1.))/sqrt(2.0) 
       hz= real(evt(0)) 
       print'(a,3f8.3,a1)', ' ey= (',hx,hy,hz,')' 
                                                                        
       ev(1)=(0.,0.) 
       ev(0)=(1.,0.) 
       ev(-1)=-conjg(ev(1)) 
       do n=-1,1 
        hc1=(0.,0.) 
        do m=-1,1 
         hc1=hc1+ev(m)*djv(m,n) 
        end do 
        evt(n)=hc1 
       end do 
       hx=-real(evt(1)-evt(-1))/sqrt(2.0) 
       hy= real((evt(1)+evt(-1))*cmplx(0.,1.))/sqrt(2.0) 
       hz= real(evt(0)) 
       print'(a,3f8.3,a1)', ' ez= (',hx,hy,hz,')' 
                                                                        
       return 
      END                                           
      
