      subroutine jacHermite(N,fOUT,A,ED,V,nrot) 
!     single precision Jacobi diagonalization of a Hermitian NxN Matrix
!     following the idea of Prof. Erich Gerdau - The diagonalization of the 2x2
!     matrix in his code has been optimized here (H. Spiering)         
      implicit none 
      integer N,nrot,is,ip,iq,ir 
      complex,dimension(N,N):: A,V
      real, dimension(N):: ED
      complex  upp,upq,uqp,uqq,vpp,vpq,vqp,vqq,                          &
               Apq,Arp,Arq,Apr,Aqr,Vpr,Vqr                               
      real    Apqb,hd,fOUT,hOUT,sd,sc,x,y
      complex,parameter:: C0=cmplx(0.,0.),C1=cmplx(1.,0.)
!      equivalence (vv,V)
      
      V=C0
      do ip=1,N 
        V(ip,ip)=C1 
      end do  
      do ip=1,N 
        ED(ip)=real(A(ip,ip)) 
      end do 
                                                                        
      sd=1.0 
      nrot=0 
      do is=1,50 
        nrot=nrot+1 
        if(sd.eq.0.) goto 1000 
        sd=0. 
        do ip=1,N-1 
          do iq=ip+1,N 
            Apq=A(ip,iq) 
            Apqb=abs(Apq) 
            hOUT=fOUT*Apqb 
            hd=ED(ip)-ED(iq) 
            if(Apqb.gt.1.e-8)then           ! 24.10.2015 to avoid underflow
                                            ! single precission!!
             if(abs(hd).gt.hOUT) then 
              A(ip,iq)=cmplx(0.,0.) 
             else 
!             write(6,'(i3,a,3f15.6)')nrot,'x abshd,sd=',abs(hd),sd,hOUT
              sd=sd+Apqb 
              x=hd/2. 
              y=real(Apq*conjg(Apq)) 
              sc=x/sqrt(x*x+y) 
              upp=cmplx(sqrt((1.0+sc)/2.0),0.)
              upq=Apq*sqrt((1.0-sc)/2.0/y) 
              uqp=-conjg(upq) 
              uqq=conjg(upp) 
                                                                        
              vpp=conjg(upp) 
              vpq=conjg(uqp) 
              vqp=conjg(upq) 
              vqq=conjg(uqq) 
                                                                        
              y=real(vpp*(upp*x+upq*conjg(Apq))+                        &
                               vqp*(upp*Apq-upq*x))                    
              x=(ED(ip)+ED(iq))/2. 
              ED(ip)=x+y 
              ED(iq)=x-y 
              A(ip,iq)=cmplx(0.,0.) 
                                                                        
              do ir=1,ip-1 
                Arp=A(ir,ip) 
                Arq=A(ir,iq) 
                A(ir,ip)=Arp*vpp+Arq*vqp 
                A(ir,iq)=Arp*vpq+Arq*vqq 
              end do 
              do ir=iq+1,N 
                Apr=A(ip,ir) 
                Aqr=A(iq,ir) 
                A(ip,ir)=upp*Apr+upq*Aqr 
                A(iq,ir)=uqp*Apr+uqq*Aqr 
              end do 
              do ir=ip+1,iq-1 
                Apr=A(ip,ir) 
                Arq=A(ir,iq) 
                A(ip,ir)=upp*Apr+upq*conjg(Arq) 
                A(ir,iq)=conjg(Apr)*vpq+Arq*vqq 
              end do 
                                                                        
              do ir=1,N 
                Vpr=V(ip,ir) 
                Vqr=V(iq,ir) 
                V(ip,ir)=upp*Vpr+upq*Vqr 
                V(iq,ir)=uqp*Vpr+uqq*Vqr 
              end do 
            end if 
           end if 
          end do   ! iq=ip+1,N                      
        end do     ! ip=1,N-1                       
       end do       ! sweeps
 1000 continue 
      return 
      END                                           
                                                                        
      subroutine jacHermite16(N,fOUT,A,ED,V,nrot) 
!     double precision jacobi diagonalization  (H. Spiering)
      implicit none 
      integer    N,nrot,is,ip,iq,ir 
      complex(kind=8),dimension(N,N):: A,V
      real(kind=8),   dimension(N)  :: ED
      complex(kind=8) :: upp,upq,uqp,uqq,vpp,vpq,vqp,vqq,            &
                         Apq,Arp,Arq,Apr,Aqr,Vpr,Vqr                        
      real(kind=8)    :: Apqb,hd,fOUT,hOUT,sd,sc,x,y 
      complex(kind=8),parameter:: C0=(0.d0,0.d0),C1=(1.d0,0.d0) 


      V=C0 
      do ip=1,N 
        V(ip,ip)=C1
      end do 
      do ip=1,N 
        ED(ip)=dreal(A(ip,ip)) 
      end do 
                                                                        
      sd=1.d0 
      nrot=0 
      do is=1,50 
        nrot=nrot+1 
        if(sd.eq.0.) goto 1000 
        sd=0.d0 
        do ip=1,N-1 
          do iq=ip+1,N 
            Apq=A(ip,iq) 
            Apqb=abs(Apq) 
            hOUT=fOUT*Apqb 
            hd=ED(ip)-ED(iq) 
            if(Apqb.gt.0.)then 
             if(abs(hd).gt.hOUT) then 
              A(ip,iq)=C0 
             else 
!             write(6,'(i3,a,3f15.6)')nrot,'x abshd,sd=',abs(hd),sd,hOUT
              sd=sd+Apqb 
              x=hd/2.d0 
              y=dreal(Apq*conjg(Apq)) 
              sc=x/sqrt(x*x+y) 
              upp=cmplx(sqrt((1.d0+sc)/2.d0),0.d0) 
              upq=Apq*sqrt((1.d0-sc)/2.d0/y) 
              uqp=-conjg(upq) 
              uqq=conjg(upp) 
                                                                        
              vpp=conjg(upp) 
              vpq=conjg(uqp) 
              vqp=conjg(upq) 
              vqq=conjg(uqq) 
                                                                        
              y=dreal(vpp*(upp*x+upq*conjg(Apq))+                       &
     &                          vqp*(upp*Apq-upq*x))                    
              x=(ED(ip)+ED(iq))/2.d0 
              ED(ip)=x+y 
              ED(iq)=x-y 
              A(ip,iq)=C0 
                                                                        
              do ir=1,ip-1 
                Arp=A(ir,ip) 
                Arq=A(ir,iq) 
                A(ir,ip)=Arp*vpp+Arq*vqp 
                A(ir,iq)=Arp*vpq+Arq*vqq 
              end do 
              do ir=iq+1,N 
                Apr=A(ip,ir) 
                Aqr=A(iq,ir) 
                A(ip,ir)=upp*Apr+upq*Aqr 
                A(iq,ir)=uqp*Apr+uqq*Aqr 
              end do 
              do ir=ip+1,iq-1 
                Apr=A(ip,ir) 
                Arq=A(ir,iq) 
                A(ip,ir)=upp*Apr+upq*conjg(Arq) 
                A(ir,iq)=conjg(Apr)*vpq+Arq*vqq 
              end do 
                                                                        
              do ir=1,N 
                Vpr=V(ip,ir) 
                Vqr=V(iq,ir) 
                V(ip,ir)=upp*Vpr+upq*Vqr 
                V(iq,ir)=uqp*Vpr+uqq*Vqr 
              end do 
                                                                        
            end if 
           end if 
           end do  ! iq=ip+1,N                      
        end do     ! ip=1,N-1                       
      end do       ! sweeps                         
 1000 continue 
      return 
      END                                           
                                                                        
      subroutine jacCmplx16(N,fOUT,A,V,W,nrot) 
!     This Jacobi-routine does the same as the routine comeig(single precission)
!     Extra feature is the factor fOUT (perturbation theory):           
!     fOUT allows to omit an offdiagonal element which is smaller       
!     than fOUTx|difference of the two diagonal elements| 

      implicit none              
      integer N,nrot

      complex(kind=8),dimension(N,N):: V,W 
      complex(kind=8),dimension(N,N):: A  !,WV
      complex(kind=8),dimension(2,2):: uu,vv,aa
              
      complex(kind=8):: upp,upq,uqp,uqq,vpp,vpq,vqp,vqq,                       &
                        Arp,Apr,Vpr,Wrp     
      real(kind=8)   :: fOUT,hOUT,absd,Apqb,sd 
      complex(kind=8),parameter:: C0=(0.d0,0.d0), C1=(1.d0,0.d0) 
      integer ip,iq,ir,is
      V=C0 
      W=C0 

      do ip=1,N 
        V(ip,ip)=C1
      end do 
      do ip=1,N 
        W(ip,ip)=C1 
      end do 
                                                                        
      sd=1.0d0 
      nrot=0 
                                      ! sweep                           
      do is=1,100 
        nrot=nrot+1 
!        write(6,'(i3,a,4f15.6)')nrot,'x abshd,sd=',absd,sd,fOUT            
        if(sd.lt.1.d-6) then 
!         write(6,'(a,g15.3,i5)')'goto 1000 sd=',sd,nrot                        
         goto 1000 
        end if 
        sd=0. 
        do ip=1,N-1 
          do iq=ip+1,N 
            Apqb=abs(A(ip,iq))+abs(A(iq,ip)) 
                                                                        
            hOUT=fOUT*Apqb 
            absd=abs(A(ip,ip))+abs(A(iq,iq)) 
            if(Apqb.gt.0.)then 
            if(absd.gt.hOUT) then 
              A(ip,iq)=C0 
              A(iq,ip)=C0 
!              write(6,'(a,2g15.3)')'absd.gt.hOUT:',absd,hOUT           
            else 
!             write(6,'(i3,a,7f15.6)')nrot,'x abshd,sd,hOUT,Apqb=', &    
!                                        absd,sd,hOUT,Apqb,A(ip,iq)    
             sd=sd+Apqb 
             aa(1,1)=A(ip,ip) 
             aa(1,2)=A(ip,iq) 
             aa(2,1)=A(iq,ip) 
             aa(2,2)=A(iq,iq)
            call comeig2x2(vv,aa,uu)  
!           the faster routine eigcmplx16 does the same as comeig2x2    
!           (but its stability cannot be guaranteed yet)                
!            call eigcmplx16(vv,aa,uu) 
              A(ip,ip)=aa(1,1) 
              A(ip,iq)=C0 
              A(iq,ip)=C0 
              A(iq,iq)=aa(2,2) 
                                                                        
              vpp=uu(1,1) 
              vpq=uu(1,2) 
              vqp=uu(2,1) 
              vqq=uu(2,2) 
                                                                        
              upp=vv(1,1) 
              upq=vv(1,2) 
              uqp=vv(2,1) 
              uqq=vv(2,2) 
                                                                        
              do ir=1,ip-1 
                Apr=A(ip,ir) 
                Arp=A(ir,ip) 
                A(ip,ir)=upp*Apr+upq*A(iq,ir) 
                A(iq,ir)=uqp*Apr+uqq*A(iq,ir) 
                A(ir,ip)=Arp*vpp+A(ir,iq)*vqp 
                A(ir,iq)=Arp*vpq+A(ir,iq)*vqq 
              end do       ! ir=ip+1,iq-1  
              do ir=ip+1,iq-1 
                Apr=A(ip,ir) 
                Arp=A(ir,ip) 
                A(ip,ir)=upp*Apr+upq*A(iq,ir) 
                A(iq,ir)=uqp*Apr+uqq*A(iq,ir) 
                A(ir,ip)=Arp*vpp+A(ir,iq)*vqp 
                A(ir,iq)=Arp*vpq+A(ir,iq)*vqq 
              end do     ! ir=ip+1,iq-1  
              do ir=iq+1,N 
                Apr=A(ip,ir) 
                Arp=A(ir,ip) 
                A(ip,ir)=upp*Apr+upq*A(iq,ir) 
                A(iq,ir)=uqp*Apr+uqq*A(iq,ir) 
                A(ir,ip)=Arp*vpp+A(ir,iq)*vqp 
                A(ir,iq)=Arp*vpq+A(ir,iq)*vqq 
              end do       ! ir=1,i  
!              h1=(0.,0.)                                               
!              do ir=1,N                                                
!               h1=h1+A(ir,ir)                                          
!              end do                                                   
!              write(6,'(a,2f12.3)')'trace=',h1                         
!              do ii=1,N                                                     
!               write(6,'(14f13.6)')(A(ii,jj),jj=1,N)                         
!              end do                                                        
!              read(5,'(a)')                                                 
              do ir=1,N 
                Vpr=V(ip,ir) 
                V(ip,ir)=upp*Vpr+upq*V(iq,ir) 
                V(iq,ir)=uqp*Vpr+uqq*V(iq,ir) 
              end do 
              do ir=1,N 
                Wrp=W(ir,ip) 
                W(ir,ip)=Wrp*vpp+W(ir,iq)*vqp 
                W(ir,iq)=Wrp*vpq+W(ir,iq)*vqq 
              end do            ! ir=1,N                          
            end if              ! Apq>0                           
           end if               ! abs(hd).gt. OUTJ*|A(p,q)|       
          end do                ! iq=ip+1,N                      
        end do                  ! ip=1,N-1                       
      end do                    ! sweeps                         
 1000 continue 

!      WV=matmul(W,V)
!do iq=1,N
!print'(16f8.2)',(aimag(WV(iq,ip)),ip=1,N)
!end do

      return 
      END                                           
                                                                        
      subroutine comeig2x2(vv,aa,tt) 
                                                                        
!      P.J. Eberlein, Nummer.Math. 14, 232-245(1970)                    
!      Algol code translated to Fortran4 by Karl Wimmer,                
!      Phys. Inst. Erlangen (1978)                                      
!      Fortran code transformed to complex arrays by                    
!      H. Spiering, Mainz (2011)                                        
                                                                        
!      aa: nxn matrix to transform to diagonal shape                    
!      by the left (vv) and right (tt) Eigenvectors.                    
!      vv*aa*tt                                                         
!      aa is changed. The eigenvalues are the diagonal elements         
                                                                        
      implicit none 
      complex(kind=8) :: ss1,cc1,ss2,cc2,hc 
      complex(kind=8),dimension(2,2):: vv,aa,tt 
      real(kind=8) :: and,anc,sisw,tau,dtau,taul,anorm,hr,hi,hj,    &
              te,tee,br,bi,er,ei,dr,di,e,b,c,s,d,g,de,root,root1,root2, &
              sig,sa,ca,sx,cx,cb,ch,sb,sh,c1,s1,s2,                     &
              tanh,sxsh,sxch,cxch,cxsh,tse,eta,cos2a,sin2a,             &
              tem,tep,cotx,cot2x                                
      integer i,it 
      real(kind=8),   parameter:: eps=1.e-12,avrperc=0.01
      complex(kind=8),parameter:: C0=(0.d0,0.d0),Cu=(1.d0,0.d0) 
      logical mark 
                                                                        
      mark=.false. 
                                                                        
       vv=C0 
       tt=C0 
      do  i=1,2 
       vv(i,i)=Cu 
       tt(i,i)=Cu 
      end do 
                                                                        
      tau=0. 
      do it=1,50 
!      iteration loop                                                   
                                                                        
       taul=tau 
       tau=abs(aa(1,2))+abs(aa(2,1)) 
                                                                        
       dtau=tau*avrperc 
       if(it.eq.1) anorm=tau*eps 
       if(tau.lt.anorm.and.taul.lt.1.001*tau) goto 2000 
                                                                        
!                                                                       
       mark=.true. 
                                                                        
        hr=0. 
        hi=0. 
        g=0. 
        hj=0. 
        hc=aa(1,2)+aa(2,1) 
        br=dreal(hc) 
        bi=dimag(hc) 
        hc=aa(1,2)-aa(2,1) 
        er=dreal(hc) 
        ei=dimag(hc) 
        hc=aa(1,1)-aa(2,2) 
        dr=dreal(hc) 
        di=dimag(hc) 
        te =br*br+ei*ei+dr*dr 
        tee=bi*bi+er*er+di*di 
        if(te.lt.tee)  goto 15 
        sisw=1. 
        c=br 
        s=ei 
        d=dr 
        de=di 
        root2=sqrt(te) 
        goto 16 
   15   sisw=-1. 
        c=bi 
        s=-er 
        d=di 
        de=dr 
        root2=sqrt(tee) 
   16   root1=sqrt(s*s+c*c) 
        sig=-1. 
        if(d.ge.0.) sig=1. 
        sa=0. 
        ca=-1. 
        if(c.ge.0.) ca=1. 
        if(root1.ge.eps) goto 17 
        sx=0. 
        sa=0. 
        cx=1. 
        ca=1. 
        e=ei 
        if(sisw.gt.0.) e=er 
        b=-br 
        if(sisw.gt.0.) b=bi 
        and=d**2+de**2 
        goto 18 
   17   if(abs(s).le.eps) goto 19 
        sa=s/root1 
        ca=c/root1 
   19   cot2x=d/root1 
        cotx=cot2x+(sig*sqrt(1.+cot2x**2)) 
        sx=sig/sqrt(1.+cotx**2) 
        cx=sx*cotx 
!                                                                       
!        elemente der tranformationsmatrix c1,c2,s1,s2                  
!                                                                       
        eta=(er*br+ei*bi)/root1 
        tse=(br*bi-er*ei)/root1 
        te =sig*(-root1*de+tse*d)/root2 
        tee=(d*de+root1*tse)/root2 
        and=root2*root2+tee*tee 
        tee=hj*cx*sx 
        cos2a=ca*ca-sa*sa 
        sin2a=2.*ca*sa 
        tem=hr*cos2a+hi*sin2a 
        tep=hi*cos2a-hr*sin2a 
        hi=cx*cx*hi+sx*sx*tep-sa*tee 
        hr=cx*cx*hr-sx*sx*tem-ca*tee 
        b =sisw*te*ca+eta*sa 
        e =ca*eta-sisw*te*sa 
                                                                        
   18   s=hr-sig*root2*e 
        c=hi-sig*root2*b 
        root=sqrt(c*c+s*s) 
        if(root.ge.eps) goto 20 
        cb=1. 
        ch=1. 
        sb=0. 
        sh=0. 
        goto 21 
   20   cb=-c/root 
        sb= s/root 
        tee=cb*b-e*sb 
        anc=tee**2 
        tanh=root/(g+2.*(anc+and)) 
        ch=1./sqrt(1.-tanh**2) 
        sh=ch*tanh 
!                                                                       
!        vorbereitung der transformation                                
!                                                                       
   21   sxsh=sx*sh 
        tem=sxsh*(sa*cb-sb*ca) 
        cxch=cx*ch 
        c1=-sxsh*(ca*cb+sa*sb) 
        cc1=cmplx(cxch-tem,c1) 
        cc2=cmplx(cxch+tem,c1) 
                                                                        
        sxch=sx*ch 
        cxsh=cx*sh 
        tep=sx*ch*sa 
        tem=cx*sh*cb 
        s1=tep+tem 
        s2=tep-tem 
        tep=sxch*ca 
        tem=cxsh*sb 
        ss1=cmplx(tep-tem,s1) 
        ss2=cmplx(-tep-tem,s2) 
                                                                        
!       entscheidung, ob eine transformation ausgefuehrt wird           
        tem=abs(ss1)+abs(ss2) 
        if(tem.gt.eps) then 
!        transformation left                                            
         mark =.false. 
         do  i=1,2 
          hc     =aa(1,i)*cc1+aa(2,i)*ss1 
          aa(2,i)=aa(1,i)*ss2+aa(2,i)*cc2 
          aa(1,i)=hc 
          hc     =vv(1,i)*cc1+vv(2,i)*ss1 
          vv(2,i)=vv(1,i)*ss2+vv(2,i)*cc2 
          vv(1,i)=hc 
         end do 
                                                                        
!        transformation right                                           
         do i=1,2 
          hc     = aa(i,1)*cc2-aa(i,2)*ss2 
          aa(i,2)=-aa(i,1)*ss1+aa(i,2)*cc1 
          aa(i,1)= hc 
          hc     = tt(i,1)*cc2-tt(i,2)*ss2 
          tt(i,2)=-tt(i,1)*ss1+tt(i,2)*cc1 
          tt(i,1)= hc 
         end do 
        end if 
                                                                        
       if(mark) goto 2000 
      end do  ! it: iteration loop     
 2000 continue 
      return 
      END                                           
                                                                        
      subroutine eigcmplx16(vv,aa,tt)
!     double precision complex 2x2 non-Hermitian matrix (H. Spiering)
      implicit none
      complex(kind=8),dimension(2,2):: tt,vv,aa
      complex(kind=8) :: amd,sqbdc,bb,cc,alp,alp2,alpha1,alpha2,        &
                         sqcdb,htr,ha1,ha2                                        
      real(kind=8) :: amdb,absbc 
      real(kind=8),   parameter:: sq2=0.707106781d0,eps=1.d-8
      complex(kind=8),parameter:: C0=(0.d0,0.d0) 
                                                                        
      amd=(aa(1,1)-aa(2,2))/2.0 
      amdb=abs(dimag(amd))+abs(dreal(amd)) 
                                                                        
      if(amdb.eq.0.0d0)then 
       absbc=abs(dreal(aa(2,1)))+abs(dimag(aa(2,1)))+                   &
             abs(dreal(aa(1,2)))+abs(dimag(aa(1,2)))                    
       if(absbc.gt.0.d0) then 
        sqbdc=sqrt(aa(1,2)/aa(2,1)) 
        sqcdb=sq2/sqbdc 
        sqbdc=sqbdc*sq2 
                                                                        
        tt(1,1)=cmplx(sq2,0.d0) 
        tt(1,2)=cmplx(sq2,0.d0) 
        tt(2,1)= sqcdb 
        tt(2,2)=-sqcdb 
                                                                        
        vv(1,1)=cmplx(sq2,0.d0) 
        vv(1,2)= sqbdc 
        vv(2,1)=cmplx(sq2,0.d0) 
        vv(2,2)=-sqbdc 
       end if 
      else 
       bb=aa(1,2)/amd 
       cc=aa(2,1)/amd 
       alp2=1.0d0+bb*cc 
       if(abs(alp2).lt.eps)then 
        bb=aa(1,2)/amd*(1.0d0+eps) 
        alp2=1.0d0+bb*cc 
       end if 
       alp=sqrt(alp2) 
!       write(6,'(a,6f15.6)')'abs(bb),abs(cc),alp=',abs(bb),abs(cc),alp 
       if(abs(bb).gt.abs(cc))then 
                                                                        
        tt(1,1)=1.d0/alp 
        tt(1,2)=tt(1,1) 
        tt(2,1)=-tt(1,1)*(1.d0-alp)/bb 
        tt(2,2)=-tt(1,1)*(1.d0+alp)/bb 
                                                                        
        vv(1,1)=(1.d0+alp)/2.d0 
        vv(1,2)=bb/2.d0 
        vv(2,1)=-(1.d0-alp)/2.d0 
        vv(2,2)=-vv(1,2) 
                                                                        
        alpha1= sqrt(tt(1,1)**2+ tt(2,1)**2) 
        tt(1,1)= tt(1,1)/alpha1 
        tt(2,1)= tt(2,1)/alpha1 
        alpha2= sqrt(tt(1,2)**2+ tt(2,2)**2) 
        tt(1,2)= tt(1,2)/alpha2 
        tt(2,2)= tt(2,2)/alpha2 
                                                                        
        vv(1,1)=vv(1,1)*alpha1 
        vv(1,2)=vv(1,2)*alpha1 
        vv(2,1)=vv(2,1)*alpha2 
        vv(2,2)=vv(2,2)*alpha2 
                                                                        
       else 
                                                                        
        tt(1,1)=0.5d0/alp 
        tt(1,2)=tt(1,1) 
        tt(2,1)=tt(1,1)*cc/(1.d0+alp) 
        tt(2,2)=tt(1,1)*cc/(1.d0-alp) 
                                                                        
        vv(1,1)=1.d0+alp 
        vv(1,2)=-(1.d0-alp2)/cc 
        vv(2,1)=-(1.d0-alp) 
        vv(2,2)=-vv(1,2) 
                                                                        
        alpha1= sqrt(vv(1,1)**2+ vv(1,2)**2) 
        vv(1,1)= vv(1,1)/alpha1 
        vv(1,2)= vv(1,2)/alpha1 
        alpha2= sqrt(vv(2,1)**2+ vv(2,2)**2) 
        vv(2,1)= vv(2,1)/alpha2 
        vv(2,2)= vv(2,2)/alpha2 
                                                                        
        tt(1,1)=tt(1,1)*alpha1 
        tt(2,1)=tt(2,1)*alpha1 
        tt(1,2)=tt(1,2)*alpha2 
        tt(2,2)=tt(2,2)*alpha2 
                                                                        
       end if 
      end if 
                                                                        
      htr=aa(1,1)+aa(2,2) 
      ha1=vv(1,1)*aa(1,1)+vv(1,2)*aa(2,1) 
      ha2=vv(1,1)*aa(1,2)+vv(1,2)*aa(2,2) 
                                                                        
      aa(1,1)=ha1*tt(1,1)+ha2*tt(2,1) 
      aa(2,2)=htr-aa(1,1) 
                                                                        
      return 
      END                                           
                                                                        
                                                                        
      subroutine comeig(n,vv,aa,tt) 
                                                                        
!      P.J. Eberlein, Nummer.Math. 14, 232-245(1970)                    
!      Algol code translated to Fortran4 by Karl Wimmer,                
!      Phys. Inst. Erlangen (1978)                                      
!      Fortran code transformed to complex arrays by                    
!      H. Spiering, Mainz (2011)                                        
                                                                        
!      aa: nxn matrix to transform to diagonal shape                    
!      by the left (vv) and right (tt) Eigenvectors.                    
!      vv*aa*tt                                                         
!      aa is changed. The eigenvalues are the diagonal elements         
                                                                        
      implicit none 
      integer   n 
      complex   ss1,cc1,ss2,cc2,hc 
      complex,dimension(n,n):: vv,aa,tt 
      real    and,anc,sisw,tau,dtau,taul,anorm,hr,hi,hj,te,tee,     &
              br,bi,er,ei,dr,di,e,b,c,s,d,g,de,root,root1,root2,sig,    &
              sa,ca,sx,cx,cb,ch,sb,sh,c1,s1,s2,                         &
              tanh,sxsh,sxch,cxch,cxsh,tse,eta,cos2a,sin2a,             &
              tem,tep,cotx,cot2x                                
      integer i,k,m,it 
      real,parameter:: eps=1.e-12,avrperc=0.01, C0=(0.,0.),CU=(1.,0.) 
      logical mark 
                                                                        
      mark=.false. 
                                                                        
       vv=C0 
       tt=C0

      do  i=1,n 
       vv(i,i)=CU
       tt(i,i)=CU 
      end do 
                                                                        
      tau=0. 
      do it=1,50 
!      iteration loop                                                   
       taul=tau 
       tau=0. 
       do i=1,n 
        do k=i+1,n 
         tau=abs(aa(i,k))+abs(aa(k,i)) 
        end do 
       end do 
!       if(taul/tau.lt.1.001) goto 2000                                 
       dtau=tau/float(n*(n-1))*avrperc 
       if(it.eq.1) anorm=tau*eps 
       if(tau.lt.anorm.and.taul.lt.1.001*tau) goto 2000 
       mark=.true. 
       do  k=1,n-1 
       do  m=k+1,n 
        if(abs(aa(k,m))+abs(aa(m,k)).lt.dtau) goto 222 
        hr=0. 
        hi=0. 
        g=0. 
        hj=0. 
        do i=1,n 
         if(i.ne.k.and.i.ne.m)then 
          hc=aa(k,i)*aa(m,i)-aa(i,k)*aa(i,m) 
          hr=hr+real(hc) 
          hi=hi+aimag(hc) 
          te =aa(i,k)*conjg(aa(i,k))+aa(m,i)*conjg(aa(m,i)) 
          tee=aa(k,i)*conjg(aa(k,i))+aa(i,m)*conjg(aa(i,m)) 
          g =g +tee+te 
          hj=hj+tee-te 
         end if 
        end do 
        hc=aa(k,m)+aa(m,k) 
        br=real(hc) 
        bi=aimag(hc) 
        hc=aa(k,m)-aa(m,k) 
        er=real(hc) 
        ei=aimag(hc) 
        hc=aa(k,k)-aa(m,m) 
        dr=real(hc) 
        di=aimag(hc) 
        te =br*br+ei*ei+dr*dr 
        tee=bi*bi+er*er+di*di 
        if(te.lt.tee)  goto 15 
        sisw=1. 
        c=br 
        s=ei 
        d=dr 
        de=di 
        root2=sqrt(te) 
        goto 16 
   15   sisw=-1. 
        c=bi 
        s=-er 
        d=di 
        de=dr 
        root2=sqrt(tee) 
   16   root1=sqrt(s*s+c*c) 
        sig=-1. 
        if(d.ge.0.) sig=1. 
        sa=0. 
        ca=-1. 
        if(c.ge.0.) ca=1. 
        if(root1.ge.eps) goto 17 
        sx=0. 
        sa=0. 
        cx=1. 
        ca=1. 
        e=ei 
        if(sisw.gt.0.) e=er 
        b=-br 
        if(sisw.gt.0.) b=bi 
        and=d**2+de**2 
        goto 18 
   17   if(abs(s).le.eps) goto 19 
        sa=s/root1 
        ca=c/root1 
   19   cot2x=d/root1 
        cotx=cot2x+(sig*sqrt(1.+cot2x**2)) 
        sx=sig/sqrt(1.+cotx**2) 
        cx=sx*cotx 
!                                                                       
!        elemente der tranformationsmatrix c1,c2,s1,s2                  
!                                                                       
        eta=(er*br+ei*bi)/root1 
        tse=(br*bi-er*ei)/root1 
        te =sig*(-root1*de+tse*d)/root2 
        tee=(d*de+root1*tse)/root2 
        and=root2*root2+tee*tee 
        tee=hj*cx*sx 
        cos2a=ca*ca-sa*sa 
        sin2a=2.*ca*sa 
        tem=hr*cos2a+hi*sin2a 
        tep=hi*cos2a-hr*sin2a 
        hi=cx*cx*hi+sx*sx*tep-sa*tee 
        hr=cx*cx*hr-sx*sx*tem-ca*tee 
        b =sisw*te*ca+eta*sa 
        e =ca*eta-sisw*te*sa 
                                                                        
   18   s=hr-sig*root2*e 
        c=hi-sig*root2*b 
        root=sqrt(c*c+s*s) 
        if(root.ge.eps) goto 20 
        cb=1. 
        ch=1. 
        sb=0. 
        sh=0. 
        goto 21 
   20   cb=-c/root 
        sb= s/root 
        tee=cb*b-e*sb 
        anc=tee**2 
        tanh=root/(g+2.*(anc+and)) 
        ch=1./sqrt(1.-tanh**2) 
        sh=ch*tanh 
!                                                                       
!        vorbereitung der transformation                                
!                                                                       
   21   sxsh=sx*sh 
        tem=sxsh*(sa*cb-sb*ca) 
        cxch=cx*ch 
        c1=-sxsh*(ca*cb+sa*sb) 
        cc1=cmplx(cxch-tem,c1) 
        cc2=cmplx(cxch+tem,c1) 
                                                                        
        sxch=sx*ch 
        cxsh=cx*sh 
        tep=sx*ch*sa 
        tem=cx*sh*cb 
        s1=tep+tem 
        s2=tep-tem 
        tep=sxch*ca 
        tem=cxsh*sb 
        ss1=cmplx(tep-tem,s1) 
        ss2=cmplx(-tep-tem,s2) 
                                                                        
!       entscheidung, ob eine transformation ausgefuehrt wird           
        tem=abs(ss1)+abs(ss2) 
        if(tem.gt.eps) then 
!        transformation left                                            
         mark =.false. 
         do  i=1,n 
          hc     =aa(k,i)*cc1+aa(m,i)*ss1 
          aa(m,i)=aa(k,i)*ss2+aa(m,i)*cc2 
          aa(k,i)=hc 
          hc     =vv(k,i)*cc1+vv(m,i)*ss1 
          vv(m,i)=vv(k,i)*ss2+vv(m,i)*cc2 
          vv(k,i)=hc 
         end do 
                                                                        
!        transformation right                                           
         do i=1,n 
          hc     = aa(i,k)*cc2-aa(i,m)*ss2 
          aa(i,m)=-aa(i,k)*ss1+aa(i,m)*cc1 
          aa(i,k)= hc 
          hc     = tt(i,k)*cc2-tt(i,m)*ss2 
          tt(i,m)=-tt(i,k)*ss1+tt(i,m)*cc1 
          tt(i,k)= hc 
         end do 
        end if 
                                                                        
  222   continue 
       end do  ! m
       end do ! k 
       if(mark) goto 2000 
      end do   ! it: iteration loop 
 2000 continue 
      return 
      END                                           
                                                                        
                                                                        
                                                                        
                                                                        
      real function clebsch(a,am,b,bm,c,cm) 
      implicit none                                                                  
! this function calculates the Clebsch Gordan coefficients              
!     C(a,b,am,bm/c,cm)
      real, intent(in):: a,am,b,bm,c,cm                                                 
      real(kind=8):: fac,del,w,sum 
      integer i,ii
      real    hi,h1,h2,h3,h4,h5  
                                                          
      if(abs(am).gt.a) goto 12 
      if(abs(bm).gt.b) goto 12 
      if(abs(cm).gt.c) goto 12 
      if(a+b.lt.c) goto 12 
      if(a+c.lt.b) goto 12 
      if(b+c.lt.a) goto 12 
      if(am+bm.ne.cm) goto 12 
      del=fac(a+b-c)*fac(a+c-b)*fac(b+c-a)/fac(a+b+c+1.) 
        w=fac(a+am)*fac(a-am)*fac(b+bm)*fac(b-bm)*                      &
     &      fac(c+cm)*fac(c-cm)                                         
        w=dsqrt(w*del) 
                                                                        
      sum=0.d0 
      hi=-1. 
     do ii=1,40 
        hi=hi+1. 
       h1=a-am-hi 
      if(h1.lt.0) goto 10 
       h2=b+bm-hi 
      if(h2.lt.0) goto 10 
       h3=a+b-c-hi 
      if(h3.lt.0) goto 10 
       h4=c-b+am+hi 
      if(h4.lt.0) goto 9 
       h5=c-a-bm+hi 
      if(h5.lt.0) goto 9 
      i=mod(ii-1,2) 
      sum=sum+(-1.d0)**i/(fac(h1)*fac(h2)*fac(h3)*                      &
     &        fac(h4)*fac(h5)*fac(hi))                                  
    9 continue 
     end do
   10 clebsch=sngl(w*sum)*sqrt(2.*c+1.) 
!      write(6,'(6f5.1,2x,f10.3)')a,am,b,bm,c,cm,clebsch
      return 
   12 clebsch=0.0 
      return 
      END                                           
                                                                        
                                                                        
                                                                        
      function fac(x) 
!     this function camculates the faculty up to 18 
      implicit none                    
      real, intent(in):: x
      real(kind=8):: fac
      integer n 
      real(kind=8), dimension(19),parameter:: & 
       f=(/1.d0,1.d0,2.d0,6.d0,24.d0,120.d0,720.d0,5040.d0,           &
           40320.d0,362880.d0,36288.d2,  399168.d2,4790016.d2,        &
           62270208.d2,87178291.d3,1307674365.d3,2092278984.d4,       &
           35568742728.d4,640237369104.d4/)                            
      n=x 
      if(n.lt.0) stop 
      fac=f(n+1) 
      return 
      END                                           
                                                                        
                                                                        
      real function wi3j(a,b,c,am,bm,cm) 
      implicit none
      real a,b,c,am,bm,cm,clebsch
      integer i
       i=a-b-cm 
       i=mod(i,2) 
       wi3j=clebsch(a,am,b,bm,c,-cm)/sqrt(2.*c+1.)*(-1.)**i 
      return 
      END                                           
                                                                        
                                                                        
                                                                        
      subroutine wigner(j,bet,dd) 
!     integral j=0,1,2,4                                                
      implicit none
      integer, intent(in):: j
      real,    intent(in):: bet
      real, intent(out), dimension(-j:j,-j:j):: dd 
      real c,s,c2,cp,cm,sq2,sq5,sq6,sq7,cp2,cp3,cm2,cm3,h 
      integer l,m,mpt
 
       c=cos(bet) 
       s=sin(bet) 
       cp=1.0+c 
       cm=1.0-c 
       sq2=sqrt(2.) 
                                                                        
       if (j.eq.0) then 
        dd(0,0)=1. 
                                                                        
       else if (j.eq.1) then 
       dd(1,1) =cos(bet/2.)**2 
       dd(1,-1)=sin(bet/2.)**2 
       dd(1,0) =-s/sq2 
       dd(0,0) =c 
                                                                        
       else if (j.eq.2) then 
       sq6=sqrt(6.0)/2.0 
       dd(2,2) = cp**2/4.0 
       dd(2,-2)= cm**2/4.0 
       dd(2,1) = -s*cp/2.0 
       dd(2,-1)= -s*cm/2.0 
       dd(2,0) = sq6/2.0*s**2 
       dd(1,1) = cp*(2.0*c-1.0)/2.0 
       dd(1,-1)= cm*(2.*c+1.)/2. 
       dd(1,0) = -sq6*s*c 
       dd(0,0) = (3.*c**2-1.)/2. 
                                                                        
       else if (j.eq.4) then 
       sq5=sqrt(5.) 
       sq7=sqrt(7.) 
       cp2=cp**2 
       cp3=cp2*cp 
       cm2=cm**2 
       cm3=cm2*cm 
       dd(4,4) =cp*cp3/16. 
       dd(4,-4)=cm*cm3/16. 
       h=sq2/8.*s 
       dd(4,3) =-h*cp3 
       dd(4,-3)=-h*cm3 
       h=sq7/8.*s**2 
       dd(4,2) =h*cp2 
       dd(4,-2)=h*cm2 
       h=h*s*sq2 
       dd(4,1) =-h*cp 
       dd(4,-1)=-h*cm 
       dd(4,0) =h*s*sq5/2. 
       dd(3,3) =cp3/8.*(4.*c-3.) 
       dd(3,-3)=cm3/8.*(4.*c+3.) 
       h=sq7*s/8. 
       dd(3,2) =-h*sq2*cp2*(2.*c-1.) 
       dd(3,-2)=-h*sq2*cm2*(2.*c+1.) 
       h=h*s 
       dd(3,1) =h*cp*(4.*c-1.) 
       dd(3,-1)=h*cm*(4.*c+1.) 
       dd(3,0) =-h*sq5*2.*s*c 
       c2=c**2 
       dd(2,2) =cp2/4.*(7.*(c2-c)+1.) 
       dd(2,-2)=cm2/4.*(7.*(c2+c)+1.) 
       h=sq2/8.*s 
       dd(2,1) =-h*cp*(7.*(2.*c2-c)-1.) 
       dd(2,-1)=-h*cm*(7.*(2.*c2+c)-1.) 
       dd(2,0) =h*sq5*s*(7.*c2-1.) 
       dd(1,1) =cp/8.*(c*(28.*c2-6.)-21.*c2+3.) 
       dd(1,-1)=cm/8.*(c*(28.*c2-6.)+21.*c2-3.) 
       dd(1,0) =-sq5/4.*s*c*(7.*c2-3.) 
       dd(0,0) =(c2*(35.*c2-30.)+3.)/8. 
       end if 
                                                                        
!      point mirror at index (0,0)                                      
       do mpt=j,1,-1 
       do m=mpt,-mpt,-1 
        l=mod(mpt-m,2) 
        dd(-mpt,-m)=dd(mpt,m)*(-1)**l 
       end do 
       end do 
                                                                        
!      diagonal mirror - indices (m,m)                                  
       do mpt=-1,-j,-1 
       do m=-mpt-1,mpt+1,-1 
        l=mod(mpt-m,2) 
        dd(m,mpt)=dd(mpt,m)*(-1)**l 
       end do 
       end do 
       do mpt=j,1,-1 
       do m=mpt-1,-mpt+1,-1 
        l=mod(mpt-m,2) 
        dd(m,mpt)=dd(mpt,m)*(-1)**l 
       end do 
       end do 
                                                                        
      return 
      END                                           
                                                                        
      subroutine wignerh(j,beta,dd)                                     
!     half integral spin j/2                                            
      implicit none
      integer,intent(in):: j
      real,   intent(in):: beta
      real,   intent(out), dimension(-j:j,-j:j):: dd 

      real c,s,c2,c3,c4,c5,h,s2,s3,s4,s5,w2,w3,w5,w10,beth 

      beth=beta/2.0                                                    
      c=cos(beth)                                                      
      s=sin(beth)                                                      
                                                                       
     if (j.eq.1) then                                                 
      dd(1,1)=c                                                        
      dd(1,-1)=-s                                                      
                                                                       
     else if (j.eq.3) then                                            
      c2=c*c                                                           
      c3=c2*c                                                          
      s2=s*s                                                           
      s3=s2*s                                                          
      w3=sqrt(3.0)                                                     
      dd(3,3)  =c3                                                     
                                                                       
            h=-w3*c2*s                                               
      dd(3,1)  =h                                                      
      dd(-3,-1)=-h                                                     
      dd(-1,-3)= h                                                     
      dd(1,3)  =-h                                                     
                                                                       
             h=w3*c*s2                                                 
      dd(3,-1)=h                                                       
      dd(1,-3)=h                                                       
      dd(-3,1)=h                                                       
      dd(-1,3)=h                                                       
                                                                       
      dd(3,-3)=-s3                                                     
                                                                       
      dd(1,1)=c*(3.0*c2-2.0)                                           
      dd(1,-1)=s*(3.0*s2-2)                                            
                                                                       
      dd(-3,-3)= dd(3,3)                                               
      dd(-3,3)=-dd(3,-3)                                               
     else if (j.eq.5) then                                            
      c2=c*c                                                           
      c3=c2*c                                                          
      c4=c3*c                                                          
      c5=c4*c                                                          
      s2=s*s                                                           
      s3=s2*s                                                          
      s4=s3*s                                                          
      s5=s4*s                                                          
      w2=sqrt(2.0)                                                     
      w5=sqrt(5.0)                                                     
      w10=sqrt(10.0)                                                   
                                                                       
      dd(5,5)  = c5                                                    
      dd(-5,-5)= c5                                                    
                                                                       
             h = -w5*c4*s                                              
      dd(5,3)  = h                                                     
      dd(-5,-3)=-h                                                     
      dd(3,5)  =-h                                                     
      dd(-3,-5)= h                                                     
                                                                       
             h = w10*c3*s2                                             
      dd(5,1)  = h                                                     
      dd(1,5)  = h                                                     
      dd(-5,-1)= h                                                     
      dd(-1,-5)= h                                                     
                                                                       
             h = -w10*c2*s3                                            
      dd(5,-1) = h                                                     
      dd(-5,1) =-h                                                     
      dd(-1,5) =-h                                                     
      dd(1,-5) = h                                                     
                                                                       
             h = w5*c*s4                                               
      dd(5,-3) = h                                                     
      dd(-5,3) = h                                                     
      dd(3,-5) = h                                                     
      dd(-3,5) = h                                                     
                                                                       
      dd(5,-5) =-s5                                                    
      dd(-5,5) = s5                                                    
                                                                       
      dd(3,3)  = c5-4.0*c3*s2                                          
                                                                       
             h = 3.0*w2*c3*s2-2.0*w2*c*s4                              
      dd(3,-1) = h                                                     
      dd(1,-3) = h                                                     
      dd(-3,1) = h                                                     
      dd(-1,3) = h                                                     
                                                                       
            h = 3.0*w2*c2*s3-2.0*w2*c4*s                               
      dd(3,1) = h                                                      
      dd(-3,-1) =-h                                                    
      dd(-1,-3) = h                                                    
      dd(1,3)   =-h                                                    
                                                                       
      dd(3,-3)=s5-4.0*c2*s3                                            
                                                                       
      dd(1,1)=c5-6.0*c3*s2+3.0*c*s4                                    
      dd(1,-1)=-s5+6.0*c2*s3-3.0*c4*s                                  
                                                                       
      dd(-3,-3)= dd(3,3)                                               
      dd(-3,3)=-dd(3,-3)                                               
     end if                                                           
                                                                       
                                                                       
      dd(-1,-1)=dd(1,1)                                                
      dd(-1,1)=-dd(1,-1)                                               
                                                                       
      return                                                           

    end subroutine wignerh        !   Dewigner(halbzahlig)              
                                                                        
!***********************************************************************
      subroutine expo_22(aa,ab,ba,a) 
       implicit none 
       complex,intent(in):: aa,ab,ba
       complex,intent(out),dimension(4):: a
       complex(kind=8):: x11,x12,x21,d,expd,expdi,shd,chd
       real(kind=8):: absd
!      traceless matrix exp(m(x11,x12,x21))= ch(d)+1/d*sh(d)*m          
!      d=sqrt(x11**2+x21*x12)                                           
!          effi (1 3)  to fortran  ( 1 3 )                              
!               (4 2)              ( 2 4 )                              

       x11=aa; x12=ab; x21=ba

       d=x11**2+x12*x21 
       d=sqrt(d) 
       absd=abs(real(d)) 
       if(absd.gt.1.d-7) then 
        expd=exp(d)/2.d0
        expdi=0.25d0/expd 
        shd=(expd-expdi)/d
        chd=expd+expdi 
        a(1)=chd+shd*x11 
        a(4)=chd-shd*x11 
        a(3)=shd*x12 
        a(2)=shd*x21 
       else 
        chd=(1.d0,0.d0) 
        a(1)=chd+x11 
        a(4)=chd-x11 
        a(3)=x12 
        a(2)=x21 
       end if
     
       return 
      END                                           
                                                                                                              
       subroutine matproD(a,b,c)
       implicit none 
!      Fortran  ( 1 3 )     convention                                  
!               ( 2 4 )                                                 
       complex(kind=8),dimension(4):: a,b,c 
       c(1)=a(1)*b(1)+a(3)*b(2) 
       c(3)=a(1)*b(3)+a(3)*b(4) 
       c(2)=a(2)*b(1)+a(4)*b(2) 
       c(4)=a(2)*b(3)+a(4)*b(4) 
       return 
      END                                           
                                                                        
       subroutine matprof(a,b,c)
       implicit none 
!      Fortran  ( 1 3 )     convention                                  
!               ( 2 4 )                                                 
       complex,dimension(4):: a,b,c 
       c(1)=a(1)*b(1)+a(3)*b(2) 
       c(3)=a(1)*b(3)+a(3)*b(4) 
       c(2)=a(2)*b(1)+a(4)*b(2) 
       c(4)=a(2)*b(3)+a(4)*b(4) 
       return 
      END                                           
                                                                        
       subroutine tracematpro4(sa,sb,sc,sd,st)
       implicit none 
       complex,intent(in),dimension(4):: sa,sb,sc,sd
       complex,intent(out):: st 
       complex(kind=8),dimension(4):: u,v,ab,cd 
       integer i 
       do i=1,4 
        u(i)=sa(i) 
        v(i)=sb(i) 
       end do 
       call matproD(u,v,ab) 
       do i=1,4 
        u(i)=sc(i) 
        v(i)=sd(i) 
       end do 
       call matproD(u,v,cd) 
!      Fortran  ( 1 3 )     convention                                  
!               ( 2 4 )                                                 
        st=ab(1)*cd(1)+ab(3)*cd(2)+                                     &
     &    ab(2)*cd(3)+ab(4)*cd(4)                                       
       return 
      END                                           
                                                                        
!  *****************************************************************    
!                                                                       
!                 Fourier-transformation                                
!                   (fortran recipies)                                  
!                   changed to complex                                  
!                transform of 4 arrays (l=1-4)                          
!                          6.6.97  (H. Spiering)                                     
!  *****************************************************************    
      subroutine fourier4(DATA,NN,mFF,ISIGN) 
      implicit none                                                                  

      integer NN,mFF,ISIGN
      real(kind=8) :: THETA 
      complex DATA(mFF) 
      complex ww,wwp,temp 
      integer I,J,jj,ii,l,M,N,MMAX,ISTEP
                                                                        
      N=2*NN 
      J=1 
!                                                                       
      do I=1,N,2 
        if(J.gt.I)then 
          jj=2*(J-1) 
          ii=2*(I-1) 
          do l=1,4 
           temp=DATA(jj+l) 
           DATA(jj+l)=DATA(ii+l) 
           DATA(ii+l)=temp 
          end do 
        endif 
        M=N/2 
    1   if ((M.ge.2).and.(J.gt.M)) then 
          J=J-M 
          M=M/2 
        go to 1 
        endif 
        J=J+M 
      end do 
!                                                                       
      MMAX=2 
    2 if (N.gt.MMAX) then 
        ISTEP=2*MMAX 
        THETA=6.28318530717959D0/(ISIGN*MMAX) 
        wwp=cmplx(1.D0-2.D0*dsin(0.5D0*THETA)**2,dsin(THETA)) 
        ww=cmplx(1.0,0.0) 
        do M=1,MMAX,2 
          do I=M,N,ISTEP 
           ii=2*(I-1) 
           jj=ii+ISTEP 
           do l=1,4 
            temp=ww*DATA(jj+l) 
            DATA(jj+l)=DATA(ii+l)-temp 
            DATA(ii+l)=DATA(ii+l)+temp 
           end do 
          end do
          ww=wwp*ww 
        end do 
        MMAX=ISTEP 
      go to 2 
      endif 
      return 
      END                                           
                                                                        
      SUBROUTINE INDEXX(N,ARRIN,INDX) 
      DIMENSION ARRIN(N),INDX(N) 
      DO  J=1,N 
        INDX(J)=J 
      END DO 
      L=N/2+1 
      IR=N 
   10 CONTINUE 
        IF(L.GT.1)THEN 
          L=L-1 
          INDXT=INDX(L) 
          Q=ARRIN(INDXT) 
        ELSE 
          INDXT=INDX(IR) 
          Q=ARRIN(INDXT) 
          INDX(IR)=INDX(1) 
          IR=IR-1 
          IF(IR.EQ.1)THEN 
            INDX(1)=INDXT 
            RETURN 
          ENDIF 
        ENDIF 
        I=L 
        J=L+L 
   20   IF(J.LE.IR)THEN 
          IF(J.LT.IR)THEN 
            IF(ARRIN(INDX(J)).LT.ARRIN(INDX(J+1)))J=J+1 
          ENDIF 
          IF(Q.LT.ARRIN(INDX(J)))THEN 
            INDX(I)=INDX(J) 
            I=J 
            J=J+J 
          ELSE 
            J=IR+1 
          ENDIF 
        GO TO 20 
        ENDIF 
        INDX(I)=INDXT 
      GO TO 10 
      END                                           
                                                                        
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
                                                                        
      subroutine mat2(A,B,C) 
! ********************************************************************  
!                    Matrix multiplication (for 2X2)                    
! *******************************************************************   
      implicit none 
      complex(kind=8),dimension(2,2):: A,B,C,F
!     C=A*B  also in case mat4(A,B,B)                                          
      F=matmul(A,B)     
      C=F
      return 
      END

                                                                        
      subroutine mat2x3(A,B,C,F) 
! ********************************************************************  
!                    Matrix multiplication (for 2X2)                    
! *******************************************************************   
!                                                                       
      implicit none 
      complex(kind=8),intent(in), dimension(2,2):: A,B,C
      complex(kind=8),intent(out),dimension(2,2):: F
      complex(kind=8),dimension(2,2):: D
!     F=A*B*C       
      D=matmul(B,C)
      F=matmul(A,D)
      return 
      END                                           
                                                                        
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   
                                                                        
      subroutine inv2(A,B) 
! ******************************************************************    
!                    Inverse  matrix (for 2X2)                          
! ******************************************************************    
!                                                                       
      implicit none 
      complex(kind=8),dimension(2,2):: A,B 
      complex(kind=8):: rdetA,A11 
      complex(kind=8),parameter:: C1=(1.D0,0.D0),C0=(0.D0,0.D0) 
      real(kind=8):: absA1,absA2 
      real(kind=8),parameter:: level=1.d17 
                                                                        
!       B=A**(-1)                                                       
      absA2=dble(A(1,2)*conjg(A(1,2))+A(2,1)*conjg(A(2,1)))*level
      absA1=dble(A(1,1)*conjg(A(1,1)+A(2,2)*conjg(A(2,2))))
                                                                        
      if(absA1.gt.absA2)then 
       B(1,1)=C1/A(1,1) 
       B(2,2)=C1/A(2,2) 
       B(1,2)=C0 
       B(2,1)=C0 
      else 
       rdetA=C1/(A(1,1)*A(2,2)-A(1,2)*A(2,1)) 
       A11=A(1,1) 
       B(1,1)= rdetA*A(2,2) 
       B(2,2)= rdetA*A11 
       B(1,2)=-rdetA*A(1,2) 
       B(2,1)=-rdetA*A(2,1) 
      endif 
!                                                                       
      return 
      END                                           
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   
                                                                        
      subroutine sqrt2(A,B) 
! ******************************************************************    
!                     sqrt matrix (for 2X2)                             
! ******************************************************************    
      implicit none 
!                                                                       
       complex(kind=8),dimension(2,2):: A,B
       complex(kind=8):: DetA,TrA,AA,sqDetA 
       real(kind=8)   :: realsqDetTrA 
!                                                                       
!       B     = sqrt(A) =[A+I*sqrt(DetA)]/sqrt[TrA+2*sqrt(DetA)]        
!                                                                       
        DetA  = A(1,1)*A(2,2)-A(1,2)*A(2,1) 
        sqDetA= sqrt(DetA) 
        TrA   = A(1,1)+A(2,2) 
                                                      ! HS 23.11.06     
        realsqDetTrA=dble(TrA*dconjg(sqDetA)) 
        if(realsqDetTrA.gt.0.D0) then 
          AA=1.d0/sqrt(TrA+2.D0*sqDetA) 
          B(1,1)= (A(1,1)+sqDetA)*AA 
          B(2,2)= (A(2,2)+sqDetA)*AA 
        else 
          AA=1.d0/sqrt(TrA-2.D0*sqDetA) 
          B(1,1)= (A(1,1)-sqDetA)*AA 
          B(2,2)= (A(2,2)-sqDetA)*AA 
        end if 
        B(1,2)= A(1,2)*AA 
        B(2,1)= A(2,1)*AA 
                                                                        
        return 
      END                                           


      subroutine mat4(A,B,C) 
! *****************************************************************     
!                    Matrix multiplication (for 4X4)                    
! ******************************************************************    
      implicit none 
      complex(kind=8),intent(in), dimension(4,4):: A,B
      complex(kind=8),intent(out),dimension(4,4):: C
      complex(kind=8),dimension(4,4):: F
!     C=A*B  also in case mat4(A,B,B)                                          
      F=matmul(A,B)     
      C=F
      return 
      END

      subroutine exp2(N) 
! ****************************************************************      
!                    Matrix exponential (for 2X2)                       
! ****************************************************************      
!                                                                       
      implicit none 
      complex(kind=8),intent(inout), dimension(2,2):: N
      complex(kind=8) ::  hTrN,detN,sqdetN,                         &   
                          sindN,cosdN,exphTrN,apsinxpx,apcosdN 
      real(kind=8) :: rehTrN 
      real            absCmatr,absC1,absC2 
      real(kind=8),   parameter:: aLIMIT=2.D1
      real(kind=4),   parameter:: level=1.E17,detcrit=1.E-14 
      complex(kind=8),parameter:: C0=(0.D0,0.D0), C1=(1.D0,0.D0) 
!                                                                       
      hTrN     = 0.5D0*(N(1,1)+N(2,2)) 
      rehTrN   = dble(hTrN) 
                                                                        
      absCmatr=real(dble(N(1,2)*conjg(N(1,2))+                          &
                         N(2,1)*conjg(N(2,1))))                        
      absC1=real(dble(N(1,1)*conjg(N(1,1)+                              &
                      N(2,2)*conjg(N(2,2)))))                       
      absC2=absCmatr*level 
                                                                        
      if(absC1.gt.absC2) then 
       N(1,1)=exp(N(1,1))        !Case of diagonal matrix
       N(1,2)=C0         !#Khaidukov
       N(2,1)=C0 
       N(2,2)=exp(N(2,2)) 
      else if(abs(rehTrN).lt.aLIMIT)then 
       N(1,1)=N(1,1)-hTrN 
       N(2,2)=N(2,2)-hTrN 
!                                                                       
       detN    = N(1,1)*N(2,2)-N(1,2)*N(2,1) 
       sqdetN  = sqrt(detN) 
       sindN   = sin(sqdetN) 
       cosdN   = cos(sqdetN) 
       exphTrN = exp(hTrN) 
!      if det N is small or zero, the sin(x)/x ~ 1-xx/6 and             
!                                cos(x)   ~ 1-xx/2 apprs. are used      
       if(real(abs(sqdetN)).lt.detcrit)then 
        apcosdN = (C1-detN/2.D0) 
        apsinxpx= (C1-detN/6.D0) 
                                                                        
        N(1,1)  = exphTrN*(apcosdN+N(1,1)*apsinxpx) 
        N(2,2)  = exphTrN*(apcosdN+N(2,2)*apsinxpx) 
        N(1,2)  = exphTrN*N(1,2)*apsinxpx 
        N(2,1)  = exphTrN*N(2,1)*apsinxpx 
       else 
        N(1,1)  = exphTrN*(cosdN+sindN*N(1,1)/sqdetN) 
        N(2,2)  = exphTrN*(cosdN+sindN*N(2,2)/sqdetN) 
        N(1,2)  = exphTrN*sindN*N(1,2)/sqdetN 
        N(2,1)  = exphTrN*sindN*N(2,1)/sqdetN 
       end if 
      end if ! abs(real(exp(hTrN))) < aLIMIT 
!                                                                       
      return 
      END                                           

! integral quantum number: j
  subroutine dimj(phi,theta,psi,j,cdj)
  integer, intent(in):: j
  real,    intent(in):: phi,theta,psi
  complex, intent(out)::cdj(-j:j,-j:j)  
  real, dimension(-j:j,-j:j):: dd
  real fm,fn  
  integer m,n 
     call wigner(j,theta,dd)  
     do m=-j,j 
     do n=-j,j 
      fm=float(m)*phi
      fn=float(n)*psi
      cdj(m,n)=cmplx(cos(fm+fn),-sin(fm+fn))*dd(m,n) 
     end do 
     end do 
   return
   end


! half integral quantum number: j means j/2

  subroutine dimjh(phi,theta,psi,j,cdj)
  integer, intent(in):: j
  real,    intent(in):: phi,theta,psi
  complex, intent(out)::cdj(-j:j,-j:j)  
  real, dimension(-j:j,-j:j):: dd
  real fm,fn  
  integer m,n 
     call wignerh(j,theta,dd)  
     do m=-j,j,2 
     do n=-j,j,2 
      fm=float(m)*phi/2.
      fn=float(n)*psi/2.
      cdj(m,n)=cmplx(cos(fm+fn),-sin(fm+fn))*dd(m,n) 
     end do 
     end do 
   return
   end

! vec(phi,theta,psi=0); dimj(phi,theta,0,1,cdj)
! phi, theta are the polar angle of th eunit vector vec.
! =>  vecxmat(1,vec,cdj,vxm) --> vx_x=vx_y=0, vx_z=1.0
! gives the unit vector in z-direction

   subroutine vecxmat(j,vec,mat,vxm)
   complex, intent(in) :: vec(-j:j)
   complex, intent(in) :: mat(-j:j,-j:j)
   complex, intent(out):: vxm(-j:j) 
   complex hc1
   complex, parameter:: C0=(0.,0.)
  
    do n=-j,j 
     hc1=C0
     do m=-j,j 
      hc1=hc1+vec(m)*mat(m,n)                  
     end do 
     vxm(n)=hc1 
    end do 
   return
   end

