!     author : h. spiering                                              
!              institut f. anorg. u. anal. chemie                       
!              johannes gutenberg-universitaet                          
!              d-55099 mainz                                            
!              last version august 2002                                 
!    modified by  L. Deak
!                 Institute for Particle and Nuclear Physics
!                 Wigner RCP of HAS
!                 H- 1121 Budapest
!                 October 2013

     subroutine fr_stroboscopy(cmch,iss,info) 
      use glbconst, only: maxfr1,maxfr2,maxn,maxthss,maxscrd,           &
                          icstrf,ivstrf,maxsub,nwr,ncl
      use comdat, only: b_data,b_theory,name_data,name_theory, &
                        mxpt,mdfpt,mfpt,mlpt 
      use complt, only: n_subset
      use comthe, only: ifrsto,theoryname,thfrcase
     implicit none 
     character(len=4), intent(inout):: cmch
     integer, intent(in) :: iss
     integer, intent(out):: info  
!    common/struct is stored by effi for each set.                     
!    istruct0 containes dimensions which are use in the                
!    main routines strob_the2exp and moss_strob                        
                                                                       
!    mstruct1,istruct1,istruct2 defines the structure of realized      
!    in moss_strob                                                     
!    l1_get,...,l3_get are the array addresses of array_fr,            
!    where the results of the routines called by get_theory            
!    are stored.                                                       
!    l1_put,...,l3_put are the array addresses of array_fr,            
!    where results of the main routine moss_strob are stored           
!    the list of addresses can be printed by the command               
!    st 4_yxxx (see he {st})                                           
!    c1_prp,...,c3_prp are properties of the stored data of different  
!    structural levels. Here the property polarized/scalar is stored   
!    by the first character ci_prp(1:1) being T (polarized) or F (scala
!    (storage of 2x2 matrices/storage of a scalar for each channel).   
                                                                       
     character(len=4) :: c1_prp,c2_prp,c3_prp
     character(len=64):: frthname 
     integer       mstruct1,istruct1,istruct2,                         &
                   l1_get,l2_get,l3_get,l1_put,l2_put,l3_put,          &
                   nstorTH,nstorTH1,nstorTHyy,nstorSI,ipTH,mpt,iaddr
     common/struct/mstruct1,istruct1(0:maxfr1),istruct2(0:maxfr2),     &
                   l1_get(0:maxfr1),l2_get(maxfr2),l3_get(maxthss),    &
                   l1_put(0:maxfr1),l2_put(maxfr2),l3_put(maxthss),    &
                   c1_prp(0:maxfr1),c2_prp(maxfr2),c3_prp(maxthss),    &
                   frthname(2),                                        &
                   mpt,ipTH,nstorTH,nstorTH1,nstorTHyy,nstorSI,iaddr(2)

     real          T_bunch          
     integer       istorTH,istorTH1,istorTHyy,nCC,mvpt,idum1,ifrth,&
                   idum,idum2,nLG,nAS,nTH,nsampling,ien
     common/strLOC/istorTH,istorTH1,istorTHyy,mvpt,idum1(2),nTH,nCC,ifrth(4),&
                   idum2(2),nLG,nAS,idum,nsampling,T_bunch,ien(13)          
!    nAS is the dimension for  th_22index, nTh for strob_the2exp       
     real          acsp,wavelength,dum3,channels,v_range,th_channels,  &
                   th_sampling,drive_mode,dum1,Tbunch,dum2,strob_order,&
                   am_window,dumcs 
     common/acstrf/acsp,wavelength,dum3(3),channels,v_range,th_channels,&
                   th_sampling,drive_mode,dum1,Tbunch,dum2(2),strob_order,&
                   am_window,dumcs(icstrf-16)                                    
     real          avsp,dum,v_max,dumv      
     common/avstrf/avsp,dum(3),v_max,dumv(ivstrf-5)  
     integer       lstruct1,lstruct2,lstruct3,mode_drive 

     character(len=132):: ch132,cp132
     character(len=1)  :: ch1
     character(len=40),dimension(3):: text 
     integer   nchansp,nRSl,jss,jpol,nstrob_order,    &
               isub,k1,k2,l1,l2,l3,mi                            
     real      omega 
                                                                       
!     -nTH, 0, +nTH channels of theoretical spectrum                    
!     (this will be projected on to the experimental spectrum)          
                                                                        
!     nTH  channel correspond to v_max of the experimental spectrum     
!     (definition n_v_max=nTH, n_v corresponds to velocity v)           
                                                                        
!     v_range velocity range of the Mossbauer spectrum of the sample    
!     v_max > v_range; if the 1. stroboscopic order can be resolved     
!     (see time windows , Tbunch dependency) v_max > 3*v_range          
                                                                        
!     nAS=(nstrob_order/2*omega/+v_max+v_range)/v_max*nTH               
!     -nAS,0,+nAS channels for the sample (scatterer) and reference     
!     spectrum is stored                                                
!     (definition nstrob=-nstrob_order,..,0,..,nstrob_order)            
                                                                        
!     integration -> summation over channels in the integral            
!      Sum T(n+n_omega/2,n_v)*T(n-n_omega/2,n_v)                        
!      runs as follows                                                  
!      +v: from -v_range/v_max*nTH-nstrob/2*omega/Tbunch                
!          to   +v_range/v_max*nTH+nstrob/2*omega/Tbunch+n_v            
                                                                        
!     the Lorentz curves are shifted up to line positions (of sample    
!     and reference) at channel nTH.                                    
                                                                        
!***********************************************************************
!                                                                       
!      write(6,'(3a,i3,a,l3)') 'frame01 cmch=',cmch,' iss=',iss,        
!     &           ' b_theory=',b_theory                                 
      if(cmch.eq.'thna') then 
       theoryname='Mossbauer synchrotron stroboscopy      v=200' 
       thfrcase(1)='stroboscopic transmission spectrum(1)' 
       thfrcase(2)='stroboscopic reflection spectrum(2)' 
       return 
      end if 
      if(cmch.eq.'dstr') goto 100 
!***********************************************************************
                                                                        
      istorTH=nstorTH
      istorTH1=nstorTH1
      istorTHyy=nstorTHyy
      nchansp=channels 
      if(cmch.eq.'arr0') then 
!     future
!     storage space
!      Data      th_A    amplitudes   th_exp    str_sub
!     |.........|......|...........|.........|..........|
!     ipt ichanDA ipTH  ipTHamp     ipTHyy     ipSI     

         ipTH=2*nchansp+nchansp/2+1
                       ! complex == 4*nchansp of real
                       ! for mfpt,mdfpt,mxpt,mvpt
       mpt=0           ! absolute address incremented in common/struct/

!      channel numbers of RF                                            
!      hc/E0= 86.0165018                                                
!      Tbunch 
       nstrob_order=strob_order 
       if(am_window.gt.0)  &                                                          
       nstrob_order=strob_order*am_window   ! obtained from subtheory
!                                             called on level 0              
       T_bunch=Tbunch                                     ! A/ns=100mm/s  
       omega=wavelength/Tbunch*100. 
       nTH=th_channels 
                                                                        
       nsampling=2**th_sampling 
       if(nsampling.gt.8.0) nsampling=8.0 
                                                                        
       nRSl=(nstrob_order*omega+2*v_max+v_range)/v_max*nTH 
       nLG=8*nRSl
       ifrsto=ipTH+2*(nTH+1)+2+nchansp/2 !+ maxsub*(1+maxthss) 
!       write(6,'(a,3i8)')'ifrsto=',ifrsto,nsampling,nCC 
!      the relativ addresses are stored after finishing                 
!      the entries arr0,i,e                                             
       nstorTH=ipTH
       nstorTH1=nstorTH+2*(nTH+1) 
       nstorTHyy=nstorTH1+2 
       nstorSI=nstorTHyy+nchansp/2 
!      the relativ addresses are reloaded before entering frtheoryxx    
!      and changed to absolute addresses which are temporarily stored   
!      the 8 integers following c3_prp(maxthss) are considered to be    
!      relative addresses which during reloading are changed to absolute
!      addresses of the complex array  'array_fr(1:marray_sp).          
                                                                        
!       write(6,'(a,8i6)')'nchansp, nTH,nRSl=',nchansp, nTH,nRSl
                                                                      
       nCC=(nRSl+4)*8/nsampling 
       nAS=nCC 
       omega=omega*nTH*8/nsampling
       return 
      else if(cmch.eq.'arri') then 
 
!      1 site storage on the site level (lstruct3)                      
!      address:   l3_put(lstruct3)                                      
!      1 layer storage on the layer level (lstruct2)                    
!      address:   l2_put(lstruct2)                                      
!      storage for each lstruct1,                                       
!      address: l1_put(lstruct1)                                        
!                                                                       
!      If any site is polarized => polarization.                        
                                                                        
!      polarization of a layer  lstruct2                                
       call l_structure(iss,lstruct1,lstruct2,lstruct3) 
!       write(6,'(a,4i3)')'jss,l1,l2,l3=',iss,lstruct1,lstruct2,lstruct3
!       write(6,'(i4,2x,a4,4x,a4)')(jss,c_prop(jss),c3_prp(jss),jss=1,10
                                                                        
       ifrsto=0 
       jpol=-1 
       if(lstruct3.eq.0.and.lstruct2.gt.0.)then 
        if(istruct1(lstruct1)-istruct1(lstruct1-1).gt.1)then 
         jpol=0 
         jss=1 
         do l1=1,lstruct1 
!          write(6,'(i4,a,8x,a4)')jss,' c1_prp(jss)=',c1_prp(jss)       
          jss=jss+1 
          do l2=istruct1(l1-1)+1,istruct1(l1) 
!          write(6,'(i4,a,12x,a4)')jss,' c2_prp(jss)=',c2_prp(jss)      
           jss=jss+1 
           do l3=istruct2(l2-1)+1,istruct2(l2) 
            if(l1.eq.lstruct1.and.l2.eq.lstruct2.and.                   &
                          c3_prp(l3)(1:4).eq.'plrs') jpol=1            
!           write(6,'(a,4i3)')'jss,l1,l2,l3=',jss,l1,l2,l3              
!           write(6,'(i4,a,a4,12x,a4)')jss,' c3_prp(jss)=',c3_prp(l3) 
            jss=jss+1 
           end do 
          end do 
         end do 
        end if 
       end if 
                                                                        
!      Storage of the theory of each lstruct1                           
       if(lstruct2+lstruct3.eq.0) then 
!       if any site is polarized => polarization.                       
         jpol=0 
         jss=1 
         do l1=1,lstruct1 
          jss=jss+1 
          do l2=istruct1(l1-1)+1,istruct1(l1) 
           jss=jss+1 
           do l3=istruct2(l2-1)+1,istruct2(l2) 
!            write(6,'(a,3i4,2x,a5)')'jss,l1,lstruct1,c3_prp(jss)=',    
!                              jss,l1,lstruct1,c3_prp(l3)              
            if(l1.eq.lstruct1.and.c3_prp(l3)(1:4).eq.'plrs')jpol=1 
            jss=jss+1 
           end do 
          end do 
         end do 
       end if 
                                                                        
       if(jpol.eq.1) then 
        ifrsto=8*(8*nCC/nsampling+1) 
       else if(jpol.eq.0) then 
        ifrsto=2*(8*nCC/nsampling+1)
       end if 
!      write(6,'(a,3i8)')'ifrsto=',ifrsto,nsampling,nCC                
       return 
      else if(cmch.eq.'arre') then 
!      The size of  storage li_get and li_put is known by the property  
!      polarized T or F.                                                
!       write(6,'(a,8a6)')' fr_strob c1_prp=',(c1_prp(i),i=0,4)         
!       write(6,'(a,8a6)')' fr_strob c2_prp=',(c2_prp(i),i=1,8)         
!       write(6,'(a,8a6)')' fr_strob c3_prp=',(c3_prp(i),i=1,8)         
                                                                        
       jss=1 
       do lstruct1=1,mstruct1 
!        write(6,'(a,i2,a2,a4)')'c1_prp(',lstruct1,')=',c1_prp(lstruct1)
        c1_prp(lstruct1)='F' 
        jss=jss+1 
        do lstruct2=istruct1(lstruct1-1)+1,istruct1(lstruct1) 
!        write(6,'(a,i2,a2,a4)')'c2_prp(',lstruct2,')=',c2_prp(lstruct2)
         c2_prp(lstruct2)='F' 
         jss=jss+1 
         do lstruct3=istruct2(lstruct2-1)+1,istruct2(lstruct2) 
!          write(6,'(a,i4)')' iset=',lstruct3                           
          if(c3_prp(lstruct3)(1:4).eq.'plrs') then 
           c3_prp(lstruct3)='T' 
           c2_prp(lstruct2)='T' 
          else 
           c3_prp(lstruct3)='F' 
          end if 
          jss=jss+1 
         end do 
         if( c2_prp(lstruct2)(1:1).eq.'T') c1_prp(lstruct1)='T' 
        end do 
       end do 
                                                                        
!       write(6,'(a,8a6)')' fr_strob c1_prp=',(c1_prp(i),i=0,4)         
!       write(6,'(a,8a6)')' fr_strob c2_prp=',(c2_prp(i),i=1,8)         
!       write(6,'(a,8a6)')' fr_strob c3_prp=',(c3_prp(i),i=1,8)         
                                                                        
       ifrsto=0 
       return 
      end if ! arr0,i,e
                                                                        
      mode_drive=drive_mode 
!      write(6,'(a,i4,L5)')'frame04 mode_drive,b_data=',mode_drive,b_dat
!      write(6,'(a)')'frame04 arre'                                     
!      write(6,'(8a4)')(c1_prp(i),i=1,mstruct1)                         
!      write(6,'(8a4)')(c2_prp(i),i=1,istruct1(mstruct1))               
!      write(6,'(8a4)')(c3_prp(i),i=1,istruct2(istruct1(mstruct1)))     
                                                                        
                                                                        
!     initializing the array_fr section for nstorint (see subspectra)   
!     (intensity of all subsets are 0)                                  
                                                                        
!      if(.not.b_theory) then 
!       absolute address of nstorSI is known here                       
!        nstorint=maxsub*(1+maxthss) 
!        do k=1,nstorint 
!         array_fr(nstorSI+k)=(0.,0.) 
!        end do 
!      end if 
                                                                        
      if(cmch.ne.'data'.and.(.not.b_data)) mode_drive=1 
      mi=istruct2(istruct1(mstruct1)) 
!     total number of intensity values of the subsets.                  
!     see select subset                                                 
                                                                        
!      storag on real_fr(2*(iarraysp+nstorFR)+m) (array_f)              
!      corresponds to the i= drive_mode (label 100)                     
      mfpt=mpt*2                 ! y data from real_fr (factor 2)  
      mdfpt=mfpt +nchansp        ! errors of y data (calculated by sqrt
      mxpt =mdfpt+nchansp        ! x data (velocity) drive_mode=3
      mvpt =mxpt +nchansp        ! velocity(i) calculated by drive
      mlpt =mvpt +nchansp
!      mgpt =mvpt +nchansp        ! geof(i) calculated by drive
!      mdgpt=mgpt +nchansp        ! dfdgeo(i) calculated by drive
!      mppt=mxpt
!     write(6,*)'....mpt=',mfpt,mdfpt,mxpt,mvpt
!     write(6,*) nstorTH,nstorTH1,nstorTHyy,nstorSI,mpt,ichanDA,iaddr
!***********************************************************************
      if(cmch.eq.'data') then 
       k1=1 
      else if(cmch.eq.'ecic') then 
       k1=2 
      else if(cmch.eq.'test') then 
       k1=3 
      else if(cmch.eq.'dfdp') then 
       k1=4 
      else if(cmch.eq.'subs'.or.cmch.eq.'subt') then 
       k1=5 
      else if(cmch(1:3).eq.'ple') then 
       k1=6 
      else if(cmch.eq.'dmod'.or.cmch.eq.'plot') then 
       k1=7 
      else if(cmch.eq.'punc')then 
       k1=8 
      end if 
                                                                        
      goto (110,120,200,300,400,500,600,700),k1 
!********************************************************************** 
  100 continue 
      call strobo_plot(cmch) 
      mode_drive=int(drive_mode) 
      if(mode_drive.lt.1.or.mode_drive.gt.3)mode_drive=1 
!       data_str=formdata(mode_drive) 
      call data2dread(0,info) 
      return 
  110 continue 
      k1=iss 
      call data2dread(k1,info) 
      return 
  120  call excinc2d(info) 
      return 
!***********************************************************************
!     'test': theory function is calculated in subroutine mossbauer     
  200 continue 
      k1=1 
      call strob_the2exp(k1,info) 
!     write(6,'(a)') ' after strob_the2exp  test'                       
      return 
!***********************************************************************
!     derivatives df                                                    
  300 continue 
      k1=2 
      call strob_the2exp(k1,info) 
      return 
!***********************************************************************
!     subs, subt                                                        
  400 continue 
                                                                        
      text(1)='        reference' 
      text(2)='           sample' 
      text(3)=' ' 
      if(cmch.eq.'subt') goto 450 
                                                                        
      ch1=' ' 
      if(b_data) then 
       ch132=name_data 
      else 
       ch132=name_theory 
      end if 
      call chcenter(ch132,ch1,ncl) 
      cp132='plot of subspectra of' 
      call chcenter(cp132,ch1,ncl) 
      write(nwr,'(/a,/,a)') cp132(:ncl),ch132(:ncl) 
      ch132='subspectra /added subspectra of' 
      call chcenter(ch132,ch1,ncl) 
      write(nwr,'(/,a)') ch132(:ncl) 
      call moss_selsubspect(text) 
      return 
!*************************************************************          
!  call of moss_subtract                                                
  450 continue 
      print'(a)',' ' 
      print'(a)',' '
      print'(a)','                     subtraction of subspectra' 
      call moss_selsubtract(text,isub) 
      call moss_subtract('stro',mi,isub,info) 
      k1=1 
      call strob_the2exp(k1,info) 
      call strobo_plot(cmch) 
      k1=0 
      k2=0 
      if(b_theory) k1=1 
      call xmosplo(k1,k2) 
      return 
!***********************************************************************
!     plot execute                                                      
!     different plot programs may be called due to a flag constant      
!     to be introduced as global constant                               
  500 continue 
      k1=0 
       if(b_theory) k1=1 
      if(cmch(4:4).eq.'2') k1=2   ! update test
      if(cmch(4:4).eq.'3') k1=3   ! update fit
      k2=1  ! 0: only 1 plot and return, 1: interactive plot 
      call xmosplo(k1,k2) 
      return 
!***********************************************************************
!     display mode , filling of plotarray                               
  600 continue 
!     write(6,'(a,a)') '********* 600 ********',cmch                    
      if(cmch.eq.'dmod') then 
       call strobo_plot(cmch) 
      else 
       if(b_theory) then 
        k1=3 
        call strob_the2exp(k1,info) 
        if(n_subset.gt.0)                                               &
         call moss_subspect('stro',mi,info)                    
       end if 
       call strobo_plot(cmch) 
      end if 
      return 
!***********************************************************************
!     punch data and theory                                             
  700 continue 
      k1=iss 
      call punch2d(k1,info) 
      cmch='dim2'
      return 
                                                                        
      END                                           
                                                                        
                                                                        
                                                                        
     subroutine strob_the2exp(k1,info) 
! ****************************************************************     
!    k1 = 1 call of theory                                             
!    k1 = 2 derivatives                                                
!    k1 = 3 call of theory , calpl is not called                       
! ****************************************************************     
     use glbconst, only: maxfr1,maxfr2,maxn,max8,maxthss,icstrf,ivstrf,infotext
     use comdat, only: mxpt,mlpt,b_theory,b_data
     use comthe, only: array_fr,real_fr,thfk,dfdp,ddpk,ib_df,kana,ibmisfit 
     implicit none 
                                                                       
     integer, intent(in) :: k1
     integer, intent(out):: info

     integer       nstorTH,nstorTh1,nstorTHyy,mvpt,mgpt,mdgpt,nTH,ien
     common/strLOC/nstorTH,nstorTh1,nstorTHyy,mvpt,mgpt,mdgpt,nTH,ien(25)
                                                                       
     real          dum5,channels,dum3,drive_mode,channels_vmax,dumcs 
     common/acstrf/dum5(5),channels,dum3(3),drive_mode,channels_vmax,dumcs(icstrf-11)
                                                                       
     real    avsp,amplitude,base,dum2,v_max,channelv0,                                 &
             v_max1,channelv01,v_max2,channelv02,v_max3,channelv03,geo,dumv
     common/avstrf/avsp,amplitude,base,dum2,v_max,channelv0,v_max1,channelv01,         &
                   v_max2,channelv02,v_max3,channelv03,geo,dumv(ivstrf-13)
     logical b_vs,b_amplitude,b_base,b_dum2,b_vmax,b_channelv0,                        &
             b_vmax1,b_channelv01,b_vmax2,b_channelv02,b_vmax3,b_channelv03,b_geo,b_dumv
     common/bvstrf/b_vs,b_amplitude,b_base,b_dum2,b_vmax,b_channelv0,b_vmax1,b_channelv01,&
                   b_vmax2,b_channelv02,b_vmax3,b_channelv03,b_geo,b_dumv(ivstrf-13)
     logical b_total 
!     real, dimension(-maxn:maxn)     :: yth
     real, dimension(:), allocatable :: velocity,yth 

     real    h,hi,y1,y2,y3,ff,dv 
     integer nchansp,nchansp2,i,istorth,j,k,mode_drive,nTHyy2 
     logical b_residual,b_abscissa 
     integer, parameter:: nl=6
     real   , dimension(nl):: vnl
      call mapsub(0,0,0) 

      nchansp=channels 
      mode_drive=drive_mode 
      do i=1,nchansp
         real_fr(mlpt+i)=0.0
      end do
      !penalty function required by subroutine fit.f90
 
      vnl=0.
      vnl(1)=v_max1; vnl(2)=channelv02; 
      vnl(3)=v_max2; vnl(4)=channelv02;
      vnl(5)=v_max3; vnl(6)=channelv03;
!     write(6,'(3(a,i6))') ' idumscrt=',idumscrt                        
!      write(6,'(a,f10.1)') ' ***strob_the2exp base=',base              
      ibmisfit=0 
      geo=0. 
                                                                        
       h=abs(channelv0/channels_vmax-1.0)*10 
      if(h.gt.1.0.or.nchansp.gt.maxn) then 
        write(infotext(3),'(a,i5,2(a,f8.2))')                           &
       'nchansp=',nchansp,' channels_vmax=',channels_vmax,              &
       ' channelv0=',channelv0                                       
                                                                        
        info=10*h 
        infotext(1)='<Mossbauer_stroboscopy>/global_pa/ch_vmax/ch_v0' 
        infotext(2)='**error** with one of the size of'//               &
                     ' channels,channels_vmax,channelv0'                        
        infotext(4)=' ' 
        return 
      end if 
                                                                        
      nchansp2=nchansp/2 
      istorTH=nstorTH+1 
                                                                        
      b_abscissa=b_vmax.or.b_channelv0.or.b_vmax1.or.b_channelv01 
      b_abscissa= b_abscissa.or.b_vmax2.or.b_channelv02.or.b_vmax3.or.b_channelv03
      dv=abs(v_max)/float(nTH) 

      i=(nTH/4+1)*5
      if(.not.allocated(yth))allocate(yth(-i:i))
                                                                        
      nTHyy2=nstorTHyy*2 
      call moss_strob(b_total,info)
      if(info.gt.0)return
 
      call drive_strobo(int(drive_mode),ibmisfit,nchansp,mvpt, &
                 mxpt,b_data, channels_vmax,channelv0,v_max, &
                 v_max1,channelv01,v_max2,channelv02,v_max3,channelv03,info)

      if(info.gt.0)return     
      if (.not.allocated(velocity))allocate(velocity(nchansp))
      do i=1,nchansp
       velocity(i)=real_fr(mvpt+i)
      end do

      goto (100,200,300),k1 
  100  continue 

       if(b_base.or.b_amplitude.or.b_abscissa.or.b_total) then 
        do j =0,nTH 
         yth(j) =amplitude*real(array_fr(istorTH+j))+base 
         yth(-j)=amplitude*aimag(array_fr(istorTH+j))+base 
        end do 
        do k=1,nTH/4 
         yth(-nTH-k)=yth(-nTH) 
         yth(nTH+k) =yth(nTH) 
        end do 
                                                                        
!       write(6,'(a,L5,f10.1,2i8)')'b_total,base,istorTH=',&              
!               b_total,base,istorTH,nTH                                   
!       write(6,'(6e15.6)') (yth(j),j=-nTH,nTH,16)                      
!       write(6,'(a,3f10.4,i5)')'velocity(j),v_max,dv,nchannel=', &      
!               amplitude,v_max,dv,nchansp                    
!       write(6,'(8i6)') (int(velocity(j)/dv),j=1,nchansp,64)           
                                                                        
        do j=1,nchansp 
         hi=velocity(j)/dv 
         i=int(hi) 
         y1=yth(i-1) 
         y2=yth(i) 
         y3=yth(i+1) 
         hi=hi-float(i) 
         ff=y2+0.5*hi*(y3-y1+hi*(y1+y3-2.*y2)) 
         thfk(j)=ff 
        end do 
        do j=1,nchansp 
         real_fr(nTHyy2+j)=thfk(j) 
        end do 
       else 
        do j=1,nchansp 
         thfk(j)=real_fr(nTHyy2+j) 
        end do
       end if 
!        write(6,'(a,2i8,4f15.4)')                                      
!     &  '*frame04 iarray_sp,nstorTHyy,R_oo=',                          
!     &  iarray_sp,nstorTHyy,array_fr(nstorTHyy+1),                     
!     &                               array_fr(nstorTHyy+2)             
       b_theory=.true. 
      return 
                                                                        
  200 continue 
      ib_df=0 
!     derivatives                                                       
!      base is analyticaly derived                                      
!      common/avstrf/avsp,amplitude,base,v_max,channelv0,v_max1,channelv01,  
!     &              v_max2,channelv02,v_max3,channelv03,geo,         
!     &              t_shift,Pxsi,Peta,Pzeta,graz_angle,dumv(idumv)     
                                                                        
      do j=1,nchansp 
       thfk(j)=real_fr(nTHyy2+j) 
      end do 
      b_residual=.not.(b_total.or.b_abscissa) 

      if(b_amplitude.and.b_residual) then 
!      write(6,'(a,f10.1)')'derivative Df/Damplitude -----amplitude=',amplitude
       do j=1,nchansp 
        dfdp(j)=(thfk(j)-base)/amplitude
       end do 
       kana=1 
       ib_df=1 
       goto 250 
      end if 

      if(b_base.and.b_residual) then 
!      write(6,'(a,f10.1)')  'derivative Df/dbase -----base=',base      
       do j=1,nchansp 
        dfdp(j)=1.
       end do 
       kana=1 
       ib_df=1 
       goto 250 
      end if 
                                                                        
      b_residual=.not.(b_total.or.b_base.or.b_amplitude.or.b_abscissa) 
      if(b_residual) goto 250 
                                                                        
      kana=0 
      do j =0,nTH 
       yth(j) =amplitude*real(array_fr(istorTH+j))+base 
       yth(-j)=amplitude*aimag(array_fr(istorTH+j))+base 
      end do 
      do k=1,nTH/4 
       yth(-nTH-k)=yth(-nTH) 
       yth(nTH+k) =yth(nTH) 
      end do 
                                                                        
      do j=1,nchansp 
       hi=velocity(j)/dv 
       i=int(hi) 
       y1=yth(i-1) 
       y2=yth(i) 
       y3=yth(i+1) 
       hi=hi-float(i) 
       ff=y2+0.5*hi*(y3-y1+hi*(y1+y3-2.*y2)) 
       dfdp(j)=(ff-thfk(j))/ddpk 
      end do   ! j=1,nchansp
      ib_df=1 
                                                                        
  250 continue 
!     write(6,'(4e15.6)') (dfdp(i),i=1,nchansp,32)                      
      return 
                                                                        
  300 continue 
      if(.not.b_theory) goto 310 
                                                                        
      call moss_strob(b_total,info) 
      do j =0,nTH 
       yth(j) =amplitude*real(array_fr(istorTH+j))+base
       yth(-j)=amplitude*aimag(array_fr(istorTH+j))+base 
      end do 
      do k=1,nTH/4 
       yth(-nTH-k)=yth(-nTH) 
       yth(nTH+k) =yth(nTH) 
      end do 
                                                                        
      do j=1,nchansp 
       hi=velocity(j)/dv 
       i=int(hi) 
       y1=yth(i-1) 
       y2=yth(i) 
       y3=yth(i+1) 
       hi=hi-float(i) 
       ff=y2+0.5*hi*(y3-y1+hi*(y1+y3-2.*y2)) 
       thfk(j)=ff 
      end do 
  310 return 
      END                                           
                                                                        
                                                                        
     subroutine moss_strob(b_total,info) 
     use glbconst, only: maxfr1,maxfr2,maxn,maxthss,maxscrd, &
                         icstrf,icfr1trf,icfr2trf,& 
                         ivstrf,ivfr1trf,ivfr2trf,ivsstrf,enwr,nwr
     use comthe, only:   array_fr,ifrcase
     implicit none 
                                                                       
     logical,intent(out):: b_total 
     integer,intent(out):: info

     integer       mstruct1,istruct1,istruct2,                        &
                   l1_get,l2_get,l3_get,l1_put,l2_put,l3_put,iaddr
     character(len=4) :: c1_prp,c2_prp,c3_prp
     common/struct/mstruct1,istruct1(0:maxfr1),istruct2(0:maxfr2),     &
                   l1_get(0:maxfr1),l2_get(maxfr2),l3_get(maxthss),    &
                   l1_put(0:maxfr1),l2_put(maxfr2),l3_put(maxthss),    &
                   c1_prp(0:maxfr1),c2_prp(maxfr2),c3_prp(maxthss),    &
                   iaddr(40)

      real          srw8
      integer       nstorTH,nstorTH1,idum4,nCC,ifrth,iflag_basis,lfwscat,   &
                    nTH,nsampling,ak8,iduma,idumb,idumc,ien    
      common/strLOC/nstorTH,nstorTH1,idum4(4),nTH,ifrth(5),iflag_basis,lfwscat, &
                    iduma,nCC,idumb,nsampling,ak8,idumc,srw8,ien(11)
                                                                       
!    a large array(1:maxn*256) is used in order to store information of
!    theory for each data set. This storage ensures that during        
!    simultanious fitting of different data sets by the same theory    
!    calculations are preserved which are not needed to be recalculated
!    for each iteration                                                
!                                                                      
!    c1_prp(0:maxfr1),c2_prp(maxfr2),c3_prp(maxthss) stores the propert
!    polarized ci_prp()(1:1)='T', non polarized ci_prp()(1:1)='F',     
!    li_get,li_put denote the pointer on array_fr                      
!    li_get points to the result of the subroutines called by          
!    get_theory(isubthe,info)                                          
!    li_put points to an array area requestet in framexx.              
!    results calculated by this main routine are stored here           
!    li belongs to the loop index lstructi.                            
!    l1_put(lstruct1) is the pointer for storage defined in framexx    
!    lstruct1.gt.0.and.(lstruct2+lstruct3).eq.0                        
!    l2_put(lstruct2) :lstruct2.gt.0.and.lstruct3.eq.0                 
!    l3_put(lstruct3) :lstruct3.gt.0                                   
!                                                                      
!    convention: the first number is stored on li_get+1, li_put+1      
                                                                       
!    !!! the 2nd variable in all common blocks of variables is         
!        an  intensity, which is used to calculate subsets (incoherent 
!        parts, subspectra, ...)                                       
                                                                       
     real          acsp,wavelength,uthick_w,uthick_A,unitchi,            &
                   channels,v_range,th_channels,                         &
                   th_sampling,drive_mode,channels_vmax,Tbunch,          &
                   flag_basis,flag_phase,strob_order,am_window,dumcs      
     common/acstrf/acsp,wavelength,uthick_w,uthick_A,unitchi,            &
                   channels,v_range,th_channels,th_sampling,             &
                   drive_mode,channels_vmax,Tbunch,flag_basis,flag_phase,&
                   strob_order,am_window,dumcs(icstrf-16)
                                                                       
     logical b_cs,b_dumc 
     common/bcstrf/b_cs,b_dumc(icstrf-1) 
                                                                       
     real    avsp,amplitude,base,dum2,v_max,channelv0,                      &
             v_max1,channelv01,v_max2,channelv02,v_max3,channelv03,dum1,     &
             ang_shift,dum0,Pxsi,Peta,Pzeta,PA0,PAxsi,PAeta,PAzeta,         &
             graz_angle,dumv                                           
     common/avstrf/avsp,amplitude,base,dum2,v_max,channelv0,                &
             v_max1,channelv01,v_max2,channelv02,v_max3,channelv03,dum1,     &
             ang_shift,dum0,Pxsi,Peta,Pzeta,PA0,PAxsi,PAeta,PAzeta,         &
             graz_angle,dumv(ivstrf-23)                                
     logical b_vs,b_amplitude,b_base,b_dum2,b_vmax,b_channelv0,             &
             b_vmax1,b_channelv01,b_vmax2,b_channelv02,b_vmax3,b_channelv03,&
             b_dum1,b_ang_shift,b_dum0,b_Pxsi,b_Peta,b_Pzeta,b_PA0,          &
             b_PAxsi,b_PAeta,b_PAzeta,b_graz_angle,b_dumv 
     common/bvstrf/b_vs,b_amplitude,b_base,b_dum2,b_vmax,b_channelv0,       &
             b_vmax1,b_channelv01,b_vmax2,b_channelv02,b_vmax3,b_channelv03,&
             b_dum1,b_ang_shift,b_dum0,b_Pxsi,b_Peta,b_Pzeta,b_PA0,          &
             b_PAxsi,b_PAeta,b_PAzeta,b_graz_angle,b_dumv(ivstrf-23)
                                                                       
                                                                       
                                                                       
     real            cfr1trf 
     common/acfr1trf/cfr1trf(ivfr1trf) 
     real            avfr1,pvfr1,HextA,thetaHeA,phiHeA,                &
                     phiA,thetaA,psiA,dumvfr1                          
     common/avfr1trf/avfr1,pvfr1,HextA,thetaHeA,phiHeA,                &
                     phiA,thetaA,psiA,dumvfr1(ivfr1trf-7)              
     logical         b_vfr1,b_pvfr1,b_HextA,b_thetaHeA,b_phiHeA,       &
                     b_phiA,b_thetaA,b_psiA,b_dumvfr1                  
     common/bvfr1trf/b_vfr1,b_pvfr1,b_HextA,b_thetaHeA,b_phiHeA,       &
                     b_phiA,b_thetaA,b_psiA,b_dumvfr1(ivfr1trf-7)      
                                                                       
     real            acfr2,rep_grp,dumcfr2 
     common/acfr2trf/acfr2,rep_grp,dumcfr2(icfr2trf-2) 
     logical         b_cfr2,b_rep_grp,b_dumcfr2 
     common/bcfr2trf/b_cfr2,b_rep_grp,b_dumcfr2(icfr2trf-2) 
                                                                       
     real            avfr2,thicklay,re_chilay,ai_chilay,roughness,     &
                     dumvfr2                                           
     common/avfr2trf/avfr2,thicklay,re_chilay,ai_chilay,roughness,     &
                     dumvfr2(ivfr2trf-5)                               
     logical         b_vfr2,b_thicklay,b_re_chilay,b_ai_chilay,        &
                     b_roughness,b_dumvfr2                             
     common/bvfr2trf/b_vfr2,b_thicklay,b_re_chilay,b_ai_chilay,        &
                     b_roughness,b_dumvfr2(ivfr2trf-5)                 
                                                                       
     real             vsstrf,sftr 
     common/avsstrf/ vsstrf(ivsstrf) 
     logical          b_vsstrf,b_sftr 
     common/bvsstrf/ b_vsstrf(ivsstrf) 
                                                                       
     logical  b_lay,b_absorber,b_isgt1                              
                                                                       
     real,dimension(maxfr1) :: winc
     real,dimension(maxthss):: wi
     real,dimension(maxfr2) :: vkd 
     integer   l3p 
     integer,dimension(maxfr1):: l1_pos,l1p
     integer,dimension(maxfr2):: l2p
     logical,dimension(maxfr2):: b_sites,b_chilay,b_chilaynz 
 
     real,   dimension(maxfr2)    :: thicklayer,roughlay
     integer,dimension(maxfr2)    :: iposlay,laysites
     integer,dimension(2*maxfr2+2):: laystr
                                                                       
     complex(kind=8),dimension(4) :: daa,dbb,dcc 

     complex,dimension(4,maxfr2)  :: aam
     complex,dimension(4)         :: aa,bb,dd,cc,rho,Arho
     complex,dimension(2,2,maxfr2):: Cmatrix
     complex,dimension(maxfr2)    :: chilay
     complex,dimension(2,2)       :: Rmatrix,trv,TSPCA,sph                 
                                                                       
     complex hc,hcc,app,apm,amp,R_oo,                     &
             swin,swinp,swinm,satraceh,traceh,satrexp
     complex, dimension(:), allocatable :: carray
     real,    dimension(:), allocatable :: yth


! *** Definitions of structure variables L. Deak 09.10.2002 ***        
     integer POINTER,GRP,MAXGRP
     integer,dimension(0:2*maxfr2):: GEOSTR 
     logical b_first,b_frcn,b_frcr,b_frci,b_frab,b_window
! ***                                                       ***        
!    lfwscat=1   ! forward forward scattering geometry                   
!    lfwscat=0   ! grazing incidence                                    
!------------------------------------------------------------------    
!     write(6,'(a,2i4)')' th_strobmoss ifrcase,flag_phase=',           
     real    grazing,vector_k,sing,eta,omega,h1,h0,hp,hm,start,finish
     real,parameter :: sqrt2r=0.70710678,rad=0.0174533,pi=3.14159          
     integer ipos,lstruct1,lstruct2,lstruct3,isubthe,                  &
             iset,is_a,is_e,j_s,n_vm,n_er,n_v,n_s,n_ch,                &
             i,j,ji,n,nn,j1,j2,j3,j4,i4,nomega,nstrob_order,           &
             k,l,ll,lav,lay,mlay,layb,laye,isctp,iflag_phase,          &
             npos,lays,mlays                
      complex, parameter:: hc1=cmplx(1.,0.), hci=cmplx(0.,1.),hc0=cmplx(0.,0.)

      call cpu_time(start) 
      n=8*(nCC+1)
      allocate(yth(-nTH:nTH), carray(n))
      b_window=am_window.eq.0.0     ! case1 time window/boxcar function

      isubthe=1 
      call get_theory(isubthe,info) 
      b_total=isubthe.gt.0.or.b_Pxsi.or.b_Peta.or.b_Pzeta.or.          &
                     b_PA0.or.b_PAxsi.or.b_PAeta.or.b_PAzeta

      iflag_phase=flag_phase 
      b_frcn=iflag_phase.eq.0 
      b_frcr=iflag_phase.eq.1 
      b_frci=iflag_phase.eq.-1 
      b_frab=abs(iflag_phase).gt.1 
                                                                       
                                                                        
      nstrob_order=strob_order 
      if(.not.b_window)nstrob_order=strob_order*am_window
                                                           
      srw8=v_max/float(nTH)/8. 
      
      omega=wavelength/Tbunch*100./srw8/float(nsampling) 
      srw8=abs(srw8) 
      ak8=v_range/srw8 
      vector_k=2.0*pi/wavelength 
                                                                        
!     write(6,'(a,3f10.2)') ' Poincare =', Pxsi,Peta, Pzeta             
!      (11 12))     fortran  ( 1 3 )                                    
!      (21 22)               ( 2 4 )                                    
!                       effi ( 1 3 )                                    
!                            ( 4 2 )                                    
!     !!!!! 2x2 matrices on array_fr use always effi notation !!!!      
!     Fortran notation                                                  
      rho(1)=cmplx(1.+Pzeta,0.)/2. 
      rho(3)=cmplx(Pxsi,-Peta)/2. 
      rho(2)=cmplx(Pxsi,Peta)/2. 
      rho(4)=cmplx(1.-Pzeta,0.)/2. 
!      write(6,'(a,8f8.2)') ' rho=', (rho(k),k=1,4)                                                                         
      Arho(1)=cmplx(PA0+PAzeta,0.)/2. 
      Arho(3)=cmplx(PAxsi,-PAeta)/2. 
      Arho(2)=cmplx(PAxsi,PAeta)/2. 
      Arho(4)=cmplx(PA0-PAzeta,0.)/2. 
!      write(6,'(a,8f8.2)') ' Arho=', (Arho(k),k=1,4)                   
!---------------------------------------------------------------        
      iflag_basis=flag_basis 
      if(iflag_basis.eq.0) then 
!      linear polarization basis                                        
       TSPCA(1,1)=cmplx( sqrt2r,0.) 
       TSPCA(1,2)=cmplx(-sqrt2r,0.) 
       TSPCA(2,1)=cmplx(0.,-sqrt2r) 
       TSPCA(2,2)=cmplx(0.,-sqrt2r) 
                                                                        
       sph(1,1)=rho(1) 
       sph(1,2)=rho(3) 
       sph(2,1)=rho(2) 
       sph(2,2)=rho(4) 
       do i=1,2 
       do k=1,2 
        hc=(0.,0.) 
        do j=1,2 
        do l=1,2 
         hc=hc+TSPCA(i,l)*sph(l,j)*conjg(TSPCA(k,j)) 
        end do 
        end do 
        trv(i,k)=hc 
       end do 
       end do 
       rho(1)=trv(1,1) 
       rho(3)=trv(1,2) 
       rho(2)=trv(2,1) 
       rho(4)=trv(2,2) 

       sph(1,1)=Arho(1) 
       sph(1,2)=Arho(3) 
       sph(2,1)=Arho(2) 
       sph(2,2)=Arho(4) 
       do i=1,2 
       do k=1,2 
        hc=(0.,0.) 
        do j=1,2 
        do l=1,2 
         hc=hc+TSPCA(i,l)*sph(l,j)*conjg(TSPCA(k,j)) 
        end do 
        end do 
        trv(i,k)=hc 
       end do 
       end do 
       Arho(1)=trv(1,1) 
       Arho(3)=trv(1,2) 
       Arho(2)=trv(2,1) 
       Arho(4)=trv(2,2) 
      end if 
!--------------------------------------------------------------         
      lstruct1=0 
 1000 continue 
      b_absorber=.false. 
                                                                        
      lstruct1=lstruct1+1 
      if(lstruct1.gt.mstruct1) goto 2500 
!      different devices have differen values for l_fwscat !!         
       lfwscat=1 
       if(lstruct1.ge.2.and.ifrcase.eq.2)lfwscat=0 
          !  stroboscopic reflection spectrum, optic is called 
                                                                        
       l1p(lstruct1)=2 
       if(c1_prp(lstruct1)(1:1).eq.'T') l1p(lstruct1)=8 
       l1_pos(lstruct1)=l1_put(lstruct1)  ! 0..2*nCC*4 
                                                                        
       call mapsub(lstruct1,0,0) 
       isubthe=1 
       call get_theory(isubthe,info) 
       b_total= b_total.or.(isubthe.gt.0) 
                                                                        
       b_absorber=b_absorber.or.b_vfr1.or.b_pvfr1 
                                                                        
       winc(lstruct1)=pvfr1 
!      write(6,'(a,i4,a,g12.4)')'moss_strob case=',lfwscat,               
!     &     ' graz_angle=',graz_angle                                   
                                                                        
       if(lfwscat.eq.0) then 
        b_absorber=b_absorber.or.b_graz_angle.or.b_ang_shift 
        grazing=0.001*(graz_angle+ang_shift) 
!       thetaA=(pi/2.-grazing)/rad  new transformationmn                
        sing=sin(grazing) 
       end if 

       layb=istruct1(lstruct1-1) 
       laye=istruct1(lstruct1) 
       do lstruct2=istruct1(lstruct1-1)+1,istruct1(lstruct1) 
!      ------------------------------------------------------------     
!      layer lay1,2 ... mlay of  absorber                               
!      ---------------------------------------------------------------  
!      The Lorentz curve of the absorber in npq is 1 at resonance so tha
!      for a singl line the exponent at resonance is just -teff (effecti
!      thickness). areaLor ~ pi*ga/2 ( area -+ infinity), ga/2=hga/2/srw
!      normalizes ( to area 1) the Lorentz curve.                       
!                                                                       
!      Sums of absorber curves at the positions Eagd.                   
!                                                                       
!      ----0--------------------------naLG                              
!       naLG--------------------------0-------------        srw8        
!        nCC------------0------------nCC                    srw         
                                                                 
        lay=lstruct2-layb 
                                                                        
        call mapsub(lstruct1,lstruct2,0) 
!        write(6,'(a,5g12.4)')' synmos r,i_chi,roughness,thicklay=',    
!     &        re_chilay,ai_chilay,roughness,thicklay                   
!    common/avfr1trf/avfr1,pvfr1,HextA,thetaHeA,phiHeA,                &
!                     phiA,thetaA,psiA,dumvfr1(ivfr1trf-7)                                                                             
!    common/bvfr1trf/b_vfr1,b_pvfr1,b_HextA,b_thetaHeA,b_phiHeA,       &
!                     b_phiA,b_thetaA,b_psiA,b_dumvfr1(ivfr1trf-7)      

         dumvfr2(2)=HextA   
         dumvfr2(3)=thetaHeA 
         dumvfr2(4)=phiHeA 
         dumvfr2(5)=phiA 
         dumvfr2(6)=thetaA 
         dumvfr2(7)=psiA 

        b_dumvfr2(2)=b_HextA  
        b_dumvfr2(3)=b_thetaHeA 
        b_dumvfr2(4)=b_phiHeA 
        b_dumvfr2(5)=b_phiA 
        b_dumvfr2(6)=b_thetaA 
        b_dumvfr2(7)=b_psiA 
        b_lay=b_vfr1

        thicklayer(lay)=thicklay 
        b_absorber=b_absorber.or.b_vfr2.or.b_thicklay 
        isubthe=1                      ! frame
        call get_theory(isubthe,info) 
        b_total= b_total.or.(isubthe.gt.0).or.b_absorber 
                                                                        
        if(lfwscat.eq.0) then 
         if(abs(re_chilay)+abs(ai_chilay).gt.1.0e-10) then 
          chilay(lay)=cmplx(re_chilay,ai_chilay)*unitchi 
          b_chilaynz(lay)=.true. 
         else 
          chilay(lay)=(0.,0.) 
          b_chilaynz(lay)=.false. 
         end if 
         b_chilay(lay)=b_vfr2.or.b_thicklay 
         i=rep_grp 
         laystr(2*lay-1)=i 
         laystr(2*lay)=10*(rep_grp-float(i)+.01) 
         roughlay(lay)=roughness 
        end if 
                                                                        
!       polarized layer                                                 
        l2p(lstruct2)=2 
        if(c2_prp(lstruct2)(1:1).eq.'T') l2p(lstruct2)=8 
                                                                        
        b_isgt1=.false. 
        if(istruct2(lstruct2)-istruct2(lstruct2-1).gt.1)then 
         b_isgt1=.true. 
!        if more than one site within layer lstruct2                    
!        than the sum of the site amplitudes times wi is stored         
!        at l2_put(lstruct2)                                            
!        (if one site the amplitudes at l3_get(lstruct3) are taken      
!         and multiplied by wi)                                         
                                                                        
         iposlay(lay)=l2_put(lstruct2) 
         do i=1,l2p(lstruct2)*(nCC+1)
          carray(i)=(0.,0.) 
         end do 
        end if 
                                                                        
        b_sites(lay)=.false. 
        b_lay=.false. 
                                                                        
        do lstruct3=istruct2(lstruct2-1)+1,istruct2(lstruct2) 

         iset=lstruct3 
         l3p=2 
         if(c3_prp(lstruct3)(1:1).eq.'T') l3p=8 
                                                                        
         iset=lstruct3 
         b_sites(lay)=.true. 
                                                                        
         call mapsub(lstruct1,lstruct2,lstruct3) 
         sftr=vsstrf(2) 
         b_sftr= b_vsstrf(2) 

                                                                        
         wi(iset)=sftr*uthick_w 
         if(ifrcase.eq.2.and.lstruct1.gt.1)wi(iset)=sftr*uthick_A 
                                                                        
                                                                        
!         write(6,'(a,L4,i4,5g10.3)') &                                     
!        'moss_strob lay,wi(iset),sftr,uthick_w,uthick_A=',   &         
!          b_vsstrf(1),lay,wi(iset),sftr,uthick_w,uthick_A                          
                                                                        
         if(lfwscat.eq.0) then 
          if(b_vsstrf(1).and.enwr(6).eq.'nynn') then 
           if(wi(iset).gt.sing/20..or.sing.gt.0.05)                    &
          write(nwr,'(2(a,g8.1),a)')                                   &
          '** f*unit_thickness=',wi(iset),' << sin(th)=',sing,' << 1 ?'
          end if 
         end if 
                                                                        
         isubthe=0 
         call get_theory(isubthe,info) 
         b_lay=b_lay.or.b_sftr.or.b_vsstrf(1) 
         b_absorber=b_absorber.or.b_lay.or.(isubthe.gt.0) 
                                                                        
         if(b_isgt1) then 
          isctp=l3_get(lstruct3) 
!         three possibilities l2p=l3p=8, 2, and l2p=8,l3p=2             
!         !! the first two cases have different array sizes             
                                                                        
          if(l2p(lstruct2).eq.l3p) then 
           do n=0,nCC 
            ll=l3p*n 
            nn=isctp+ll 
             do k=1,l3p 
             carray(ll+k)=carray(ll+k)+ array_fr(nn+k)*wi(iset) 
             end do 
            end do
           else !  if(l2p(lstruct2).eq.4*l3p) then  
           do n=0,nCC 
            nn=isctp+l3p*n 
            ll=l2p(lstruct2)*n 
            hc=array_fr(nn+1)*wi(iset) 
            carray(ll+1)=carray(ll+1)+ hc 
            carray(ll+4)=carray(ll+4)+ hc 
            hc=array_fr(nn+2)*wi(iset) 
            carray(ll+5)=carray(ll+5)+ hc 
            carray(ll+8)=carray(ll+8)+ hc 
           end do 
          end if 
         else 
          iposlay(lay)=l3_get(lstruct3) 
          
         end if 
!        write(6,'(a)') 'moss_strob  carray(1..)='                      
        end do     ! lstruct3 
                                                                        
        if(b_isgt1) then 
!        write(6,'(a,4i8)')'b_isgt1.and.b_lay:',lstruct2,iposlay(lay),l2p(lstruct2)*(nCC+1)
         do i=1,l2p(lstruct2)*(nCC+1) 
          array_fr(iposlay(lay)+i)=carray(i) 
         end do 
        end if 
       end do  ! lstruct2 

       mlay=lay 
       laystr(2*lay+2)=-1 
       laystr(2*lay+1)=0 
      b_total=b_total.or.b_absorber 
      if(.not.b_absorber) goto 2000 
                                                                        
!-----------------------------------------------------------------------
!     here the matrix R for the layers are given :                      
!     satrlay(n,lay)*1 +  (sapplay(n,lay)  sap(n,lay))                  
!                         (samplay(n,lay) -sapplay(n,lay))              
!                                                                       
!     calculation of the transmittivity matrix T for all channels n     
!     by   T = exp(iR)*rho*exp(-iR+)                                    
                                                                        
                                                                        
      do lstruct2=layb+1,laye 
       lay=lstruct2-layb 
       is_a=istruct2(lstruct2-1)+1 
       is_e=istruct2(lstruct2) 
!       if(is_e.ge.is_a)                                                
!     &  write(6,'(a,i4,2g10.3)')'synmos lay,wi(is_a),thicklayer',      
!     &   lay,wi(is_a),thicklayer(lay)                                  
      end do 
                                                                        
      do k=1,mlay 
       laysites(k)=0 
      end do 
      if(lfwscat.eq.1) then 
       mlays=0 
       do lstruct2=layb+1,laye 
        lay=lstruct2-layb 
        is_a=istruct2(lstruct2-1)+1 
        is_e=istruct2(lstruct2) 
        if(b_sites(lay)) then 
         mlays=mlays+1 
         l2p(mlays)=l2p(lay) 
         iposlay(mlays)=iposlay(lay) 
         if(is_e.gt.is_a) thicklayer(mlays)=thicklayer(lay) 
         if(is_e.eq.is_a) thicklayer(mlays)=thicklayer(lay)*wi(is_a)
!         write(6,'(a,i4,2f10.3)')'synmos lay,wi(is_a),thicklayer',   &     
!                                  lay,wi(is_a),thicklayer(lay)
        end if 
       end do 
      else 
       mlays=mlay-1 
       do lstruct2=layb+1,laye 
        lay=lstruct2-layb 
        is_a=istruct2(lstruct2-1)+1 
        is_e=istruct2(lstruct2) 
        vkd(lay)=vector_k*thicklayer(lay) 
        if(is_e.lt.is_a) then 
         thicklayer(lay)=0. 
        else 
         if(is_e.gt.is_a) thicklayer(lay)=1./vector_k 
         if(is_e.eq.is_a) thicklayer(lay)=wi(is_a)/vector_k 
        end if 
       end do 
      end if  
                                                                        
      
       if(lfwscat.eq.0) then 
!       ****************************************************************
!       We reconstruct the layer strucrure from the laystr variabale.   
!       ELEMENT(GRP) is the number of layers in group GRP.              
!        *** L. Deak, 09.10.2002    ***                                 
        GRP         =0 
        GEOSTR(GRP) =0 
        MAXGRP      =0 
!                                                                       
        b_first     =.true. 
        POINTER=1 
        do lay=2,2*mlay,2 
         if(b_first)then 
          if(GRP.eq.laystr(lay))then 
           GEOSTR(2*GRP)   =GEOSTR(2*GRP)+1 
          else 
           POINTER         =POINTER+GEOSTR(2*GRP) 
           GRP             =GRP+1 
           MAXGRP          =GRP 
           GEOSTR(2*GRP)   =1 
           GEOSTR(2*GRP-1) =laystr(lay-1) 
          end if 
                                                                        
          if(vkd(lay/2).lt.0.) then 
           b_first         =.false. 
           vkd(lay/2)      =1.e30 
           GEOSTR(2*GRP-1) =1 
          end if  ! VKD < 0  
         end if   ! b_first 
        end do 
!       ****************************************************************
!       write(6,'(a,10i8)')'moss_strob iposlay(1.)=',(iposlay(i),i=1,mla
!       do lstruct2=layb+1,laye                                         
!        lay=lstruct2-layb                                              
!        write(6,'(i4,2x,L3,a,2g12.4,2(a,g12.4))')                      
!     & lay,b_sites(lay),' chilay=',chilay(lay)/unitchi,' vkd=',vkd(lay)
!     & ' thicklayer(lay)=',thicklayer(lay)                             
!       end do                                                          
!       write(6,'(20i4)')(GEOSTR(i),i=1,20)                             
                                                                        
        do lay=1,mlay 
         Cmatrix(1,1,lay)=hc0 
         Cmatrix(2,2,lay)=hc0 
         Cmatrix(1,2,lay)=hc0 
         Cmatrix(2,1,lay)=hc0 
        end do 
        call optic(mlay,b_sites,MAXGRP,GEOSTR,grazing,             &
                     Cmatrix,chilay,roughlay,vkd,Rmatrix)              
        R_oo=Rmatrix(1,1) 
        array_fr(nstorTH1+2)=R_oo 
       else 
        R_oo=hc1 
        array_fr(nstorTH1+1)=R_oo 
       end if 
       ipos=l1_pos(lstruct1) 
                                                                        
!      construction of the input for general optics program             
!      here the matrix R for the layers are given :                     
!      satrlay(n,lay)*1 +  (sapplay(n,lay)  sapmlay(n,lay))             
!                          (samplay(n,lay) -sapplay(n,lay))             
       if(mlay.eq.1) then 
        if(lfwscat.eq.1) then 
                                                                        
         if(l1p(lstruct1).eq.8) then 
          do n=-nCC,nCC 
           npos=8*abs(n) 
           if(n.lt.0)npos=npos+4 
           nn=iposlay(1)+npos 
           satraceh=array_fr(nn+1)*thicklayer(1) 
           app=array_fr(nn+2)*thicklayer(1) 
           apm=array_fr(nn+3)*thicklayer(1) 
           amp=array_fr(nn+4)*thicklayer(1) 
           call expo_22(app,apm,amp,dd)
!           write(11,'(i6,8g10.2)')nn,dd
 
           satrexp=exp(satraceh*hci) 
!          multiplication of the matrices exp(iR(mlay))*...*exp(iR(1))  
!          laystr(rep,grp,rep,grp,.....)                                
!          dd Fortran mode                                              
           nn=(nCC+n)*4 
           array_fr(ipos+nn+1)=dd(1)*satrexp 
           array_fr(ipos+nn+4)=dd(2)*satrexp 
           array_fr(ipos+nn+3)=dd(3)*satrexp 
           array_fr(ipos+nn+2)=dd(4)*satrexp 
          end do 
!           write(11,'(i6,8g10.2)')nn,dd
         else     ! if non polarized  
                                                                        
          do n=-nCC,0 
           nn=iposlay(1)-2*n+1 
!          if(abs(n).lt.5)write(6,'(a2,i3,i8,8f12.4)')'##',n,nn,array_fr(nn+1)                       
           satraceh=array_fr(nn+1)*thicklayer(1) 
           carray(nCC+n+1)=exp(satraceh*hci) 
          end do 
          do n=1,nCC 
           nn=iposlay(1)+2*n 
           satraceh=array_fr(nn+1)*thicklayer(1) 
           carray(nCC+n+1)=exp(satraceh*hci) 
          end do 
          do nn=0,2*nCC 
           array_fr(ipos+nn+1)=carray(nn+1) 
          end do 
         end if  ! l1p(lstruct1).eq.8 
        else if(lfwscat.eq.0) then 
!        multilayer can only be calculated with the full polarization    
!        matrix; only 2x2 matrix theories are connected to be called    
         do n=-nCC,nCC 
          npos=8*abs(n) 
          if(n.lt.0)npos=npos+4 
                                                                        
          nn=iposlay(1)+npos 
          Cmatrix(1,1,1)= array_fr(nn+1)*thicklayer(1) 
          Cmatrix(2,2,1)= array_fr(nn+2)*thicklayer(1) 
          Cmatrix(1,2,1)= array_fr(nn+3)*thicklayer(1) 
          Cmatrix(2,1,1)= array_fr(nn+4)*thicklayer(1) 
                                                                        
          call optic(mlay,b_sites,MAXGRP,GEOSTR,grazing,           &
                     Cmatrix,chilay,roughlay,vkd,Rmatrix)              
                                                                        
          nn=(nCC+n)*4 
          carray(nn+1)=Rmatrix(1,1) 
          carray(nn+3)=Rmatrix(1,2) 
          carray(nn+2)=Rmatrix(2,1) 
          carray(nn+4)=Rmatrix(2,2) 
         end do 
         do nn=0,8*nCC,4 
          array_fr(ipos+nn+1)=carray(nn+1) 
          array_fr(ipos+nn+4)=carray(nn+2) 
          array_fr(ipos+nn+3)=carray(nn+3) 
          array_fr(ipos+nn+2)=carray(nn+4) 
         end do 
!         if(abs(n).lt.5)write(6,'(a2,i3,i8,8f12.4)') &                
!                          '##',n,nn,array_fr(ipos+nn+1)                  
        end if ! lfwscat.eq.0  
       else    ! mlay > 1  
                                                                       
        if(lfwscat.eq.1) then 
!        different layers may be polarized or non polarized.               
!        if l1p(lstruct1)=2 then all layers are non polarized           
!        the dd matrix of a non polarized scatterer is the unit matrix  
!        (dd(1)=(1,0),dd(2)=(0,0),dd(3)=(0,0),dd(4)=(1,0))              
         lav=0 
         do lays=1,mlays 
          lav=lav+l2p(lays) 
         end do 
         lav=lav/mlays 
                                                                        
         if(l1p(lstruct1).eq.lav) then 
!         write(6,'(a,2i5)')'l1p(lstruct1)=',lstruct1,l1p(lstruct1)
          if(l1p(lstruct1).eq.2)then 
           do n=-nCC,nCC 
            npos=2*abs(n) 
            if(n.lt.0)npos=npos+1 
            satraceh=cmplx(0.,0.) 
            do lays=1,mlays 
             nn=iposlay(lays)+npos 
             satraceh=satraceh+array_fr(nn+1)*thicklayer(lays) 
            end do   ! lays=1,mlays 
            satrexp=exp(satraceh*hci) 
!           multiplication of the matrices exp(iR(mlay))*...*exp(iR(1)) 
!           laystr(rep,grp,rep,grp,.....)                               
            nn=(nCC+n) 
            array_fr(ipos+nn+1)=satrexp 
           end do 
          else if(l1p(lstruct1).eq.8) then 
           do n=-nCC,nCC 
            npos=8*abs(n) 
            if(n.lt.0)npos=npos+4 
                                                                        
            satraceh=cmplx(0.,0.) 
            do lays=1,mlays 
             nn=iposlay(lays)+npos 
             satraceh=satraceh+array_fr(nn+1)*thicklayer(lays) 
             app=array_fr(nn+2)*thicklayer(lays) 
             apm=array_fr(nn+3)*thicklayer(lays) 
             amp=array_fr(nn+4)*thicklayer(lays) 
             call expo_22(app,apm,amp,dd) 
             do k=1,4 
              aam(k,lays)=dd(k) 
              end do 
            end do   ! lays=1,mlays 
            satrexp=exp(satraceh*hci) 

!           multiplication of the matrices exp(iR(mlay))*...*exp(iR(1)) 
!           laystr(rep,grp,rep,grp,.....)                               
                                                                        
            if(mlays.gt.1) then 
             do k=1,4 
              dcc(k)=dcmplx(aam(k,mlays))
             end do 
                                                                        
             do lays=mlays-1,1,-1 
              do k=1,4 
               daa(k)=dcc(k) 
               dbb(k)=dcmplx(aam(k,lays)) 
              end do 
              call matproD(daa,dbb,dcc) 
             end do 
             do k=1,4 
              dd(k)=dcc(k) 
             end do 
            end if 
                                                                        
            nn=(nCC+n)*4 
            array_fr(ipos+nn+1)=dd(1)*satrexp 
            array_fr(ipos+nn+4)=dd(2)*satrexp 
            array_fr(ipos+nn+3)=dd(3)*satrexp 
            array_fr(ipos+nn+2)=dd(4)*satrexp 
           end do 

          end if ! l1p(lstruct1)=l .eq.8 
         else     ! if l1p(lstruct1).ne.lav    
!         at least one of l2p(lays) is equal 2                          

          do n=-nCC,nCC 
           satraceh=cmplx(0.,0.) 
           do lays=1,mlays 
            npos=8*abs(n) 
            if(n.lt.0)npos=npos+8/2 
            nn=iposlay(lays)+npos 
            satraceh=satraceh+array_fr(nn+1)*thicklayer(lays) 
            app=array_fr(nn+2)*thicklayer(lays) 
            apm=array_fr(nn+3)*thicklayer(lays) 
            amp=array_fr(nn+4)*thicklayer(lays) 
!           if(abs(n).lt.2)write(6,'(2x,2i8,2x,8g13.6)')lay,nn+1,  &          
!                    array_fr(nn+1),array_fr(nn+3),thicklayer(lays) 
            call expo_22(app,apm,amp,dd) 
            do k=1,4 
             aam(k,lays)=dd(k) 
            end do 
           end do   ! lays=1,mlays 
           satrexp=exp(satraceh*hci) 
!          multiplication of the matrices exp(iR(mlay))*...*exp(iR(1))  
!          laystr(rep,grp,rep,grp,.....)                                
                                                                        
           if(mlays.gt.1) then 
            do k=1,4 
             dcc(k)=dcmplx(aam(k,mlays)) 
            end do 
                                                                        
            do lays=mlays-1,1,-1 
             do k=1,4 
              daa(k)=dcc(k) 
              dbb(k)=dcmplx(aam(k,lays)) 
             end do 
             call matproD(daa,dbb,dcc) 
            end do 
            do k=1,4 
             dd(k)=dcc(k) 
            end do 
           end if 
                                                                        
           nn=(nCC+n)*4 
           array_fr(ipos+nn+1)=dd(1)*satrexp 
           array_fr(ipos+nn+4)=dd(2)*satrexp 
           array_fr(ipos+nn+3)=dd(3)*satrexp 
           array_fr(ipos+nn+2)=dd(4)*satrexp 
          end do 
         end if  ! l1p(lstruct1)#l=8 

        else if(lfwscat.eq.0) then 
                                                                        
         do n=-nCC,nCC 
          npos=8*abs(n) 
          if(n.lt.0)npos=npos+4 
                                                                        
          do lay=1,mlay 
           if(b_sites(lay)) then 
            nn=iposlay(lay)+npos 
            Cmatrix(1,1,lay)= array_fr(nn+1)*thicklayer(lay) 
            Cmatrix(2,2,lay)= array_fr(nn+2)*thicklayer(lay) 
            Cmatrix(1,2,lay)= array_fr(nn+3)*thicklayer(lay) 
            Cmatrix(2,1,lay)= array_fr(nn+4)*thicklayer(lay) 
!            if(abs(n).lt.2)write(6,'(2x,2i8,2x,8g13.6)')lay,nn+1,  &          
!                    array_fr(nn+1),array_fr(nn+3),thicklayer(lay) 
           end if 
          end do   ! lay=1,mlay 
                                                                        
          call optic(mlay,b_sites,MAXGRP,GEOSTR,grazing,           &
                     Cmatrix,chilay,roughlay,vkd,Rmatrix)              
                                                                        
          nn=(nCC+n)*4 
          array_fr(ipos+nn+1)=Rmatrix(1,1) 
          array_fr(ipos+nn+3)=Rmatrix(1,2) 
          array_fr(ipos+nn+4)=Rmatrix(2,1) 
          array_fr(ipos+nn+2)=Rmatrix(2,2) 
!         if(abs(n).lt.5)write(6,'(2x,2i8,2x,8g13.6)')ipos,nn,Rmatrix
         end do 
        end if  ! lfwscat.eq.0  
       end if  ! mlay.eq.1
                                                                        
 2000 continue 
!      write(6,'(a,2i6)')'synmos lstruct1,mstruct1=',lstruct1,mstruct1  
      if(mstruct1.ge.2) goto 1000 
                                                                 
!     address of reference absorber                                     
!     ipos_mstruct1(1)+n, n=1,...,4*(2*nCC+1)                           
 2500 continue 
      if(.not.b_total) return 
                                                                        
      j_s=8/nsampling 
      n_vm=nTH*j_s 
      n_er=ak8/8*j_s 
!      write(6,'(a,8i6)')' l1_put(1),l1_put(2),nCC,n_er,n_vm,nomega=',  &
!                          l1_put(1),l1_put(2),nCC,n_er,n_vm,nomega  
                                                                        
      do n_s=0,nstrob_order 
        swin=array_fr(l1_get(0)+1+abs(n_s)) 
        if(enwr(15).eq.'swin')print '(i5,2x,2f12.4)', n_s,swin 
      end do
      R_oo=array_fr(nstorTH1+1)  
      if(lfwscat.eq.0)R_oo=array_fr(nstorTH1+2)                     
!      write(6,'(a,2f10.3)')'moss_strob R_oo=',R_oo                     
!      write(6,'(a,5i8)')'moss_strob l1_pos(1..=',l1_pos(1),l1_pos(2)   
!      write(6,'(a,5i8)')'moss_strob l1p(1,2),lfwscat=',l1p(1),l1p(2)
!      write(6,'(a,5i8)')'moss_strob nstrob_order,n_vm,j_s,nCC=',  &
!                                  nstrob_order,n_vm,j_s,nCC    
!      write(6,'(a,5L3)')'b_frcn,b_frcr,b_frci=',b_frcn,b_frcr,b_frci
!      write(6,'(a,4i4)')'lfwscat=',lfwscat

      eta=1./6. 
!  the array yth is calculated for the different cases of polarisation:      

      if(l1p(1)+l1p(2).eq.16) then 
!       write(6,'(a)') ' l1p(1)+l1p(2).eq.16'                           
       do n_v=-n_vm,n_vm,j_s 
        k=n_v/j_s 
        h0=0. ! 
        hp=0. ! real summation
        hm=0. !
        do n_s=0,nstrob_order 
         nomega=int(float(n_s)*omega/2.)

         swin=array_fr(l1_get(0)+1+abs(n_s)) 
         if(n_s.lt.0) swin=conjg(swin) 
         if(b_window)then
          swinp=swin*exp(-hci*2.*pi*n_s*eta) 
          swinm=swin*exp(hci*2.*pi*n_s*eta) 
         else
          swinp=array_fr(l1_get(0)+2+nstrob_order+abs(n_s)) 
          swinm=array_fr(l1_get(0)+3+2*nstrob_order+abs(n_s)) 
         end if

         h1=real(conjg(swin)*swin) 
         if(h1.gt.1.e-6)then 
!         write(6,'(I4,2F12.5)')n_s,abs(swin)                           
!         integration of Int;(-n_ch,n_ch)                               
!          n_ch=n_er+n_vm+nomega                                  
                                                                        
          j1=l1_pos(1)+4*(nCC+n_v+nomega) 
          j2=l1_pos(2)+4*(nCC+nomega) 
                                                                        
          j3=l1_pos(1)+4*(nCC+n_v-nomega) 
          j4=l1_pos(2)+4*(nCC-nomega) 
                                                                        
          n_ch=n_er+n_vm+nomega 
          hcc=hc0 
          do i=-n_ch,n_ch 
           i4=4*i 
           j=j1+i4 
           bb(1)=array_fr(j+1) 
           bb(2)=array_fr(j+4) 
           bb(3)=array_fr(j+3) 
           bb(4)=array_fr(j+2) 
           j=j2+i4 
           dd(1)=array_fr(j+1) 
           dd(2)=array_fr(j+4) 
           dd(3)=array_fr(j+3) 
           dd(4)=array_fr(j+2) 
                                                                        
           call matprof(bb,dd,aa) 
           aa(1)=aa(1)-R_oo 
           aa(4)=aa(4)-R_oo 
                                                                        
           j=j3+i4 
           bb(1)=conjg(array_fr(j+1)) 
           bb(2)=conjg(array_fr(j+3)) 
           bb(3)=conjg(array_fr(j+4)) 
           bb(4)=conjg(array_fr(j+2)) 
                                                                        
           j=j4+i4 
           dd(1)=conjg(array_fr(j+1)) 
           dd(2)=conjg(array_fr(j+3)) 
           dd(3)=conjg(array_fr(j+4)) 
           dd(4)=conjg(array_fr(j+2)) 
                                                                        
           call matprof(dd,bb,cc) 
           cc(1)=cc(1)-conjg(R_oo) 
           cc(4)=cc(4)-conjg(R_oo) 
                                                                        
           call tracematpro4(Arho,aa,rho,cc,traceh) 
           hcc=hcc+traceh 
          end do ! nch 
          h0=h0+real(hcc*swin)*2.  ! 
          hp=hp+real(hcc*swinp)*2. ! real sum
          hm=hm+real(hcc*swinm)*2. !
         end if   ! abs(swin) >0.001 
        end do    ! n_s 
        if(b_frcn) then 
         yth(k)=h0                      !real sum
        else if(b_frcr) then 
         yth(k)=0.5*(hp+hm)-h0          ! real sum 
        else if(b_frci) then 
         yth(k)=0.5*(hp-hm)             ! real sum
        else if(b_frab) then 
         yth(k)=sqrt(0.5*(hp*hp+hm*hm)+h0*(h0-hp-hm)) ! real sum
        end if
       end do 
                                                                        
      else if(l1p(1)+l1p(2).eq.4) then 
!      write(6,'(a)') 'l1p(1)+l1p(2).eq.4'                           
!      both unpolarized 
       do n_v=-n_vm,n_vm,j_s 
        k=n_v/j_s 
        h0=0. ! 
        hp=0. ! real summation
        hm=0. !
        do n_s=0,nstrob_order               ! real loop for summation
         nomega=int(float(n_s)*omega/2.)

         swin=array_fr(l1_get(0)+1+abs(n_s)) 
         if(n_s.lt.0) swin=conjg(swin) 
         if(b_window)then
          swinp=swin*exp(-hci*2.*pi*n_s*eta) 
          swinm=swin*exp(hci*2.*pi*n_s*eta) 
         else
          swinp=array_fr(l1_get(0)+2+nstrob_order+abs(n_s)) 
          swinm=array_fr(l1_get(0)+3+2*nstrob_order+abs(n_s)) 
         end if

         h1=real(conjg(swin)*swin) 
         if(h1.gt.1.e-6)then 
!         integration of Int;(-n_ch,n_ch)                               
!          n_ch=n_er+n_vm+nomega                                  
                                                                        
          j1=l1_pos(1)+(nCC+n_v+nomega) 
          j2=l1_pos(2)+(nCC+nomega) 
                                                                        
          j3=l1_pos(1)+(nCC+n_v-nomega) 
          j4=l1_pos(2)+(nCC-nomega) 
                                                                        
          n_ch=n_er+n_vm+nomega
          hcc=hc0 
          do i=-n_ch,n_ch 
           j=j1+i 
           bb(1)=array_fr(j+1) 
           j=j2+i 
           dd(1)=array_fr(j+1) 
           aa(1)=bb(1)*dd(1)-R_oo 
           j=j3+i 
           bb(1)=conjg(array_fr(j+1)) 
           j=j4+i 
           dd(1)=conjg(array_fr(j+1)) 
           cc(1)=dd(1)*bb(1)-conjg(R_oo) 
           hcc=hcc+cc(1)*aa(1) 
          end do  ! nch
          h0=h0+real(hcc*swin)*2.  ! 
          hp=hp+real(hcc*swinp)*2. ! real sum
          hm=hm+real(hcc*swinm)*2. !
         end if   ! abs(swin) >0.001  
        end do    ! n_s 
        if(b_frcn) then 
         yth(k)=h0        !real sum
        else if(b_frcr) then 
         yth(k)=0.5*(hp+hm)-h0          ! real sum 
        else if(b_frci) then 
         yth(k)=0.5*(hp-hm)    ! real sum
        else if(b_frab) then 
         yth(k)=sqrt(0.5*(hp*hp+hm*hm)+h0*(h0-hp-hm)) ! real sum
        end if
       end do 
!      write(6,'(6e15.6)') (yth(i),i=-nTH,nTH,16)

      else if(l1p(1)+l1p(2).eq.10) then 
!      write(6,'(a)') 'l1p(1)+l1p(2).eq.10' 
       do n_v=-n_vm,n_vm,j_s 
        k=n_v/j_s 
        h0=0. ! 
        hp=0. ! real summation
        hm=0. !
        do n_s=0,nstrob_order                  ! real loop for summation
         nomega=int(float(n_s)*omega/2.)

         swin=array_fr(l1_get(0)+1+abs(n_s)) 

         if(n_s.lt.0) swin=conjg(swin) 
         if(b_window)then
          swinp=swin*exp(-hci*2.*pi*n_s*eta) 
          swinm=swin*exp(hci*2.*pi*n_s*eta) 
         else
          swinp=array_fr(l1_get(0)+2+nstrob_order+abs(n_s)) 
          swinm=array_fr(l1_get(0)+3+2*nstrob_order+abs(n_s)) 
         end if
         h1=real(conjg(swin)*swin) 
         if(h1.gt.1.e-6)then 

          n_ch=n_er+n_vm+nomega
          if(l1p(1).eq.2)then 
           j1=l1_pos(1)+(nCC+n_v+nomega) 
           j2=l1_pos(2)+4*(nCC+nomega) 
           j3=l1_pos(1)+(nCC+n_v-nomega) 
           j4=l1_pos(2)+4*(nCC-nomega) 
           hcc=hc0 
           do i=-n_ch,n_ch 
            i4=4*i 
            j=j2+i4 
            ji=j1+i+1 
            aa(1)=array_fr(j+1)*array_fr(ji)-R_oo 
            aa(2)=array_fr(j+4)*array_fr(ji) 
            aa(3)=array_fr(j+3)*array_fr(ji) 
            aa(4)=array_fr(j+2)*array_fr(ji)-R_oo 
            j=j4+i4 
            ji=j3+i+1 
            cc(1)=conjg(array_fr(j+1)*array_fr(ji)-R_oo) 
            cc(2)=conjg(array_fr(j+3)*array_fr(ji)) 
            cc(3)=conjg(array_fr(j+4)*array_fr(ji)) 
            cc(4)=conjg(array_fr(j+2)*array_fr(ji)-R_oo) 
                                                                        
            call tracematpro4(Arho,aa,rho,cc,traceh) 
            hcc=hcc+traceh 
           end do  ! nch    
          else 
           j1=l1_pos(1)+4*(nCC+n_v+nomega) 
           j2=l1_pos(2)+(nCC+nomega) 
           j3=l1_pos(1)+4*(nCC+n_v-nomega) 
           j4=l1_pos(2)+(nCC-nomega) 
           do i=-n_ch,n_ch 
            i4=4*i 
            j =j1+i4 
            ji=j2+i+1 
            aa(1)=array_fr(j+1)*array_fr(ji)-R_oo 
            aa(2)=array_fr(j+4)*array_fr(ji) 
            aa(3)=array_fr(j+3)*array_fr(ji) 
            aa(4)=array_fr(j+2)*array_fr(ji)-R_oo 
                                                                        
            j =j3+i4 
            ji=j4+i+1 
            cc(1)=conjg(array_fr(ji)*array_fr(j+1)-R_oo) 
            cc(2)=conjg(array_fr(ji)*array_fr(j+3)) 
            cc(3)=conjg(array_fr(ji)*array_fr(j+4)) 
            cc(4)=conjg(array_fr(ji)*array_fr(j+2)-R_oo) 
                                                                        
            call tracematpro4(Arho,aa,rho,cc,traceh) 
            hcc=hcc+traceh 
           end do  ! nch 
          end if 
          h0=h0+real(hcc*swin)*2.  ! 
          hp=hp+real(hcc*swinp)*2. ! real sum
          hm=hm+real(hcc*swinm)*2. !
         end if   ! abs(swin) >0.0001  
        end do    ! n_s
        if(b_frcn) then 
         yth(k)=h0                 !real sum  conventinal stroboscopy
        else if(b_frcr) then 
         yth(k)=0.5*(hp+hm)-h0     ! real sum 
        else if(b_frci) then 
         yth(k)=0.5*(hp-hm)        ! real sum
        else if(b_frab) then 
         yth(k)=sqrt(0.5*(hp*hp+hm*hm)+h0*(h0-hp-hm)) ! real sum
        end if                        
       end do 

!      write(6,'(6f12.5)')(yth(i),i=0,nTH,4)    
      end if 
            
      h1=0.
      do j=-nTH,nTH 
       h1=h1+yth(j)
      end do
      h1=h1/(2*nTH+1) 

      j1=nstorTH+1 
      do i=0,nTH 
       array_fr(j1+i)=cmplx(yth(i)-h1,yth(-i)-h1) 
      end do
      
      if(enwr(15).eq.'ampl') print'(a,f12.2,a,f12.4)','average yth =',h1,' &
                             base/amplitude=',base/amplitude
      deallocate(yth, carray)
      call cpu_time(finish) 
      if(enwr(16).eq.'time')                                            &
          print '(" runtime = ",f15.3," seconds.")',finish-start        
      return 
      END                                           


      subroutine drive_strobo(mode_drive,ibfit,nchansp,mvpt,   &
                       mxpt,b_data,channels_vmax,channel_v0,v_max, &
                       vcos1,phase1,vcos2,phase2,vcos3,phase3,info)
      use glbconst, only: infotext,enwr
      use comthe, only: real_fr 
      implicit none
      logical, intent(in):: b_data
      integer,intent(in):: mode_drive,ibfit,nchansp,mxpt,mvpt 
      real,   intent(in):: channels_vmax,channel_v0,v_max,             &
                           vcos1,phase1,vcos2,phase2,vcos3,phase3
      real           arg,c1,c4,c1p,c2p,c3p,                   &
                     fold_point,period,period2,&
                     s,vh,vmax_chan,x,x1,x2,x3,y,dv0,chv0per,       &
                     v1,v2,v3,ph1,ph2,ph3,piqp
      integer,intent(out):: info
      integer        ii,mode_case,j
      logical        b_nonlinear
!     calculation  of velocity                             
!     nchen mode = +1   triangular wave mode                            
!     nchen mode = +2   sine wave mode 

      period=channels_vmax*2.0 

      piqp=3.14159/period; c1p=piqp; c2p=2.0*c1p; c3p=3.0*c1p
      fold_point=channel_v0+channels_vmax 
      b_nonlinear=abs(vcos1)+abs(vcos2)+abs(vcos3).gt.1.e-5

    if(b_nonlinear)then   
     if(abs(phase1)+abs(phase2)+abs(phase3).lt.2.)then
      call sin2phase(phase1,vcos1,v1,ph1); ph1=ph1/c1p 
      call sin2phase(phase2,vcos2,v2,ph2); ph2=ph2/c2p 
      call sin2phase(phase3,vcos3,v3,ph3); ph3=ph3/c3p
      if(enwr(9).eq.'nlco')print'(a8,f12.5)','vcos1=',v1,'phase1=',ph1,  &
                                             'vcos2=',v2,'phase1=',ph2,  &
                                             'vcos3=',v3,'phase3=',ph3
    
     else
      v1=vcos1; v2=vcos2; v3=vcos3; ph1=phase1; ph2=phase2; ph3=phase3; 
      if(enwr(9).eq.'nlco') print '(a8,f12.5)',              &
         'vcos1=',v1*cos(c1p*ph1),'vsin1=',v1*sin(c1p*ph1),  &
         'vcos2=',v2*cos(c2p*ph2),'vsin2=',v2*sin(c2p*ph2),  &
         'vcos3=',v3*cos(c3p*ph3),'vsin3=',v3*sin(c3p*ph3)      
     end if
!    velocity correction vanishes at x=fold_point -> y= 0
     dv0= v1*cos(c1p*ph1)+v2*cos(c2p*ph2)+v3*cos(c3p*ph3)
    end if
    mode_case=abs(mode_drive)
    if(mode_case.gt.3) mode_case= mode_case-3
    if(mode_case.eq.3.and..not.b_data)mode_case=2 

   select case(mode_case)                                                                     
     case(1)
!     triangular wave                                                   
      period2=period*2.0 
      vmax_chan=v_max/channels_vmax 
      chv0per=channel_v0+period 
      do ii=1,nchansp 
       x=float(ii)
       y= x-fold_point
       if(y.gt.0.)goto 30 
!      left side                                                        
       if(y.gt.-period) goto 22 
       x=x+period2  
       goto 32 
   22  c1=channel_v0-x
       s=1. 
       goto 40 
!      right side                                                       
   30  if(y.lt.period) goto 32 
       x=x-period2 
       goto 22 
   32  c1=chv0per-x
       s=-1. 
   40  c4=s*vmax_chan 
       vh=c4*c1 
       if(b_nonlinear)vh=vh+dv0-v1*cos(c1p*(y-ph1))   &
                               -v2*cos(c2p*(y-ph2))-v3*cos(c3p*(y-ph3)) 
!      velocity=vh 
       real_fr(mvpt+ii) = vh
       end do 

    case(2)
!     sine wave  
      j=mvpt-3*nchansp 
      x=piqp/v_max                                                      
      x1=x*v1/c1p; x2=x*v2/c2p; x3=x*v3/c3p
      do ii=1,nchansp 
       y=float(ii)-fold_point 
       arg=piqp*y                ! 2*pi/T*t       
       vh=-v_max*cos(arg) 
       if(b_nonlinear)vh=vh+dv0-v1*cos(c1p*(y-ph1))-v2*cos(c2p*(y-ph2))  & 
                               -v3*cos(c3p*(y-ph3)) 
       real_fr(mvpt+ii) = vh
      end do 
    
    case(3)
!     mode_drive=3
      if(real_fr(mxpt+1).gt.0.0.or.real_fr(mxpt+nchansp).gt.2*abs(v_max))then
       info=nchansp 
       infotext(1)='<Mossbauer_spectrum>/velocity/scale' 
       infotext(2)='**error** x-data do not fit to v_max ' 
       infotext(3)=' ' 
       return
      end if
      do ii=1,nchansp 
       vh=real_fr(mxpt+ii) 
!      velocity=vh 
       real_fr(mvpt+ii) = vh
      end do 
    end select
   return 
   END                                           
   
