 program vector_lab
 real :: a,b,c,beta
 real :: h,k,l,v2_length
 real, dimension(3):: exL,ezL,va,vb,vc,v2,ev2,ot
 real, parameter:: rad=atan(1.)/45., sq2=sqrt(2.)

! monoclinic crystal
  a = 6.2482; b = 12.5979; c = 9.2931
!  c=12.5979 ; b=12.5979;  a=9.2931
!  a=9.2931 ;  b=12.5979;  c= 6.2482
! beta angle between a and c
  beta=106.787
! Unit cell axes va,vb,vc expressed by unit vectors ex, ey, ez
! P21/c: va= a*(cos(90°-beta)*ex+ sin(90-beta)*ey), vb=b*ez, vc= c*ey
 
  va(1)=a*sin(beta*rad); va(2)=a*cos(beta*rad); va(3)=0.
  vb=0.; vb(3)=b
  vc=0.; vc(2)=c

! Miller Indices (h,k,l) are a symbolic vector representation for the
! orientation of an atomic plane in a crystal lattice and are defined as the 
! reciprocals of the fractional intercepts which the plane makes with 
! the crystallographic axes.
  h=-2; k=0.; l=1.

! plane (hkl) is defined by the points Pa=va/h, Pb=vb/k, Pc=vc/l
!  
! k=0 => vector vb is a vector in the plane
! second vector v2(Pa -> Pc) in the plane: Pa + v2 - Pc=0 
!                                v2=Pc - Pa=vc/l - va/h 
  write(6,*)'va/h=',va/h
  write(6,*)'vc/l=',vc/l
  v2=vc/l - va/h
  write(6,*)'in plane vector v2=',v2
  v2_length=sqrt(dot_product(v2,v2))
  write(6,*)'      length of v2=',v2_length 
  ev2=v2/v2_length
  write(6,*)'   unit vector ev2=',ev2

! the plane (hkl) is defined by 2 unit vectors: vb/b=ez and ev2
! cross product ot=ezxev2 is normal to the plane:
 ot(1)=-ev2(2); ot(2)=ev2(1); ot(3)=0.
  write(6,*)'         vector ot=',ot
  write(6,'(a)')' rotation axis ot |_ in the plane ex,ey (a,c)'

  write(6,*)'angle <(ot,ex)=',acos(ot(1))/rad
  write(6,*)'angle <(ot,ey)=',asin(ot(2))/rad

! angle  <(ot,ex) is the required angle PHI of the perpendicular rotation axis 
! PHI+45° and  PHI-45° shall be two axes of the lab-system.
! One is taken as the x and one as the z axis, such that the rotation axes
! has the polar angle theta=45°, phi=0°. The z-axis is the gamma-direction.

write(6,'(a,/)')
PHI=acos(ot(1))/rad
write(6,'(a,f12.4,/)')' PHI=',PHI          !  137.8009 is calculated running
                                           !  the code above

!Craig: PHI=136.995 / 137.161 neueste mail vom 3.10.2012

!The crystal is oriented in the Lab such that vectors exL, ezL and the
!rotation axis are coplanar and perpendicular to the b-axis. Then

 print*,'Two possibilities: The ezL-vector corresponds to  "0°" ' 
    
exL(1)=cos((PHI-45)*rad);exL(2)=sin((PHI-45)*rad);exL(3)=0.
ezL(1)=cos((PHI+45)*rad);ezL(2)=sin((PHI+45)*rad);ezL(3)=0. 

  write(6,'(a,3f8.4,a,3f8.4,a1,/)')'exL=(',exL,')  ezL=(',ezL,')'

 print*,'or other around:The exL-vector corresponds to  "0°" ' 

exL(1)=cos((PHI+45)*rad);exL(2)=sin((PHI+45)*rad);exL(3)=0.
ezL(1)=cos((PHI-45)*rad);ezL(2)=sin((PHI-45)*rad);ezL(3)=0. 

  write(6,'(a,3f8.4,a,3f8.4,a1,/)')'exL=(',exL,')  ezL=(',ezL,')'

 print*,' the Lab system with ezL parallel to the rotation axis' 

exL(1)=cos((PHI-90)*rad);exL(2)=sin((PHI-90)*rad);exL(3)=0.
ezL(1)=cos((PHI)*rad);ezL(2)=sin((PHI)*rad);ezL(3)=0. 

  write(6,'(a,3f8.4,a,3f8.4,a1,/)')'exL=(',exL,')  ezL=(',ezL,')'
!the program euler calculates the Euler angle of the transformation
!from Lab to crystal system.

  stop
  end

